/* 
 * Copyright (C) 2015 Antoine Joux, Vanessa Vitse and Titouan Coladon
 * 
 * This file is part of openf4.
 * 
 * openf4 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openf4 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with openf4.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file check-trivial2-gf2-extension-32bits.cpp
 *  \example check-trivial2-gf2-extension-32bits.cpp
 *  \brief Library check tests.
 *  \ingroup examples
 *  \author Vanessa VITSE, Antoine JOUX, Titouan COLADON
 */

#include <iostream>
#include <string>
#include <vector>
#include <libopenf4.h>

using namespace std;

int main (int argc, char **argv)
{
    cout << "#########################################################" << endl;
    cout << "#         CHECK TRIVIAL2 GF2 EXTENSION 32 BITS          #" << endl;
    cout << "#########################################################" << endl << endl;
    
    // Create polynomial array
    vector<string> polynomialArray;
    
    // Create variable name array
    vector<string> variableName;
    for(int i = 0; i < 5; i++)
    {
        variableName.push_back('x'+to_string(i));
    }
    
    // Fill the polynomial array
    polynomialArray.emplace_back("(h^29 + h^26 + h^24 + h^22 + h^20 + h^19 + h^18 + h^16 + h^13 + h^12 + h^11 + h^8 + h^6 + h^5 + h^3 + h + 1)*x1^2*x2^2 + (h^29 + h^28 + h^19 + h^18 + h^17 + h^16 + h^15 + h^12 + h^7 + h^5 + h^4 + h^2)*x1^2*x4^2 + (h^29 + h^24 + h^21 + h^20 + h^17 + h^10 + h^8 + h^7 + h^4 + h^2)*x0*x1*x2 + (h^26 + h^23 + h^21 + h^20 + h^17 + h^15 + h^14 + h^9 + h^6 + h^5 + h^3 + h^2 + h + 1)*x2*x3*x4 + (h^29 + h^28 + h^27 + h^25 + h^24 + h^23 + h^18 + h^17 + h^16 + h^15 + h^13 + h^12 + h^11 + h^8 + h^6 + h^5 + h^3 + h^2)*x3");
    polynomialArray.emplace_back("(h^28 + h^24 + h^20 + h^18 + h^17 + h^16 + h^15 + h^13 + h^11 + h^10 + h^9 + h^4 + h^3 + h^2 + 1)*x2^3*x4 + (h^27 + h^26 + h^24 + h^21 + h^19 + h^17 + h^16 + h^15 + h^14 + h^13 + h^12 + h^11 + h^10 + h^9 + h^8 + h^6 + h^5 + h^4 + h^3 + h^2 + h + 1)*x0*x1*x3 + (h^29 + h^23 + h^20 + h^19 + h^17 + h^11 + h^10 + h^8 + h^5 + h^4 + h^3 + h^2 + 1)*x2*x3^2 + (h^29 + h^24 + h^23 + h^20 + h^19 + h^17 + h^14 + h^13 + h^11 + h^10 + h^9 + h^8 + h^2 + h)*x2*x4^2 + (h^26 + h^25 + h^23 + h^22 + h^21 + h^19 + h^16 + h^14 + h^11 + h^10 + h^9 + h^8 + h^6 + h^5 + h^3 + h^2 + h + 1)*x4");
    polynomialArray.emplace_back("(h^27 + h^25 + h^22 + h^21 + h^19 + h^16 + h^15 + h^13 + h^11 + h^10 + h^8 + h^7 + h^4 + h^2 + h + 1)*x0^2*x2^2 + (h^28 + h^21 + h^16 + h^11 + h^10 + h^5 + h^2 + 1)*x0*x1*x2*x3 + (h^29 + h^26 + h^22 + h^20 + h^19 + h^14 + h^13 + h^11 + h^6 + h^5 + h)*x2*x4^3 + (h^27 + h^26 + h^25 + h^24 + h^22 + h^21 + h^18 + h^17 + h^15 + h^13 + h^12 + h^10 + h^9 + h^8 + h^7 + h^6 + h^5 + h^4 + h^3 + h)*x0*x1^2 + (h^29 + h^28 + h^26 + h^25 + h^23 + h^21 + h^15 + h^13 + h^12 + h^11 + h^9 + h^8 + h^6 + h^4 + h^2)*x0*x3*x4");
    polynomialArray.emplace_back("(h^29 + h^28 + h^23 + h^20 + h^19 + h^18 + h^17 + h^16 + h^15 + h^14 + h^13 + h^12 + h^8 + h^3 + h)*x1^3*x3 + (h^29 + h^26 + h^25 + h^22 + h^21 + h^18 + h^17 + h^16 + h^11 + h^10 + h^7 + h^6 + h^5 + h^3)*x0*x1*x2*x4 + (h^29 + h^28 + h^27 + h^25 + h^24 + h^23 + h^22 + h^20 + h^19 + h^18 + h^17 + h^15 + h^14 + h^11 + h^10 + h^9 + h^8 + h^7 + h^3 + h + 1)*x1*x3*x4^2 + (h^27 + h^26 + h^25 + h^22 + h^21 + h^20 + h^17 + h^16 + h^15 + h^14 + h^12 + h^11 + h^9 + h^8 + h^7 + h^3 + h)*x3*x4^3 + (h^28 + h^27 + h^22 + h^21 + h^16 + h^14 + h^12 + h^11 + h^8 + h^7 + h^4 + h^2 + h)*x0*x1^2");
    polynomialArray.emplace_back("(h^29 + h^27 + h^26 + h^25 + h^24 + h^22 + h^20 + h^18 + h^15 + h^13 + 1)*x1^2*x3*x4 + (h^25 + h^23 + h^22 + h^19 + h^17 + h^16 + h^13 + h^7 + h^4 + h^2)*x1*x2*x3*x4 + (h^25 + h^24 + h^20 + h^18 + h^16 + h^15 + h^14 + h^13 + h^11 + h^10 + h^9 + h^3 + 1)*x0*x3*x4^2 + (h^27 + h^23 + h^19 + h^16 + h^15 + h^13 + h^11 + h^9 + h^7 + h^6 + h^5 + h^4 + h^3 + h)*x1^2*x2 + (h^29 + h^26 + h^24 + h^21 + h^18 + h^11 + h^8 + h^5 + h^2 + 1)*x0^2*x3");
    polynomialArray.emplace_back("(h^27 + h^25 + h^23 + h^22 + h^20 + h^19 + h^16 + h^15 + h^13 + h^11 + h^9 + h^5 + h^4 + h^3 + h + 1)*x0*x1^2*x3 + (h^29 + h^28 + h^27 + h^22 + h^21 + h^19 + h^17 + h^16 + h^15 + h^14 + h^12 + h^10 + h^6 + h^3 + h^2 + h + 1)*x0^2*x2*x3 + (h^29 + h^28 + h^26 + h^24 + h^22 + h^20 + h^18 + h^16 + h^15 + h^14 + h^10 + h^9 + h^8 + h^6 + h^5 + h^3 + h)*x1*x3^2 + (h^28 + h^25 + h^23 + h^21 + h^20 + h^18 + h^16 + h^15 + h^13 + h^12 + h^11 + h^10 + h^9 + h^8 + h^7 + h^6 + h^5 + h^4 + 1)*x1^2*x4 + (h^29 + h^28 + h^26 + h^25 + h^24 + h^23 + h^21 + h^20 + h^18 + h^17 + h^16 + h^14 + h^13 + h^11 + h^8 + h^7 + h^4 + h^2 + h + 1)*x3*x4^2");
    polynomialArray.emplace_back("(h^29 + h^28 + h^26 + h^25 + h^24 + h^19 + h^18 + h^17 + h^16 + h^15 + h^14 + h^12 + h^10 + h^9 + h^7 + h^6 + h^2 + 1)*x0*x2*x3^2 + (h^29 + h^26 + h^25 + h^24 + h^21 + h^20 + h^18 + h^15 + h^14 + h^10 + h^8 + 1)*x2*x3^2*x4 + (h^29 + h^28 + h^27 + h^25 + h^24 + h^21 + h^20 + h^19 + h^16 + h^10 + h^8 + h^7 + h^6 + h^5 + h^4 + h^3 + h^2)*x0*x2*x4^2 + (h^29 + h^27 + h^25 + h^24 + h^23 + h^20 + h^17 + h^14 + h^13 + h^11 + h^10 + h^8 + h^5 + h^4 + h^2)*x1*x2*x4 + (h^29 + h^28 + h^27 + h^26 + h^25 + h^24 + h^22 + h^21 + h^19 + h^18 + h^10 + h^7 + h^5 + h^4 + h^3 + h^2 + h + 1)");
    polynomialArray.emplace_back("(h^29 + h^27 + h^25 + h^23 + h^21 + h^20 + h^19 + h^18 + h^15 + h^13 + h^11 + h^10 + h^9 + h^8 + h^7 + h^6 + h^5 + h^4 + h^3 + 1)*x0^3*x3 + (h^28 + h^26 + h^25 + h^22 + h^21 + h^19 + h^18 + h^17 + h^16 + h^14 + h^10 + h + 1)*x1^3*x3 + (h^28 + h^26 + h^25 + h^23 + h^22 + h^20 + h^19 + h^16 + h^15 + h^13 + h^10 + h^9 + h^5 + h^4 + h^3 + h^2)*x0^2*x1*x4 + (h^28 + h^26 + h^24 + h^22 + h^21 + h^20 + h^15 + h^14 + h^12 + h^10 + h^5 + h^3 + 1)*x1^2*x2*x4 + (h^28 + h^27 + h^25 + h^23 + h^22 + h^21 + h^20 + h^19 + h^18 + h^17 + h^15 + h^14 + h^13 + h^12 + h^10 + h^8 + h^7 + h^6 + h^3 + h^2 + h + 1)*x0*x3*x4");
    polynomialArray.emplace_back("(h^29 + h^28 + h^27 + h^26 + h^23 + h^22 + h^18 + h^15 + h^13 + h^12 + h^11 + h^10 + h^9 + h^7 + h^6 + h^5 + h^4 + h^3 + 1)*x0^2*x1*x4 + (h^28 + h^25 + h^24 + h^23 + h^21 + h^20 + h^18 + h^16 + h^14 + h^13 + h^10 + h^9 + h^6 + h^2 + h)*x0^2*x1 + (h^28 + h^26 + h^24 + h^19 + h^17 + h^11 + h^8 + h^7 + h^4)*x0*x4^2 + (h^27 + h^25 + h^22 + h^20 + h^19 + h^16 + h^14 + h^13 + h^8 + h^7 + h^6 + h^5 + h^4 + h + 1)*x1*x3 + (h^28 + h^27 + h^25 + h^24 + h^23 + h^22 + h^21 + h^19 + h^17 + h^16 + h^12 + h^11 + h^8 + h^7 + h^5 + h^3 + h^2)*x1*x4");
    polynomialArray.emplace_back("(h^29 + h^27 + h^25 + h^24 + h^22 + h^19 + h^17 + h^14 + h^13 + h^12 + h^11 + h^10 + h^9 + h^8 + h^7 + h^5 + h^4 + h^3 + h)*x0*x2*x3^2 + (h^29 + h^26 + h^24 + h^23 + h^22 + h^21 + h^20 + h^18 + h^17 + h^16 + h^15 + h^14 + h^13 + h^9 + h^7 + h^6 + h^3 + h)*x1*x2*x3^2 + (h^28 + h^26 + h^24 + h^23 + h^19 + h^17 + h^15 + h^14 + h^13 + h^12 + h^11 + h^8 + h^7 + h^6 + h^5 + h^4 + h^2 + 1)*x3^3*x4 + (h^26 + h^24 + h^23 + h^19 + h^14 + h^12 + h^8 + h^7 + h^6 + h^5 + h^3 + h + 1)*x1*x3*x4^2 + (h^29 + h^27 + h^25 + h^20 + h^18 + h^16 + h^15 + h^13 + h^12 + h^7 + h^5 + h^4 + h + 1)*x3^3");
    polynomialArray.emplace_back("(h^28 + h^25 + h^23 + h^19 + h^18 + h^17 + h^14 + h^13 + h^12 + h^11 + h^10 + h^7 + h^5 + h^3 + h^2 + 1)*x1*x2^3 + (h^28 + h^27 + h^25 + h^24 + h^22 + h^21 + h^20 + h^19 + h^18 + h^16 + h^15 + h^14 + h^13 + h^12 + h^11 + h^7 + h^6 + h^4 + h^3 + h^2)*x0*x2^2*x3 + (h^27 + h^23 + h^22 + h^20 + h^18 + h^16 + h^15 + h^9 + h^6 + h^5 + h^4 + h^3 + h + 1)*x0*x2^2*x4 + (h^29 + h^27 + h^23 + h^22 + h^21 + h^18 + h^17 + h^16 + h^14 + h^13 + h^12 + h^8 + h^6 + h^3 + h^2 + h)*x0*x1*x4^2 + (h^28 + h^25 + h^24 + h^22 + h^21 + h^19 + h^18 + h^17 + h^16 + h^14 + h^12 + h^11 + h^10 + h^9 + h^8 + h^7 + h^6 + h^5 + h^4 + h^3 + h + 1)*x2^2");
    polynomialArray.emplace_back("(h^28 + h^27 + h^26 + h^25 + h^23 + h^22 + h^21 + h^20 + h^17 + h^16 + h^14 + h^13 + h^11 + h^9 + h^5 + h^4 + h^3 + 1)*x0^2*x1*x2 + (h^29 + h^28 + h^26 + h^18 + h^17 + h^16 + h^15 + h^13 + h^11 + h^9 + h^6 + h^5 + h^4 + h^3 + 1)*x0*x1^2*x2 + (h^29 + h^28 + h^25 + h^22 + h^17 + h^15 + h^8 + h^4 + h^3 + h + 1)*x0*x3^3 + (h^29 + h^27 + h^25 + h^23 + h^22 + h^21 + h^19 + h^18 + h^17 + h^15 + h^14 + h^12 + h^9 + h^8 + h^5 + h^3 + 1)*x1^2*x4^2 + (h^29 + h^25 + h^24 + h^22 + h^20 + h^19 + h^18 + h^17 + h^16 + h^14 + h^12 + h^8 + h^6 + h^5 + h^3)*x1^2*x2");
    polynomialArray.emplace_back("(h^28 + h^25 + h^21 + h^20 + h^18 + h^15 + h^14 + h^10 + h^9 + h^2 + h + 1)*x0^3*x1 + (h^29 + h^27 + h^22 + h^20 + h^18 + h^14 + h^13 + h^11 + h^10 + h^8 + h^6 + h^5 + h^4 + h^3 + h^2 + h)*x1^2*x2^2 + (h^29 + h^25 + h^22 + h^19 + h^17 + h^16 + h^13 + h^12 + h^9 + h^6 + h^5 + h^4 + h^3 + h^2 + h)*x1*x2^2*x3 + (h^29 + h^26 + h^25 + h^21 + h^20 + h^19 + h^18 + h^16 + h^14 + h^12 + h^10 + h^9 + h + 1)*x2*x4^2 + (h^29 + h^27 + h^24 + h^23 + h^22 + h^21 + h^19 + h^10 + h^8 + h^7 + h^6 + h^4 + h^3 + h^2 + h)*x4^2");
    polynomialArray.emplace_back("(h^29 + h^27 + h^26 + h^25 + h^24 + h^23 + h^22 + h^17 + h^15 + h^10 + h^9 + h^8 + h^6 + h^5 + h + 1)*x0*x1*x2*x3 + (h^29 + h^28 + h^27 + h^26 + h^25 + h^21 + h^20 + h^19 + h^18 + h^17 + h^11 + h^8 + h^5 + h^2 + 1)*x1^2*x3^2 + (h^29 + h^28 + h^27 + h^26 + h^25 + h^22 + h^21 + h^20 + h^16 + h^15 + h^14 + h^8 + h^7 + h^6 + h + 1)*x3^3*x4 + (h^28 + h^21 + h^19 + h^17 + h^14 + h^13 + h^12 + h^11 + h^10 + h^8 + h^6 + h^2 + h)*x0*x4^2 + (h^28 + h^26 + h^22 + h^18 + h^16 + h^15 + h^14 + h^13 + h^12 + h^11 + h^10 + h^8 + h^6 + h^5 + h^4 + h^3 + h^2 + h)*x1*x3");
    polynomialArray.emplace_back("(h^26 + h^23 + h^22 + h^19 + h^18 + h^17 + h^13 + h^11 + h^9 + h^6 + h^4 + h^3 + h^2)*x0*x2^2*x3 + (h^29 + h^28 + h^26 + h^20 + h^19 + h^17 + h^16 + h^15 + h^14 + h^13 + h^12 + h^11 + h^8 + h^6 + h^2 + h)*x2*x3^2*x4 + (h^29 + h^26 + h^24 + h^23 + h^22 + h^19 + h^17 + h^16 + h^15 + h^14 + h^12 + h^11 + h^8 + h^7 + h^6 + h^5 + h^4 + h^2 + h + 1)*x0*x3*x4^2 + (h^29 + h^26 + h^24 + h^20 + h^19 + h^18 + h^10 + h^5 + h^4 + h^2 + 1)*x1*x2^2 + (h^29 + h^24 + h^22 + h^19 + h^18 + h^16 + h^12 + h^7 + h^4 + h^2)*x4^3");
    polynomialArray.emplace_back("(h^29 + h^24 + h^21 + h^19 + h^17 + h^10 + h^9 + h^8 + h^6 + h^5 + 1)*x0^3*x1 + (h^29 + h^26 + h^25 + h^22 + h^16 + h^11 + h^10 + h^8 + h^7 + h^5 + h^3 + h^2 + h + 1)*x1*x3^3 + (h^29 + h^27 + h^25 + h^24 + h^23 + h^22 + h^21 + h^20 + h^19 + h^18 + h^17 + h^15 + h^14 + h^12 + h^7 + h^6 + h^5 + h^3 + 1)*x1*x3*x4^2 + (h^29 + h^28 + h^26 + h^25 + h^21 + h^20 + h^19 + h^15 + h^10 + h^9 + h^4 + 1)*x0^2*x4 + (h^28 + h^25 + h^24 + h^22 + h^21 + h^20 + h^18 + h^14 + h^13 + h^11 + h^8 + h^6 + h^4 + h^2 + h + 1)*x3*x4^2");
    polynomialArray.emplace_back("(h^29 + h^28 + h^26 + h^23 + h^22 + h^21 + h^20 + h^19 + h^16 + h^15 + h^13 + h^12 + h^11 + h^10 + h^8 + h^7 + h^5 + h^4 + h^3 + h^2)*x0^3*x2 + (h^29 + h^27 + h^23 + h^21 + h^20 + h^19 + h^17 + h^16 + h^13 + h^12 + h^9 + h^8 + h^2 + h + 1)*x2^3*x3 + (h^29 + h^26 + h^22 + h^21 + h^20 + h^17 + h^14 + h^9 + h^8 + h^7 + 1)*x2^2*x4^2 + (h^28 + h^27 + h^25 + h^24 + h^23 + h^20 + h^18 + h^17 + h^16 + h^15 + h^14 + h^12 + h^11 + h^10 + h^9 + h^8 + h^7 + h^6 + h^5 + h^4 + h^2)*x2*x3*x4 + (h^23 + h^21 + h^18 + h^17 + h^10 + h^9 + h^8 + h^7 + h^6 + h^5 + h^3 + h^2 + 1)*x0*x3");
    polynomialArray.emplace_back("(h^29 + h^28 + h^24 + h^22 + h^17 + h^12 + h^10 + h^9 + h^8 + h^7 + h^6 + h^3 + h)*x0^3*x2 + (h^27 + h^25 + h^24 + h^23 + h^19 + h^18 + h^17 + h^16 + h^12 + h^10 + h^7 + h^5 + h^2 + 1)*x0^2*x2*x4 + (h^27 + h^26 + h^24 + h^23 + h^22 + h^21 + h^16 + h^14 + h^11 + h^10 + h^8 + h^2 + 1)*x0*x1*x4^2 + (h^29 + h^27 + h^26 + h^24 + h^23 + h^20 + h^19 + h^18 + h^14 + h^10 + h^9 + h^8 + h^7 + h^5 + h^4 + 1)*x0*x3*x4 + (h^25 + h^23 + h^22 + h^20 + h^18 + h^17 + h^15 + h^13 + h^10 + h^9 + h^8 + h^7 + h^6 + h^4 + h^3 + h + 1)*x4");
    polynomialArray.emplace_back("(h^29 + h^28 + h^27 + h^26 + h^25 + h^22 + h^21 + h^14 + h^11 + h^9 + h^8 + h^2 + h)*x0^2*x1^2 + (h^29 + h^28 + h^26 + h^25 + h^24 + h^21 + h^20 + h^19 + h^17 + h^16 + h^15 + h^14 + h^13 + h^10 + h^7 + h^5 + h^4 + h^3 + 1)*x0*x2^3 + (h^27 + h^25 + h^24 + h^23 + h^19 + h^18 + h^14 + h^13 + h^11 + h^10 + h^5 + h^4 + 1)*x1^2*x2*x3 + (h^28 + h^27 + h^25 + h^24 + h^21 + h^19 + h^18 + h^17 + h^14 + h^13 + h^8 + h^6 + h^5 + h^4 + h^2 + h)*x0^2*x3^2 + (h^28 + h^26 + h^22 + h^20 + h^17 + h^14 + h^13 + h^11 + h^9 + h^8 + h^7 + h^6 + h^5 + h^4 + h^3 + h^2 + h + 1)*x3^4");
    polynomialArray.emplace_back("(h^27 + h^26 + h^25 + h^24 + h^22 + h^19 + h^15 + h^11 + h^7 + h^6 + h^5 + h^4 + h^3 + h^2)*x1^3*x3 + (h^29 + h^27 + h^26 + h^25 + h^22 + h^20 + h^18 + h^14 + h^12 + h^10 + h^8 + h^7 + h^3 + h)*x0*x2^2*x3 + (h^23 + h^20 + h^18 + h^16 + h^15 + h^14 + h^13 + h^6 + h^4 + h^3 + h^2 + h + 1)*x0^2*x2 + (h^29 + h^28 + h^27 + h^23 + h^20 + h^19 + h^18 + h^15 + h^14 + h^13 + h^12 + h^9 + h^8 + h^7 + h^6 + h^5 + h^3 + 1)*x2*x4^2 + (h^29 + h^27 + h^25 + h^24 + h^19 + h^17 + h^14 + h^11 + h^10 + h^9 + h^8 + h^7 + h^6 + h^4 + h^2 + h)*x4");
    polynomialArray.emplace_back("(h^25 + h^23 + h^22 + h^20 + h^18 + h^17 + h^13 + h^12 + h^7 + h^6 + h^5 + h^4 + 1)*x0^2*x1*x3 + (h^29 + h^28 + h^27 + h^26 + h^23 + h^20 + h^14 + h^12 + h^9 + h^8 + h^7 + h + 1)*x1^2*x2*x3 + (h^28 + h^27 + h^26 + h^25 + h^24 + h^20 + h^19 + h^18 + h^15 + h^14 + h^13 + h^11 + h^9 + h^6 + h^5 + h^4)*x0^2*x2*x4 + (h^28 + h^26 + h^25 + h^23 + h^20 + h^19 + h^17 + h^16 + h^15 + h^14 + h^13 + h^9 + h^6 + h^5 + h^3 + h)*x1^2*x3*x4 + (h^26 + h^23 + h^22 + h^21 + h^16 + h^13 + h^12 + h^10 + h^5 + h^4 + h^2)*x3*x4^3");
    polynomialArray.emplace_back("(h^28 + h^27 + h^26 + h^25 + h^24 + h^21 + h^20 + h^17 + h^14 + h^12 + h^8 + h^7 + h^5 + h^2 + h)*x1^4 + (h^28 + h^27 + h^26 + h^25 + h^24 + h^23 + h^21 + h^20 + h^19 + h^18 + h^17 + h^16 + h^14 + h^13 + h^12 + h^8 + h^7 + h^6 + h^3 + h^2 + h)*x1^3*x2 + (h^29 + h^25 + h^21 + h^19 + h^18 + h^14 + h^11 + h^10 + h^9 + h^8 + h^6 + h^3 + h + 1)*x1^2*x2*x3 + (h^28 + h^26 + h^25 + h^24 + h^23 + h^18 + h^17 + h^14 + h^2 + h)*x1*x2 + (h^28 + h^27 + h^24 + h^21 + h^19 + h^14 + h^13 + h^7 + h^5 + h + 1)*x3");
    polynomialArray.emplace_back("(h^28 + h^27 + h^25 + h^24 + h^22 + h^18 + h^16 + h^14 + h^13 + h^11 + h^8 + h^6 + h^5 + h^4)*x3^2*x4^2 + (h^27 + h^26 + h^24 + h^23 + h^15 + h^14 + h^13 + h^12 + h^8 + h^7 + h^6 + h^5 + h^4 + h^2)*x0*x4^3 + (h^29 + h^28 + h^27 + h^24 + h^22 + h^21 + h^20 + h^19 + h^18 + h^13 + h^12 + h^11 + h^8 + h^7 + h^6 + h^5 + h^4 + h)*x0*x2^2 + (h^27 + h^24 + h^21 + h^16 + h^15 + h^14 + h^12 + h^11 + h^10 + h^9 + h^8 + h^6 + h^5 + h^4 + h + 1)*x3*x4^2 + (h^29 + h^26 + h^23 + h^21 + h^19 + h^18 + h^17 + h^16 + h^15 + h^14 + h^13 + h^11 + h^10 + h^9 + h^7 + h^6 + h^2)*x4^2");
    polynomialArray.emplace_back("(h^26 + h^24 + h^23 + h^22 + h^21 + h^19 + h^18 + h^17 + h^15 + h^11 + h^9 + h^7 + h^5 + h^4 + h^3 + h^2)*x0^2*x2*x3 + (h^28 + h^25 + h^23 + h^22 + h^20 + h^19 + h^17 + h^16 + h^15 + h^14 + h^12 + h^11 + h^10 + h^6 + h^4 + h^3)*x3^3*x4 + (h^28 + h^26 + h^25 + h^23 + h^20 + h^17 + h^16 + h^15 + h^13 + h^12 + h^10 + h^9 + h^7 + h^6 + h^5 + h^2 + 1)*x1*x2*x4^2 + (h^26 + h^25 + h^24 + h^19 + h^18 + h^12 + h^11 + h^9 + h^8 + h^7 + h^3)*x3^2*x4^2 + (h^29 + h^28 + h^26 + h^24 + h^21 + h^19 + h^18 + h^16 + h^10 + h^8 + h^5 + h^4 + h^3 + h + 1)*x3^3");
    polynomialArray.emplace_back("(h^28 + h^27 + h^26 + h^25 + h^21 + h^18 + h^17 + h^16 + h^15 + h^14 + h^12 + h^11 + h^10 + h^5 + h^4 + h^2 + 1)*x0^2*x2^2 + (h^29 + h^28 + h^26 + h^25 + h^24 + h^23 + h^21 + h^19 + h^18 + h^17 + h^16 + h^15 + h^14 + h^13 + h^12 + h^11 + h^10 + h^9 + h^7 + h^6 + h^4 + h^3 + h^2)*x1^2*x2^2 + (h^26 + h^21 + h^19 + h^16 + h^15 + h^13 + h^5 + h^4 + h^3 + h^2 + h + 1)*x1^2*x4^2 + (h^27 + h^25 + h^23 + h^21 + h^20 + h^19 + h^18 + h^16 + h^13 + h^6 + h^5 + h^3 + h)*x2^3 + (h^29 + h^27 + h^25 + h^24 + h^23 + h^22 + h^18 + h^14 + h^9 + h^8 + h^6)*x0*x2*x3");
    polynomialArray.emplace_back("(h^29 + h^26 + h^25 + h^24 + h^22 + h^21 + h^19 + h^18 + h^17 + h^16 + h^13 + h^10 + h^8 + h^7 + h^6 + 1)*x0*x1^3 + (h^29 + h^26 + h^22 + h^19 + h^16 + h^15 + h^13 + h^10 + h^9 + h^8 + h^7 + h^4 + h^3 + h^2 + h + 1)*x0^2*x2*x3 + (h^28 + h^26 + h^25 + h^23 + h^19 + h^18 + h^17 + h^16 + h^15 + h^11 + h^10 + h^9 + h^8)*x2^2*x3^2 + (h^29 + h^26 + h^25 + h^23 + h^21 + h^20 + h^17 + h^16 + h^15 + h^14 + h^12 + h^11 + h^10 + h^7 + h^6 + h^4 + h^3 + 1)*x0*x2*x4^2 + (h^29 + h^27 + h^25 + h^24 + h^23 + h^21 + h^19 + h^17 + h^15 + h^13 + h^12 + h^11 + h^9 + h^8 + h^7 + h^4 + h^3)*x0*x2*x4");
    polynomialArray.emplace_back("(h^26 + h^23 + h^21 + h^20 + h^19 + h^18 + h^17 + h^16 + h^15 + h^13 + h^11 + h^10 + h^9 + h^6 + h^4)*x2*x3^3 + (h^29 + h^28 + h^27 + h^23 + h^21 + h^20 + h^18 + h^17 + h^13 + h^11 + h^9 + h^7 + h^6 + h^4 + h^2 + h)*x1*x2*x3*x4 + (h^29 + h^25 + h^24 + h^23 + h^19 + h^18 + h^16 + h^15 + h^14 + h^12 + h^11 + h^8 + h^6 + h^4 + h^3 + h^2 + 1)*x0*x4^3 + (h^28 + h^25 + h^23 + h^22 + h^21 + h^17 + h^16 + h^15 + h^14 + h^12 + h^11 + h^10 + h^9 + h^6 + h^4 + h^3 + 1)*x3^2 + (h^21 + h^20 + h^18 + h^17 + h^15 + h^14 + h^13 + h^11 + h^10 + h^7 + h^6 + h^2 + h)*x3*x4");
    polynomialArray.emplace_back("(h^27 + h^25 + h^24 + h^23 + h^22 + h^20 + h^19 + h^18 + h^16 + h^13 + h^11 + h^8 + h^6 + h^5 + h^4 + 1)*x2*x3^3 + (h^29 + h^28 + h^27 + h^24 + h^23 + h^22 + h^21 + h^20 + h^17 + h^14 + h^13 + h^12 + h^11 + h^9 + h^8 + h^4 + h^3 + h)*x1^2*x4 + (h^29 + h^28 + h^26 + h^21 + h^19 + h^18 + h^17 + h^15 + h^12 + h^11 + h^10 + h^9 + h^6 + h^5 + h + 1)*x2^2 + (h^29 + h^28 + h^24 + h^23 + h^21 + h^20 + h^17 + h^16 + h^14 + h^13 + h^12 + h^10 + h^9 + h^8 + h^7 + h^6 + h^3 + 1)*x0*x4 + (h^29 + h^28 + h^27 + h^26 + h^24 + h^22 + h^19 + h^18 + h^17 + h^15 + h^13 + h^10 + h^6 + h^2 + 1)*x3*x4");
    polynomialArray.emplace_back("(h^29 + h^27 + h^26 + h^25 + h^24 + h^23 + h^20 + h^18 + h^17 + h^13 + h^12 + h^9 + h^8 + h^5 + h^3 + h)*x0*x1^2*x4 + (h^28 + h^27 + h^26 + h^25 + h^24 + h^21 + h^18 + h^16 + h^9 + h^7 + h^4 + h^3 + h + 1)*x1^3 + (h^27 + h^26 + h^22 + h^20 + h^17 + h^16 + h^11 + h^10 + h^2 + h + 1)*x2^3 + (h^27 + h^26 + h^22 + h^20 + h^19 + h^18 + h^17 + h^15 + h^14 + h^13 + h^12 + h^11 + h^8 + h^7 + h^2 + h + 1)*x3*x4^2 + (h^27 + h^22 + h^20 + h^16 + h^15 + h^14 + h^13 + h^11 + h^10 + h^5 + h^3 + h^2 + 1)*x2^2");
    polynomialArray.emplace_back("(h^27 + h^26 + h^25 + h^24 + h^22 + h^19 + h^18 + h^16 + h^15 + h^14 + h^12 + h^10 + h^9 + h^8 + h^4 + h^3 + h + 1)*x0^3*x1 + (h^28 + h^26 + h^24 + h^23 + h^21 + h^20 + h^15 + h^9 + h^7 + h^6 + h^2)*x0^2*x1*x3 + (h^29 + h^27 + h^24 + h^22 + h^21 + h^19 + h^18 + h^17 + h^14 + h^13 + h^12 + h^11 + h^10 + h^9 + h^6 + h^3 + h + 1)*x1*x2*x3*x4 + (h^29 + h^26 + h^24 + h^22 + h^16 + h^13 + h^10 + h^7 + h^5 + h + 1)*x1^2*x3 + (h^27 + h^26 + h^25 + h^22 + h^18 + h^15 + h^10 + h^7 + h^5 + h^4 + h^2 + h)*x2*x3");
    polynomialArray.emplace_back("(h^29 + h^26 + h^25 + h^24 + h^22 + h^19 + h^16 + h^15 + h^14 + h^12 + h^10 + h^9 + h^4 + h + 1)*x0*x1*x2*x3 + (h^27 + h^26 + h^25 + h^24 + h^18 + h^16 + h^15 + h^14 + h^9 + h^8 + h^2 + 1)*x2^3*x3 + (h^28 + h^25 + h^23 + h^21 + h^20 + h^18 + h^16 + h^14 + h^11 + h^10 + h^6 + h^4 + 1)*x1^2*x2*x4 + (h^29 + h^26 + h^23 + h^19 + h^14 + h^10 + h^8 + h)*x1^2*x3 + (h^28 + h^26 + h^25 + h^24 + h^22 + h^20 + h^18 + h^16 + h^14 + h^13 + h^12 + h^11 + h^10 + h^9 + h^8 + h^7 + h^6)*x3");
    polynomialArray.emplace_back("(h^29 + h^28 + h^27 + h^24 + h^23 + h^22 + h^20 + h^19 + h^17 + h^15 + h^12 + h^11 + h^10 + h^9 + h^7 + h^6 + h^3)*x1^2*x2*x3 + (h^28 + h^27 + h^22 + h^19 + h^18 + h^14 + h^13 + h^12 + h^11 + h^8 + h^6 + h^5 + h^4 + h^2 + h + 1)*x0*x4^3 + (h^29 + h^26 + h^25 + h^24 + h^23 + h^22 + h^21 + h^16 + h^14 + h^13 + h^12 + h^10 + h^9 + h^8 + h^6 + h^3 + h + 1)*x1^2 + (h^29 + h^26 + h^23 + h^22 + h^20 + h^19 + h^18 + h^17 + h^16 + h^15 + h^14 + h^13 + h^12 + h^9 + h^8 + h^7 + h^6 + h^5 + h^4 + h^2 + 1)*x0*x2 + (h^29 + h^28 + h^27 + h^26 + h^17 + h^15 + h^10 + h^9 + h^7 + h^4 + h^3 + h^2)*x2*x4");
    polynomialArray.emplace_back("(h^29 + h^26 + h^24 + h^22 + h^19 + h^18 + h^16 + h^11 + h^8 + h^2 + h)*x0^2*x3*x4 + (h^29 + h^27 + h^25 + h^21 + h^18 + h^17 + h^16 + h^15 + h^10 + h^9 + h^8 + h^5)*x3^3*x4 + (h^28 + h^26 + h^25 + h^21 + h^19 + h^18 + h^17 + h^12 + h^11 + h^10 + h^7 + h^6 + h^4 + h^2)*x0^2*x2 + (h^28 + h^27 + h^23 + h^20 + h^19 + h^18 + h^15 + h^13 + h^11 + h^9 + h^8 + h^2 + h)*x0*x1*x2 + (h^28 + h^27 + h^25 + h^23 + h^21 + h^18 + h^17 + h^14 + h^13 + h^12 + h^11 + h^10 + h^9 + h^8 + h^7 + h^6 + h^5 + h^3 + h)*x3*x4");
    polynomialArray.emplace_back("(h^29 + h^28 + h^24 + h^22 + h^21 + h^20 + h^18 + h^17 + h^16 + h^15 + h^9 + h^6 + h^5 + h^4 + h^2 + h)*x0*x1^2*x2 + (h^28 + h^27 + h^24 + h^23 + h^21 + h^20 + h^18 + h^13 + h^11 + h^10 + h^8 + h^7 + h^6 + h^4 + h^2)*x0*x3^2*x4 + (h^28 + h^26 + h^24 + h^23 + h^22 + h^19 + h^18 + h^16 + h^15 + h^14 + h^13 + h^12 + h^10 + h^9 + h^8 + h^7 + h^5 + h^3 + h)*x0^2*x2 + (h^26 + h^25 + h^19 + h^17 + h^16 + h^15 + h^14 + h^11 + h^8 + h^7 + 1)*x2*x3^2 + (h^27 + h^26 + h^25 + h^24 + h^22 + h^19 + h^18 + h^16 + h^13 + h^10 + h^9 + h^7 + h^6 + h^5 + h^4 + h^2 + 1)*x3^2");
    polynomialArray.emplace_back("(h^29 + h^27 + h^26 + h^25 + h^22 + h^21 + h^19 + h^18 + h^16 + h^11 + h^10 + h^7 + h^5 + h^4 + h^3 + 1)*x0^2*x1*x4 + (h^29 + h^27 + h^25 + h^24 + h^21 + h^19 + h^18 + h^17 + h^16 + h^13 + h^8 + h^6 + h^2)*x0^2*x3*x4 + (h^27 + h^26 + h^23 + h^22 + h^21 + h^20 + h^18 + h^17 + h^15 + h^14 + h^13 + h^10 + h^9 + h^7 + h^6 + h^5 + h^4 + 1)*x2*x3^2*x4 + (h^29 + h^28 + h^27 + h^21 + h^18 + h^17 + h^13 + h^12 + h^11 + h^10 + h^9 + h^7 + h^6 + h^5 + h^4 + 1)*x1^2 + (h^29 + h^27 + h^19 + h^16 + h^14 + h^13 + h^11 + h^9)*x2*x4");
    polynomialArray.emplace_back("(h^28 + h^26 + h^25 + h^23 + h^22 + h^21 + h^20 + h^19 + h^18 + h^17 + h^16 + h^13 + h^12 + h^4 + h^3 + h + 1)*x0*x1*x3*x4 + (h^29 + h^28 + h^26 + h^25 + h^24 + h^23 + h^22 + h^19 + h^17 + h^16 + h^15 + h^14 + h^13 + h^12 + h^8 + h^7 + h^5 + h^3 + h + 1)*x1^3 + (h^27 + h^26 + h^25 + h^24 + h^23 + h^21 + h^19 + h^14 + h^10 + h^9 + h^4 + h)*x0*x2*x3 + (h^29 + h^27 + h^23 + h^21 + h^20 + h^17 + h^15 + h^13 + h^10 + h^8 + h^7 + h^5 + h^2 + h)*x2^2*x4 + (h^25 + h^24 + h^21 + h^20 + h^18 + h^17 + h^15 + h^12 + h^11 + h^10 + h^6 + h^5 + 1)*x0*x4^2");
    polynomialArray.emplace_back("(h^28 + h^26 + h^25 + h^18 + h^15 + h^14 + h^13 + h^12 + h^11 + h^10 + h^9 + h^6 + h^4 + h^2)*x0*x1^2*x3 + (h^29 + h^27 + h^25 + h^24 + h^19 + h^16 + h^15 + h^14 + h^13 + h^12 + h^11 + h^10 + h^8 + h^7 + h^3 + 1)*x0^3*x4 + (h^26 + h^21 + h^20 + h^19 + h^17 + h^16 + h^14 + h^12 + h^11 + h^9 + h^8 + h^6 + h^5 + h^4 + h^2 + 1)*x0*x2*x4^2 + (h^29 + h^28 + h^26 + h^23 + h^19 + h^16 + h^14 + h^12 + h^11 + h^5 + h)*x0^3 + (h^29 + h^26 + h^23 + h^22 + h^21 + h^19 + h^14 + h^9 + h^6 + h^4 + h^3 + h^2)*x1*x3^2");
    polynomialArray.emplace_back("(h^28 + h^25 + h^24 + h^23 + h^21 + h^20 + h^19 + h^18 + h^17 + h^15 + h^14 + h^12 + h^9 + h^8 + h^6 + h^5 + h^4 + h^3 + h^2 + h)*x1^2*x2*x3 + (h^25 + h^23 + h^19 + h^18 + h^15 + h^14 + h^13 + h^12 + h^11 + h^10 + h^9 + h^8 + h^5 + h^3 + h^2 + h)*x0*x2*x3^2 + (h^27 + h^26 + h^25 + h^22 + h^21 + h^20 + h^19 + h^18 + h^17 + h^13 + h^12 + h^11 + h^10 + h^7 + h^6 + h^3 + h^2 + h)*x3^4 + (h^29 + h^28 + h^27 + h^26 + h^25 + h^23 + h^22 + h^19 + h^17 + h^14 + h^11 + h^10 + h^8 + h^7 + h^6 + h^3 + 1)*x3^2 + (h^28 + h^26 + h^24 + h^22 + h^20 + h^19 + h^17 + h^16 + h^15 + h^14 + h^9 + h^8 + h^6 + h^5 + h^2 + h)*x4^2");
    polynomialArray.emplace_back("(h^29 + h^27 + h^21 + h^17 + h^16 + h^15 + h^12 + h^11 + h^10 + h^9 + h^8 + h^4 + h^3 + h + 1)*x1*x2^3 + (h^27 + h^26 + h^25 + h^24 + h^22 + h^21 + h^18 + h^17 + h^16 + h^14 + h^13 + h^12 + h^10 + h^9 + h^8 + h^6 + h^2 + 1)*x2*x3*x4^2 + (h^29 + h^26 + h^25 + h^22 + h^19 + h^17 + h^16 + h^15 + h^14 + h^12 + h^10 + h^9 + h^3 + h^2)*x4^4 + (h^29 + h^24 + h^23 + h^22 + h^20 + h^19 + h^16 + h^14 + h^12 + h^11 + h^10 + h^8 + h^7 + h^6 + h^3 + h^2 + 1)*x0*x2^2 + (h^29 + h^28 + h^26 + h^19 + h^14 + h^13 + h^8 + h^7 + h^5 + h^4 + h^3 + h + 1)*x1^2");
    polynomialArray.emplace_back("(h^28 + h^27 + h^19 + h^18 + h^17 + h^16 + h^13 + h^9 + h^8 + h^6 + h^3 + h^2 + 1)*x0^2*x1^2 + (h^27 + h^25 + h^21 + h^20 + h^19 + h^17 + h^15 + h^14 + h^12 + h^10 + h^7 + h^6 + h^5)*x1^2*x3^2 + (h^28 + h^26 + h^25 + h^23 + h^18 + h^17 + h^9 + h^7 + h^4 + h^3 + h^2 + 1)*x0*x1^2*x4 + (h^28 + h^27 + h^26 + h^25 + h^24 + h^23 + h^22 + h^21 + h^19 + h^18 + h^16 + h^12 + h^11 + h^10 + h^9 + h^8 + h^6)*x0^2*x1 + (h^29 + h^27 + h^25 + h^21 + h^20 + h^16 + h^8 + h^3 + h^2 + h)*x0^2*x2");
    
    // Compute a reduce groebner basis
    vector<string> basis= groebnerBasisGF2ExtensionF4("h^30 + h^29 + h^27 + h^26 + h^24 + h^23 + h^22 + h^19 + h^18 + h^17 + h^16 + h^15 + h^13 + h^12 + h^5 + h^4 + h^2 + h + 1", 5, variableName, "h", polynomialArray, 1, 0);
    
    // Fill reference vectors
    vector<string> groebnerBasis;
    groebnerBasis.push_back("((+1)*1)");

    /* 32 bits */
    bool test = true;
    size_t i = 0;
    while(i < basis.size() && test == true )
    {
        test = test && (groebnerBasis[i].compare(basis[i])==0);
        i++;
    }
    if(test==true)
    {
        cout << "Test trivial2 gf2 extension 32 bits pass" << endl;
        return 0;
    }
    else
    {
        cout << "Test trivial2 gf2 extension 32 bits failed" << endl;
        return -1;
    }
}


