/* 
 * Copyright (C) 2015 Antoine Joux, Vanessa Vitse and Titouan Coladon
 * 
 * This file is part of openf4.
 * 
 * openf4 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openf4 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with openf4.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file check-random-gf2-extension-64bits.cpp
 *  \example check-random-gf2-extension-64bits.cpp
 *  \brief Library check tests.
 *  \ingroup examples
 *  \author Vanessa VITSE, Antoine JOUX, Titouan COLADON
 */

#include <iostream>
#include <string>
#include <vector>
#include <libopenf4.h>

using namespace std;

int main (int argc, char **argv)
{
    cout << "#########################################################" << endl;
    cout << "#            CHECK RANDOM GF(2^53) BITS                 #" << endl;
    cout << "#########################################################" << endl << endl;
    
    // Create polynomial array
    vector<string> polynomialArray;
    
    // Create variable name array
    vector<string> variableName;
    for(int i = 0; i < 5; i++)
    {
        variableName.push_back('x'+to_string(i));
    }
    
    // Fill the polynomial array
    polynomialArray.emplace_back("(t^51 + t^50 + t^46 + t^45 + t^43 + t^40 + t^39 + t^36 + t^33 + t^32 + t^30 + t^29 + t^28 + t^24 + t^23 + t^22 + t^20 + t^19 + t^17 + t^15 + t^14 + t^13 + t^9 + t^7 + t^6 + t^4)*x0^2*x1*x4 + (t^51 + t^50 + t^49 + t^48 + t^46 + t^45 + t^44 + t^41 + t^35 + t^34 + t^33 + t^31 + t^30 + t^29 + t^27 + t^26 + t^25 + t^23 + t^21 + t^16 + t^14 + t^13 + t^10 + t^8 + t^2)*x1^2*x3 + (t^48 + t^46 + t^42 + t^41 + t^40 + t^39 + t^36 + t^35 + t^34 + t^33 + t^30 + t^29 + t^28 + t^27 + t^26 + t^25 + t^24 + t^23 + t^21 + t^19 + t^17 + t^10 + t^9 + t^8 + t^7 + t^6 + t^5 + t^4 + t^2 + t + 1)*x1*x3^2 + (t^52 + t^50 + t^40 + t^33 + t^32 + t^29 + t^28 + t^27 + t^25 + t^24 + t^22 + t^19 + t^18 + t^17 + t^16 + t^15 + t^12 + t^10 + t^5 + t + 1)*x0*x1 + (t^52 + t^51 + t^44 + t^43 + t^42 + t^41 + t^39 + t^37 + t^35 + t^34 + t^33 + t^30 + t^29 + t^28 + t^25 + t^24 + t^23 + t^16 + t^15 + t^14 + t^12 + t^11 + t^10 + t^8 + t^7 + t + 1)*x1*x3");
    polynomialArray.emplace_back("(t^51 + t^47 + t^46 + t^45 + t^41 + t^38 + t^35 + t^34 + t^33 + t^32 + t^29 + t^27 + t^25 + t^24 + t^23 + t^22 + t^18 + t^14 + t^9 + t^7 + t^6 + t^5 + t^2 + 1)*x3^4 + (t^51 + t^49 + t^45 + t^43 + t^42 + t^41 + t^38 + t^37 + t^36 + t^35 + t^33 + t^31 + t^30 + t^27 + t^26 + t^23 + t^21 + t^20 + t^16 + t^15 + t^12 + t^11 + t^10 + t^9 + t^7 + t^4 + t^3 + t^2)*x0*x1*x2*x4 + (t^52 + t^47 + t^46 + t^40 + t^38 + t^32 + t^28 + t^25 + t^20 + t^18 + t^16 + t^15 + t^14 + t^12 + t^11 + t^10 + t^8 + t^5 + t^3 + t^2 + t + 1)*x0*x3^2*x4 + (t^47 + t^46 + t^43 + t^42 + t^41 + t^38 + t^37 + t^35 + t^30 + t^27 + t^26 + t^24 + t^23 + t^20 + t^18 + t^16 + t^15 + t^14 + t^13 + t^12 + t^11 + t^10 + t^9 + t^8 + t^7 + t^4 + t + 1)*x0*x1*x2 + (t^52 + t^49 + t^47 + t^45 + t^44 + t^42 + t^41 + t^40 + t^36 + t^34 + t^32 + t^31 + t^28 + t^26 + t^25 + t^23 + t^19 + t^17 + t^14 + t^13 + t^12 + t^10 + t^9 + t^8 + t^6 + t^3 + t^2 + t + 1)*x0*x2*x4");
    polynomialArray.emplace_back("(t^49 + t^48 + t^47 + t^46 + t^39 + t^38 + t^36 + t^35 + t^30 + t^28 + t^24 + t^23 + t^21 + t^20 + t^19 + t^18 + t^17 + t^16 + t^14 + t^8 + t^6 + t^5 + t^3 + t^2 + t)*x0*x2^3 + (t^47 + t^45 + t^44 + t^43 + t^42 + t^39 + t^35 + t^34 + t^33 + t^32 + t^31 + t^29 + t^27 + t^25 + t^20 + t^19 + t^18 + t^16 + t^14 + t^12 + t^11 + t^9 + t^4 + t)*x0*x2^2*x3 + (t^48 + t^47 + t^43 + t^41 + t^39 + t^37 + t^36 + t^35 + t^30 + t^27 + t^23 + t^22 + t^21 + t^20 + t^19 + t^16 + t^15 + t^14 + t^12 + t^11 + t^10 + t^5 + t^3)*x1*x2^2*x3 + (t^52 + t^46 + t^43 + t^42 + t^40 + t^39 + t^38 + t^37 + t^36 + t^35 + t^33 + t^32 + t^30 + t^29 + t^27 + t^26 + t^22 + t^21 + t^20 + t^17 + t^16 + t^15 + t^14 + t^13 + t^11 + t^10 + t^9 + t^6 + t^4 + t^3 + t^2)*x3*x4^3 + (t^52 + t^50 + t^49 + t^48 + t^46 + t^45 + t^43 + t^42 + t^37 + t^35 + t^34 + t^30 + t^28 + t^26 + t^25 + t^24 + t^23 + t^22 + t^21 + t^19 + t^16 + t^13 + t^12 + t^8 + t^7 + t^6 + t^5 + t^4 + t^2 + t + 1)*x4^4");
    polynomialArray.emplace_back("(t^51 + t^50 + t^48 + t^47 + t^46 + t^45 + t^44 + t^40 + t^39 + t^36 + t^35 + t^33 + t^31 + t^29 + t^28 + t^27 + t^25 + t^24 + t^23 + t^22 + t^18 + t^16 + t^14 + t^13 + t^12 + t^10 + t^7 + t^5 + t^4 + t^2)*x2^2*x3^2 + (t^52 + t^51 + t^49 + t^48 + t^45 + t^44 + t^42 + t^41 + t^40 + t^36 + t^32 + t^31 + t^27 + t^26 + t^25 + t^19 + t^17 + t^16 + t^14 + t^13 + t^10 + t^8 + t^7 + t^6 + t^5 + 1)*x0*x3*x4^2 + (t^52 + t^51 + t^50 + t^49 + t^46 + t^45 + t^43 + t^41 + t^40 + t^39 + t^38 + t^35 + t^32 + t^31 + t^29 + t^28 + t^27 + t^20 + t^19 + t^18 + t^16 + t^15 + t^14 + t^13 + t^12 + t^10 + t^9 + t^7 + t^3 + t^2)*x0*x3^2 + (t^50 + t^49 + t^47 + t^46 + t^45 + t^40 + t^39 + t^38 + t^37 + t^36 + t^34 + t^31 + t^28 + t^26 + t^23 + t^22 + t^19 + t^18 + t^16 + t^15 + t^13 + t^12 + t^10 + t^9 + t^8 + t^6 + t^4 + t^2)*x2^2*x4 + (t^52 + t^51 + t^48 + t^47 + t^46 + t^42 + t^40 + t^38 + t^37 + t^35 + t^34 + t^33 + t^32 + t^31 + t^30 + t^29 + t^22 + t^19 + t^16 + t^14 + t^12 + t^11 + t^10 + t^8 + t^7 + t^6 + t^5 + t^4 + t^3 + t + 1)*x0*x4^2");
    polynomialArray.emplace_back("(t^52 + t^51 + t^49 + t^43 + t^42 + t^41 + t^40 + t^39 + t^37 + t^35 + t^33 + t^31 + t^29 + t^27 + t^25 + t^24 + t^23 + t^22 + t^21 + t^18 + t^16 + t^15 + t^12 + t^11 + t^10 + t^9 + t^7 + t^6 + t^3 + t + 1)*x1*x2^2*x3 + (t^51 + t^50 + t^49 + t^48 + t^47 + t^45 + t^43 + t^40 + t^39 + t^36 + t^32 + t^30 + t^29 + t^26 + t^25 + t^24 + t^23 + t^22 + t^20 + t^17 + t^16 + t^15 + t^14 + t^12 + t^11 + t^9 + t^7 + t^5 + t^4 + t^2)*x1^3*x4 + (t^52 + t^50 + t^47 + t^46 + t^43 + t^42 + t^35 + t^34 + t^33 + t^32 + t^31 + t^28 + t^22 + t^21 + t^18 + t^17 + t^16 + t^13 + t^12 + t^11 + t^10 + t^7 + t^6 + t^4 + t^3 + t^2 + 1)*x1*x2^2*x4 + (t^52 + t^50 + t^47 + t^40 + t^38 + t^37 + t^36 + t^34 + t^33 + t^32 + t^30 + t^27 + t^25 + t^24 + t^23 + t^22 + t^19 + t^16 + t^15 + t^14 + t^12 + t^11 + t^10 + t^7 + t^5 + t^3 + t^2)*x1^2*x3*x4 + (t^52 + t^51 + t^49 + t^47 + t^46 + t^44 + t^43 + t^42 + t^41 + t^40 + t^39 + t^38 + t^37 + t^36 + t^32 + t^30 + t^29 + t^26 + t^25 + t^24 + t^23 + t^22 + t^18 + t^16 + t^13 + t^11 + t^10 + t^8 + t^7 + t^4 + t^3 + t^2 + t)*x3^2");
    polynomialArray.emplace_back("(t^52 + t^51 + t^47 + t^45 + t^44 + t^42 + t^41 + t^39 + t^38 + t^36 + t^32 + t^31 + t^29 + t^27 + t^26 + t^25 + t^22 + t^19 + t^16 + t^14 + t^11 + t^8 + t^6 + t^4 + t^3 + t^2 + 1)*x0^2*x1*x2 + (t^52 + t^51 + t^49 + t^48 + t^46 + t^45 + t^43 + t^42 + t^40 + t^39 + t^38 + t^36 + t^35 + t^34 + t^32 + t^31 + t^29 + t^25 + t^23 + t^13 + t^12 + t^7 + t^5 + 1)*x2^4 + (t^51 + t^50 + t^47 + t^44 + t^42 + t^41 + t^40 + t^39 + t^38 + t^34 + t^33 + t^32 + t^31 + t^30 + t^28 + t^27 + t^25 + t^24 + t^23 + t^22 + t^21 + t^20 + t^18 + t^10 + t^7 + t^2 + 1)*x0*x3^3 + (t^52 + t^48 + t^47 + t^46 + t^44 + t^43 + t^41 + t^39 + t^38 + t^37 + t^34 + t^31 + t^25 + t^21 + t^20 + t^19 + t^16 + t^15 + t^14 + t^9 + t^8 + t^7 + t^5 + t^4 + t^3 + t^2 + 1)*x1*x2^2 + (t^52 + t^51 + t^49 + t^47 + t^45 + t^44 + t^39 + t^36 + t^31 + t^30 + t^27 + t^24 + t^23 + t^22 + t^21 + t^20 + t^18 + t^17 + t^15 + t^14 + t^12 + t^10 + t^7 + t^4 + t^3 + 1)*x4^3");
    
    // Compute a reduce groebner basis
    vector<string> basis= groebnerBasisGF2ExtensionF4("t^53 + t^52 + t^50 + t^49 + t^48 + t^42 + t^41 + t^40 + t^37 + t^36 + t^34 + t^33 + t^32 + t^10 + t^9 + t^8 + t^5 + t^4 + t^2 + t + 1", 5, variableName, "t", polynomialArray, 1, 0);
    
    // Fill reference vectors
    vector<string> groebnerBasis;
    groebnerBasis.push_back("((+1)*x0^2*x1^1*x4^1) + ((t^48+t^47+t^44+t^41+t^40+t^38+t^36+t^34+t^33+t^32+t^31+t^28+t^27+t^26+t^23+t^22+t^21+t^19+t^18+t^16+t^12+t^10+t^9+t^6+t^5)*x1^2*x3^1) + ((t^51+t^44+t^43+t^42+t^39+t^38+t^36+t^35+t^34+t^30+t^26+t^25+t^24+t^22+t^18+t^17+t^14+t^12+t^8+t^5+t^4+t^3)*x1^1*x3^2) + ((t^51+t^49+t^46+t^43+t^41+t^40+t^36+t^34+t^31+t^26+t^25+t^23+t^20+t^19+t^13+t^12+t^10+t^7+t^5+t+1)*x0^1*x1^1) + ((t^49+t^48+t^41+t^39+t^35+t^34+t^28+t^27+t^26+t^25+t^24+t^22+t^18+t^12+t^9+t^7+t^6+t^3+t^2+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x3^4) + ((t^51+t^50+t^45+t^41+t^40+t^39+t^37+t^33+t^30+t^29+t^28+t^27+t^26+t^23+t^22+t^20+t^19+t^17+t^13+t^12+t^11+t^10+t^5+t^4+t^3+t^2+t+1)*x2^3*x4^1) + ((t^52+t^48+t^46+t^44+t^43+t^42+t^41+t^40+t^38+t^37+t^36+t^33+t^32+t^31+t^25+t^24+t^22+t^21+t^19+t^18+t^16+t^15+t^11+t^9+t^8+t^5+t^3+t^2+t)*x1^1*x3^2*x4^1) + ((t^51+t^49+t^47+t^43+t^38+t^33+t^32+t^30+t^29+t^27+t^26+t^22+t^20+t^19+t^18+t^17+t^14+t^12+t^11+t^9+t^8+t^3+t^2+t)*x3^3*x4^1) + ((t^52+t^51+t^49+t^48+t^47+t^46+t^42+t^40+t^37+t^35+t^33+t^30+t^26+t^24+t^23+t^20+t^18+t^14+t^12+t^10+t^5+t^4+t^2)*x0^1*x2^1*x4^2) + ((t^50+t^47+t^46+t^45+t^44+t^43+t^42+t^41+t^40+t^38+t^37+t^36+t^34+t^33+t^31+t^26+t^23+t^22+t^18+t^15+t^14+t^13+t^10+t^9+t^8+t^5+t+1)*x2^2*x4^2) + ((t^52+t^48+t^47+t^40+t^39+t^37+t^35+t^34+t^33+t^29+t^28+t^27+t^22+t^20+t^19+t^17+t^14+t^13+t^11+t^9+t^8+t^4+t^2)*x0^1*x4^3) + ((t^51+t^50+t^49+t^47+t^46+t^45+t^44+t^43+t^42+t^38+t^34+t^31+t^30+t^29+t^26+t^24+t^23+t^21+t^20+t^19+t^18+t^17+t^13+t^12+t^11+t^6+t^5+t^3+t^2+t+1)*x0^1*x1^1*x2^1) + ((t^52+t^50+t^49+t^47+t^45+t^44+t^35+t^31+t^26+t^21+t^19+t^16+t^15+t^14+t^13+t^8+t^7+t^6+t^4+t^3+1)*x1^2*x3^1) + ((t^52+t^46+t^45+t^44+t^43+t^41+t^40+t^39+t^37+t^35+t^34+t^33+t^30+t^28+t^27+t^25+t^23+t^19+t^18+t^17+t^16+t^15+t^11+t^4+t^2+1)*x1^1*x3^2) + ((t^51+t^49+t^48+t^47+t^45+t^42+t^40+t^37+t^29+t^26+t^25+t^22+t^21+t^13+t^11+t^10+t^9+t^6+t^5+t^4+t^2+t)*x0^1*x2^1*x4^1) + ((t^51+t^49+t^48+t^47+t^46+t^44+t^43+t^40+t^38+t^35+t^34+t^33+t^31+t^30+t^29+t^27+t^25+t^24+t^23+t^21+t^15+t^12+t^7+t+1)*x0^1*x1^1) + ((t^51+t^50+t^48+t^46+t^43+t^42+t^41+t^39+t^38+t^37+t^36+t^34+t^32+t^30+t^29+t^28+t^27+t^26+t^25+t^24+t^23+t^20+t^19+t^17+t^14+t^12+t^11+t^10+t^4+t^3+t^2)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x0^1*x2^3) + ((t^51+t^48+t^46+t^45+t^41+t^39+t^38+t^36+t^34+t^33+t^30+t^29+t^28+t^27+t^26+t^25+t^24+t^23+t^21+t^19+t^18+t^15+t^13+t^12+t^9+t^8+t^7+t^5+t^3+t^2+1)*x0^1*x2^2*x3^1) + ((t^51+t^50+t^48+t^47+t^45+t^44+t^36+t^28+t^27+t^24+t^23+t^21+t^19+t^17+t^16+t^12+t^6+t^5+t^3+t^2+t)*x0^1*x2^2*x4^1) + ((t^50+t^49+t^41+t^40+t^39+t^37+t^35+t^33+t^32+t^30+t^28+t^27+t^24+t^23+t^20+t^18+t^17+t^16+t^10+t^9+t^8+t^7+t^6+t^3+t)*x2^3*x4^1) + ((t^52+t^51+t^50+t^48+t^43+t^42+t^40+t^39+t^35+t^34+t^33+t^30+t^27+t^21+t^17+t^16+t^15+t^14+t^11+t^10+t^9+t^8+t^7+t^6+t^4+t^2)*x1^1*x3^2*x4^1) + ((t^52+t^51+t^49+t^48+t^47+t^43+t^38+t^37+t^34+t^33+t^25+t^24+t^22+t^20+t^19+t^18+t^17+t^16+t^11+t^10+t^7+t^6+t^2)*x3^3*x4^1) + ((t^52+t^49+t^48+t^47+t^43+t^41+t^39+t^38+t^37+t^36+t^34+t^32+t^31+t^28+t^27+t^23+t^21+t^20+t^19+t^18+t^13+t^10+t^9+t^8+t^6+t^2+1)*x0^1*x2^1*x4^2) + ((t^52+t^51+t^48+t^46+t^42+t^41+t^40+t^39+t^38+t^36+t^34+t^31+t^30+t^28+t^27+t^26+t^24+t^23+t^19+t^18+t^15+t^13+t^11+t^8+t^7+t^6+t^4+t^3+t)*x2^2*x4^2) + ((t^52+t^50+t^49+t^48+t^47+t^42+t^39+t^38+t^37+t^36+t^34+t^33+t^32+t^31+t^29+t^26+t^21+t^20+t^17+t^16+t^14+t^12+t^11+t^9+t^7+t^6+t^4+t^3+t^2+t)*x0^1*x4^3) + ((t^52+t^50+t^47+t^43+t^41+t^40+t^38+t^34+t^32+t^31+t^30+t^28+t^27+t^26+t^25+t^24+t^22+t^21+t^20+t^19+t^18+t^17+t^16+t^15+t^14+t^12+t^10+t^9+t^8+t^7+t^6+t^4+t^3+t^2)*x3^1*x4^3) + ((t^52+t^49+t^47+t^46+t^45+t^44+t^41+t^40+t^39+t^38+t^35+t^34+t^30+t^29+t^28+t^27+t^25+t^22+t^18+t^16+t^14+t^11+t^8+t^7+t^2+1)*x4^4) + ((t^50+t^49+t^48+t^46+t^45+t^44+t^43+t^40+t^39+t^38+t^36+t^34+t^33+t^32+t^31+t^30+t^29+t^28+t^27+t^24+t^21+t^20+t^14+t^13+t^7+t^5+t^4+t^2+t)*x0^1*x1^1*x2^1) + ((t^52+t^51+t^50+t^49+t^48+t^46+t^44+t^43+t^42+t^41+t^38+t^36+t^34+t^33+t^32+t^31+t^27+t^26+t^25+t^22+t^21+t^20+t^19+t^18+t^16+t^14+t^13+t^12+t^11+t^5+t^3+t)*x1^2*x3^1) + ((t^52+t^50+t^49+t^48+t^47+t^46+t^45+t^44+t^42+t^41+t^38+t^37+t^36+t^35+t^32+t^31+t^29+t^26+t^25+t^23+t^20+t^19+t^17+t^15+t^14+t^9+t^7+t^6+t^4+t^3+1)*x1^1*x3^2) + ((t^50+t^48+t^45+t^43+t^42+t^41+t^40+t^36+t^33+t^30+t^25+t^23+t^20+t^18+t^17+t^16+t^8+t^7+t^5+t^4+1)*x0^1*x2^1*x4^1) + ((t^48+t^47+t^45+t^44+t^42+t^36+t^35+t^32+t^30+t^22+t^20+t^18+t^17+t^15+t^14+t^12+t^8+t^4+1)*x0^1*x1^1) + ((t^52+t^51+t^50+t^49+t^48+t^46+t^45+t^43+t^42+t^32+t^31+t^30+t^27+t^26+t^23+t^22+t^21+t^19+t^18+t^12+t^11+t^10+t^8+t^6+t^5+t^2+t)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x2^2*x3^2)");
    groebnerBasis.push_back("((+1)*x1^1*x2^2*x3^1) + ((t^52+t^51+t^50+t^47+t^45+t^44+t^41+t^40+t^39+t^36+t^35+t^33+t^32+t^28+t^26+t^22+t^17+t^16+t^15+t^14+t^13+t^12+t^11+t^5+t^4+t^3+t)*x0^1*x2^2*x4^1) + ((t^51+t^47+t^46+t^44+t^42+t^38+t^37+t^36+t^35+t^33+t^30+t^29+t^27+t^22+t^20+t^18+t^17+t^13+t^10+t^9+t^8+t^6+t^4+t^3+t^2+t+1)*x2^3*x4^1) + ((t^52+t^51+t^50+t^49+t^48+t^46+t^45+t^44+t^42+t^41+t^40+t^35+t^34+t^33+t^32+t^31+t^30+t^29+t^26+t^25+t^24+t^23+t^21+t^20+t^18+t^17+t^12+t^10+t^7+t^6+t^5+t^4+t^2)*x1^1*x3^2*x4^1) + ((t^52+t^51+t^50+t^49+t^48+t^46+t^43+t^42+t^41+t^40+t^35+t^30+t^29+t^28+t^26+t^24+t^23+t^22+t^21+t^20+t^19+t^18+t^17+t^16+t^14+t^11+t^10+t^9+t^8+t^7+t^5+t^3+t)*x3^3*x4^1) + ((t^52+t^51+t^50+t^43+t^42+t^41+t^39+t^33+t^32+t^31+t^30+t^27+t^25+t^23+t^22+t^19+t^18+t^15+t^14+t^13+t^12+t^9+t^8+t^5+t^4+t^3+t^2+t)*x0^1*x2^1*x4^2) + ((t^51+t^49+t^48+t^47+t^45+t^44+t^40+t^37+t^33+t^32+t^30+t^28+t^24+t^23+t^19+t^18+t^17+t^16+t^13+t^11+t^9+t^6+t^3+t^2)*x2^2*x4^2) + ((t^52+t^50+t^49+t^45+t^42+t^41+t^40+t^39+t^38+t^35+t^33+t^31+t^30+t^28+t^27+t^25+t^24+t^23+t^22+t^21+t^18+t^17+t^10+t^7+t^5+t^4+t^2)*x0^1*x4^3) + ((t^52+t^51+t^48+t^47+t^46+t^44+t^42+t^39+t^37+t^34+t^32+t^31+t^25+t^24+t^21+t^19+t^18+t^17+t^16+t^15+t^11+t^10+t^9+t^7+t^3)*x0^1*x1^1*x2^1) + ((t^52+t^51+t^48+t^45+t^44+t^43+t^42+t^39+t^37+t^35+t^34+t^33+t^32+t^31+t^29+t^27+t^26+t^25+t^24+t^23+t^22+t^19+t^17+t^16+t^14+t^12+t^11+t^9+t^7+t^4+1)*x1^2*x3^1) + ((t^52+t^51+t^50+t^47+t^46+t^45+t^37+t^35+t^32+t^29+t^25+t^20+t^19+t^18+t^16+t^13+t^11+t^10+t^6+t+1)*x1^1*x3^2) + ((t^51+t^50+t^49+t^46+t^45+t^44+t^43+t^42+t^39+t^34+t^33+t^31+t^28+t^27+t^26+t^25+t^20+t^18+t^17+t^15+t^11+t^10+t^9+t^8+t^6+t^5+t^4+t+1)*x0^1*x2^1*x4^1) + ((t^52+t^51+t^50+t^46+t^45+t^44+t^43+t^41+t^40+t^39+t^35+t^31+t^30+t^28+t^27+t^26+t^21+t^20+t^18+t^17+t^11+t^9+t^8+t^6+t^4)*x0^1*x1^1) + ((t^51+t^49+t^48+t^47+t^46+t^44+t^43+t^41+t^39+t^35+t^34+t^33+t^32+t^30+t^28+t^26+t^25+t^24+t^23+t^21+t^19+t^18+t^17+t^15+t^14+t^10+t^9+t^8+t^6+t^5+t^4+t^2+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x0^2*x1^1*x2^1) + ((t^51+t^49+t^48+t^45+t^41+t^40+t^39+t^38+t^35+t^31+t^30+t^29+t^26+t^21+t^19+t^18+t^17+t^12+t^10+t^8+t^5+t^4+1)*x2^3*x4^1) + ((t^51+t^50+t^49+t^47+t^44+t^43+t^42+t^41+t^39+t^38+t^37+t^35+t^30+t^29+t^28+t^26+t^24+t^22+t^18+t^16+t^15+t^12+t^11+t^9+t^6+t^5+t^3+t^2)*x0^1*x2^1*x3^1*x4^1) + ((t^49+t^46+t^40+t^39+t^37+t^36+t^35+t^34+t^31+t^30+t^28+t^27+t^23+t^22+t^21+t^18+t^14+t^12+t^10+t^8+t^7+t^5+t+1)*x1^1*x3^2*x4^1) + ((t^52+t^50+t^48+t^47+t^45+t^36+t^33+t^30+t^27+t^26+t^23+t^22+t^21+t^20+t^19+t^17+t^16+t^15+t^14+t^12+t^10+t^8+t^5+t^4+t^3+t^2+1)*x3^3*x4^1) + ((t^52+t^51+t^49+t^48+t^47+t^46+t^45+t^43+t^40+t^39+t^38+t^34+t^32+t^31+t^30+t^26+t^22+t^20+t^18+t^17+t^15+t^11+t^10+t^5+t^4+t^3)*x0^1*x2^1*x4^2) + ((t^52+t^51+t^48+t^44+t^43+t^42+t^41+t^39+t^38+t^37+t^36+t^35+t^33+t^32+t^29+t^28+t^26+t^24+t^21+t^18+t^15+t^14+t^13+t^12+t^11+t^10+t^9+t^8+t^7+t^6+t^5+t^3+t^2+t)*x2^2*x4^2) + ((t^50+t^49+t^46+t^45+t^43+t^40+t^39+t^37+t^36+t^34+t^32+t^31+t^27+t^26+t^25+t^23+t^22+t^21+t^19+t^18+t^16+t^15+t^14+t^13+t^12+t^8+t^6+t^4+t+1)*x0^1*x4^3) + ((t^52+t^51+t^50+t^49+t^48+t^47+t^46+t^44+t^43+t^42+t^41+t^39+t^38+t^36+t^34+t^32+t^31+t^30+t^29+t^28+t^25+t^22+t^21+t^20+t^18+t^17+t^16+t^11+t^10+t^7+t^4+t^3+t^2+1)*x0^1*x1^1*x2^1) + ((t^51+t^50+t^49+t^46+t^44+t^43+t^42+t^36+t^35+t^34+t^32+t^31+t^29+t^27+t^25+t^24+t^23+t^22+t^19+t^13+t^9+t^8)*x1^2*x3^1) + ((t^52+t^51+t^50+t^49+t^48+t^47+t^45+t^44+t^43+t^41+t^40+t^39+t^38+t^36+t^35+t^34+t^32+t^29+t^24+t^23+t^19+t^18+t^15+t^14+t^13+t^10+t^7+t^4+t^3+t+1)*x1^1*x3^2) + ((t^51+t^50+t^49+t^47+t^46+t^44+t^42+t^41+t^37+t^35+t^34+t^33+t^32+t^31+t^30+t^25+t^21+t^20+t^18+t^15+t^14+t^13+t^11+t^10+t^8+t^6+t^4+t^2+t+1)*x0^1*x2^1*x4^1) + ((t^51+t^50+t^44+t^43+t^42+t^41+t^40+t^36+t^33+t^32+t^31+t^30+t^28+t^26+t^25+t^21+t^19+t^18+t^15+t^14+t^13+t^12+t^11+t^10+t^9+t^2)*x0^1*x1^1) + ((t^51+t^50+t^47+t^46+t^44+t^43+t^42+t^37+t^34+t^28+t^27+t^26+t^22+t^21+t^19+t^18+t^14+t^13+t^9+t^8+t^7+t^4+t^3+t^2+t)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x0^1*x2^2*x3^1*x4^1)");
    groebnerBasis.push_back("((+1)*x2^3*x3^1*x4^1) + ((t^52+t^46+t^45+t^44+t^41+t^38+t^37+t^36+t^33+t^29+t^28+t^25+t^23+t^20+t^19+t^17+t^16+t^13+t^10+t^9+t^3+t+1)*x2^3*x4^1) + ((t^50+t^49+t^48+t^46+t^45+t^43+t^42+t^39+t^35+t^32+t^31+t^30+t^29+t^28+t^24+t^21+t^18+t^16+t^12+t^10+t^9+t^8+t^5+t^4+t^3+1)*x0^1*x2^1*x4^2)");
    groebnerBasis.push_back("((+1)*x1^1*x2^1*x3^2*x4^1) + ((t^51+t^49+t^42+t^40+t^38+t^37+t^34+t^32+t^31+t^28+t^27+t^26+t^25+t^24+t^21+t^20+t^18+t^15+t^10+t^7+t^6+t^4+1)*x2^3*x4^1) + ((t^51+t^48+t^43+t^41+t^40+t^39+t^38+t^36+t^34+t^31+t^30+t^27+t^24+t^23+t^22+t^21+t^20+t^18+t^16+t^13+t^11+t^8+t^7+t^6+t^5+t^3+t^2+t+1)*x0^1*x2^1*x4^2)");
    groebnerBasis.push_back("((+1)*x0^3*x4^2)");
    groebnerBasis.push_back("((+1)*x0^1*x2^2*x4^2) + ((t^51+t^50+t^49+t^47+t^42+t^41+t^38+t^37+t^33+t^32+t^29+t^22+t^16+t^15+t^14+t^13+t^10+t^8+t^7+t^6+t^4+t^3+t)*x2^3*x4^1) + ((t^52+t^51+t^49+t^46+t^45+t^43+t^40+t^39+t^37+t^34+t^31+t^30+t^29+t^26+t^25+t^21+t^20+t^18+t^15+t^14+t^9+t^8+t^4+t^3+t^2+t)*x1^1*x3^2*x4^1) + ((t^51+t^50+t^48+t^47+t^46+t^45+t^44+t^43+t^42+t^41+t^38+t^34+t^33+t^32+t^30+t^29+t^23+t^22+t^21+t^19+t^18+t^17+t^14+t^12+t^9+t^8+t^7+t^6+t^4+t^2+t)*x3^3*x4^1) + ((t^52+t^49+t^48+t^46+t^45+t^43+t^41+t^40+t^36+t^35+t^28+t^27+t^25+t^23+t^22+t^20+t^12+t^11+t^9+t^7+t^6+t^5+t^4+1)*x0^1*x2^1*x4^2) + ((t^51+t^50+t^45+t^43+t^39+t^38+t^37+t^34+t^33+t^31+t^30+t^25+t^24+t^23+t^20+t^19+t^16+t^13+t^12+t^10+t^8+t^7+t^6+t^4+t^3+t)*x2^2*x4^2) + ((t^52+t^49+t^47+t^46+t^41+t^39+t^38+t^37+t^35+t^33+t^30+t^28+t^25+t^23+t^22+t^17+t^15+t^13+t^12+t^10+t^9+t^6+t^4+t^3+t^2+1)*x0^1*x4^3) + ((t^50+t^49+t^47+t^46+t^45+t^43+t^41+t^40+t^37+t^35+t^30+t^29+t^28+t^27+t^26+t^25+t^24+t^22+t^19+t^16+t^11+t^10+t^7+t^4+t^3+t^2)*x0^1*x1^1*x2^1) + ((t^52+t^51+t^50+t^49+t^48+t^47+t^42+t^37+t^36+t^28+t^25+t^22+t^17+t^16+t^15+t^14+t^12+t^10+t^6+t^4)*x1^2*x3^1) + ((t^51+t^48+t^45+t^43+t^42+t^40+t^39+t^38+t^37+t^35+t^34+t^31+t^30+t^28+t^27+t^26+t^25+t^24+t^21+t^18+t^16+t^15+t^14+t^13+t^12+t^11+t^5+t^4+t^2+1)*x1^1*x3^2) + ((t^51+t^49+t^48+t^47+t^46+t^43+t^41+t^40+t^37+t^36+t^35+t^34+t^31+t^29+t^28+t^27+t^26+t^23+t^22+t^20+t^18+t^14+t^12+t^11+t^10+t^8+t^6+t^5+t^4+t^2+t+1)*x0^1*x2^1*x4^1) + ((t^51+t^50+t^49+t^48+t^46+t^43+t^42+t^41+t^39+t^38+t^37+t^32+t^28+t^27+t^26+t^24+t^22+t^21+t^19+t^18+t^15+t^12+t^11+t^10+t^9+t^6+1)*x0^1*x1^1) + ((t^52+t^50+t^49+t^44+t^42+t^41+t^38+t^35+t^32+t^30+t^25+t^24+t^22+t^12+t^11+t^9+t^6+t^5)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x2^3*x4^2) + ((t^52+t^51+t^50+t^48+t^46+t^45+t^44+t^41+t^40+t^38+t^37+t^36+t^34+t^33+t^32+t^30+t^29+t^23+t^18+t^17+t^15+t^14+t^13+t^12+t^8+t^7+t+1)*x2^3*x4^1) + ((t^52+t^51+t^50+t^49+t^46+t^44+t^42+t^38+t^37+t^33+t^32+t^31+t^28+t^25+t^24+t^23+t^22+t^16+t^14+t^13+t^9+t^5+t^2)*x0^1*x2^1*x4^2)");
    groebnerBasis.push_back("((+1)*x1^2*x3^1*x4^2) + ((t^49+t^44+t^43+t^42+t^41+t^39+t^34+t^33+t^32+t^27+t^25+t^24+t^20+t^19+t^18+t^17+t^16+t^15+t^9+t^8+t^7+t^6)*x2^3*x4^1) + ((t^52+t^50+t^49+t^48+t^42+t^40+t^39+t^38+t^37+t^36+t^35+t^34+t^31+t^29+t^28+t^27+t^25+t^24+t^23+t^20+t^17+t^14+t^12+t^9+t^8+t^6+t^4+t^3+1)*x1^2*x3^1*x4^1) + ((t^49+t^48+t^47+t^43+t^40+t^39+t^37+t^36+t^34+t^33+t^32+t^30+t^26+t^24+t^21+t^15+t^14+t^12+t^11+t^10+t^7+t^6+t^4)*x1^1*x3^2*x4^1) + ((t^52+t^51+t^45+t^43+t^42+t^35+t^34+t^33+t^31+t^29+t^27+t^26+t^22+t^21+t^18+t^17+t^16+t^14+t^13+t^11+t^9+t^6+t^4+t+1)*x3^3*x4^1) + ((t^52+t^50+t^47+t^45+t^43+t^42+t^41+t^40+t^38+t^36+t^35+t^32+t^31+t^30+t^28+t^27+t^25+t^24+t^23+t^22+t^21+t^20+t^17+t^14+t^13+t^11+t^7+t^6+t^4+t^3+t^2+1)*x0^1*x1^1*x4^2) + ((t^51+t^50+t^48+t^47+t^43+t^42+t^41+t^40+t^39+t^37+t^35+t^32+t^29+t^27+t^25+t^24+t^21+t^19+t^18+t^16+t^15+t^14+t^13+t^11+t^10+t^9+t^8+t^7+t^4+t^3+t)*x0^1*x2^1*x4^2) + ((t^52+t^51+t^50+t^49+t^48+t^47+t^44+t^41+t^39+t^31+t^26+t^25+t^24+t^21+t^20+t^19+t^18+t^14+t^13+t^12+t^11+t^9+t^8+t^7+t^5+t^4+t^3+1)*x2^2*x4^2) + ((t^52+t^50+t^49+t^47+t^46+t^40+t^39+t^38+t^33+t^32+t^27+t^25+t^20+t^18+t^12+t^11+t^10+t^9+t^8+t^7+t^5+t^4+t^3+t^2+t)*x1^1*x3^1*x4^2) + ((t^50+t^48+t^47+t^45+t^43+t^41+t^40+t^37+t^36+t^35+t^34+t^31+t^30+t^29+t^25+t^22+t^19+t^17+t^15+t^14+t^13+t^11+t^9+t^5+t^4+t^3+1)*x0^1*x4^3) + ((t^51+t^50+t^49+t^48+t^47+t^46+t^43+t^42+t^39+t^38+t^37+t^30+t^28+t^27+t^25+t^22+t^20+t^18+t^17+t^11+t^10+t^7+t^6+t^5+t+1)*x0^1*x1^1*x2^1) + ((t^49+t^48+t^47+t^46+t^43+t^42+t^39+t^30+t^29+t^23+t^22+t^21+t^19+t^18+t^17+t^16+t^14+t^13+t^10+t^9+t^8+t^7+t^5+t^2+1)*x1^2*x3^1) + ((t^52+t^50+t^49+t^48+t^46+t^45+t^42+t^41+t^40+t^39+t^36+t^32+t^31+t^30+t^29+t^28+t^26+t^25+t^24+t^22+t^21+t^20+t^19+t^18+t^16+t^12+t^10+t^5+t^4+t^2+t)*x1^1*x3^2) + ((t^52+t^51+t^50+t^45+t^41+t^40+t^39+t^38+t^36+t^32+t^31+t^30+t^29+t^28+t^27+t^25+t^22+t^20+t^19+t^14+t^9+t^7+t^6+t^4+t^3+1)*x0^1*x1^1*x4^1) + ((t^50+t^46+t^45+t^43+t^42+t^41+t^38+t^37+t^35+t^33+t^30+t^29+t^27+t^24+t^21+t^19+t^18+t^17+t^15+t^14+t^12+t^11+t^9+t^8+t^7+t^6+t^3+t+1)*x0^1*x2^1*x4^1) + ((t^48+t^47+t^46+t^45+t^41+t^40+t^39+t^38+t^37+t^36+t^35+t^34+t^33+t^30+t^29+t^26+t^25+t^23+t^20+t^17+t^13+t^12+t^11+t^9+t^6+t^4+1)*x1^1*x3^1*x4^1) + ((t^52+t^51+t^50+t^48+t^47+t^44+t^43+t^42+t^41+t^37+t^36+t^33+t^30+t^25+t^24+t^22+t^20+t^19+t^18+t^17+t^15+t^12+t^10+t^7+t^6+t^5+t^2+t)*x0^1*x1^1) + ((t^52+t^51+t^49+t^48+t^47+t^46+t^45+t^40+t^35+t^34+t^32+t^30+t^29+t^26+t^25+t^21+t^17+t^16+t^14+t^9+t^7+t^6+t^5+t)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x1^1*x2^1*x3^1*x4^2) + ((t^50+t^49+t^48+t^47+t^46+t^45+t^42+t^41+t^39+t^37+t^35+t^32+t^30+t^29+t^28+t^27+t^26+t^24+t^23+t^19+t^18+t^17+t^13+t^12+t^9+t^8+t^6+t^5+t^3+t^2)*x2^3*x4^1) + ((t^52+t^50+t^48+t^47+t^46+t^44+t^43+t^40+t^36+t^34+t^29+t^27+t^26+t^21+t^20+t^18+t^17+t^16+t^15+t^14+t^13+t^12+t^11+t^10+t^9+t^3+t)*x0^1*x2^1*x4^2)");
    groebnerBasis.push_back("((+1)*x2^2*x3^1*x4^2) + ((t^52+t^50+t^49+t^48+t^47+t^46+t^44+t^43+t^41+t^40+t^38+t^37+t^36+t^35+t^32+t^31+t^28+t^23+t^22+t^21+t^20+t^18+t^16+t^15+t^14+t^12+t^11+t^10+t^9+t^7+t^6+t^5+t^4+t^2+t)*x2^3*x4^1) + ((t^51+t^50+t^46+t^44+t^41+t^39+t^37+t^35+t^29+t^28+t^26+t^25+t^24+t^23+t^21+t^19+t^18+t^17+t^15+t^14+t^10+t^9+t^8+t^7+t^6+t^3+t+1)*x1^1*x3^2*x4^1) + ((t^49+t^48+t^43+t^41+t^40+t^35+t^34+t^31+t^27+t^25+t^23+t^22+t^21+t^20+t^18+t^16+t^15+t^13+t^12+t^9+t^8+t^7+t^6+t^4+t^2+t)*x3^3*x4^1) + ((t^51+t^50+t^49+t^48+t^46+t^44+t^43+t^35+t^30+t^29+t^25+t^24+t^23+t^22+t^21+t^19+t^18+t^17+t^16+t^10+t^8+t^7+t^5+t^4+t^3+t^2)*x0^1*x2^1*x4^2) + ((t^51+t^48+t^47+t^43+t^42+t^40+t^39+t^37+t^35+t^31+t^29+t^27+t^26+t^25+t^23+t^20+t^19+t^17+t^16+t^15+t^12+t^8+t^6+t^3+t+1)*x2^2*x4^2) + ((t^51+t^50+t^48+t^47+t^45+t^44+t^43+t^39+t^37+t^36+t^33+t^32+t^31+t^29+t^25+t^23+t^22+t^19+t^16+t^14+t^12+t^11+t^8+t^6+t^5+t^2)*x0^1*x4^3) + ((t^52+t^51+t^47+t^45+t^43+t^42+t^37+t^35+t^34+t^31+t^27+t^26+t^24+t^23+t^19+t^16+t^15+t^13+t^11+t^8+t^7+t^5+t^4)*x0^1*x1^1*x2^1) + ((t^52+t^51+t^50+t^46+t^44+t^42+t^39+t^38+t^37+t^36+t^24+t^23+t^22+t^21+t^20+t^19+t^17+t^16+t^10+t^9+t^6+t^3+t^2+t+1)*x1^2*x3^1) + ((t^52+t^48+t^47+t^45+t^40+t^39+t^36+t^34+t^32+t^30+t^26+t^24+t^23+t^22+t^21+t^20+t^19+t^18+t^16+t^15+t^13+t^10+t^9+t^7+t^5+t^4+t)*x1^1*x3^2) + ((t^52+t^50+t^46+t^45+t^39+t^37+t^34+t^32+t^31+t^30+t^28+t^26+t^24+t^21+t^18+t^17+t^14+t^12+t^11+t^6+t^5+t^3+t^2+1)*x0^1*x2^1*x4^1) + ((t^52+t^50+t^48+t^47+t^46+t^45+t^44+t^41+t^39+t^38+t^37+t^35+t^33+t^31+t^28+t^27+t^26+t^24+t^23+t^22+t^21+t^18+t^17+t^16+t^15+t^12+t^11+t^10+t^4+t^2+t+1)*x0^1*x1^1) + ((t^51+t^50+t^49+t^48+t^46+t^43+t^42+t^41+t^39+t^38+t^37+t^32+t^28+t^27+t^26+t^24+t^22+t^21+t^19+t^18+t^15+t^12+t^11+t^10+t^9+t^6+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x1^1*x3^2*x4^2) + ((t^50+t^49+t^47+t^40+t^39+t^38+t^35+t^34+t^33+t^31+t^28+t^26+t^24+t^23+t^21+t^20+t^19+t^18+t^14+t^13+t^11+t^10+t^8+t^6+t^3+t^2+t+1)*x2^3*x4^1) + ((t^52+t^49+t^45+t^44+t^43+t^42+t^41+t^38+t^35+t^34+t^33+t^30+t^27+t^26+t^22+t^20+t^19+t^18+t^16+t^13+t^12+t^11+t^10+t^9+t^5+t)*x1^2*x3^1*x4^1) + ((t^52+t^51+t^48+t^46+t^45+t^41+t^39+t^37+t^35+t^31+t^28+t^25+t^20+t^19+t^17+t^15+t^10+t^7+t^6+t^5+t^4+t)*x1^1*x3^2*x4^1) + ((t^51+t^49+t^47+t^46+t^45+t^43+t^42+t^40+t^37+t^36+t^32+t^29+t^28+t^27+t^26+t^25+t^24+t^23+t^22+t^21+t^20+t^18+t^17+t^16+t^14+t^12+t^11+t^10+t^9+t^7+t^6+t^5+t^3)*x3^3*x4^1) + ((t^51+t^49+t^48+t^45+t^42+t^41+t^39+t^37+t^35+t^32+t^31+t^30+t^29+t^25+t^22+t^21+t^18+t^17+t^16+t^13+t^8+t^5+t^4+t^3+t+1)*x0^1*x2^1*x4^2) + ((t^52+t^48+t^47+t^45+t^44+t^43+t^41+t^40+t^39+t^38+t^32+t^24+t^22+t^20+t^19+t^18+t^17+t^15+t^12+t^11+t^9+t^8+t^5+t^3)*x2^2*x4^2) + ((t^52+t^50+t^49+t^46+t^45+t^42+t^39+t^38+t^36+t^35+t^30+t^29+t^28+t^23+t^22+t^20+t^19+t^15+t^13+t^10+t^6+t^5+t+1)*x0^1*x4^3) + ((t^52+t^47+t^45+t^41+t^40+t^38+t^37+t^31+t^29+t^26+t^24+t^23+t^22+t^19+t^16+t^15+t^13+t^12+t^9+t^8+t+1)*x0^1*x1^1*x2^1) + ((t^51+t^50+t^49+t^48+t^45+t^44+t^43+t^41+t^38+t^37+t^34+t^33+t^32+t^30+t^27+t^26+t^23+t^22+t^20+t^19+t^18+t^17+t^16+t^12+t^6+t^5+t^4+1)*x1^2*x3^1) + ((t^49+t^48+t^47+t^46+t^43+t^42+t^39+t^30+t^29+t^23+t^22+t^21+t^19+t^18+t^17+t^16+t^14+t^13+t^10+t^9+t^8+t^7+t^5+t^2+1)*x1^1*x3^2) + ((t^51+t^50+t^44+t^43+t^42+t^41+t^35+t^34+t^32+t^31+t^30+t^27+t^25+t^24+t^23+t^21+t^18+t^17+t^16+t^12+t^11+t^8+t^5+t^4+t+1)*x0^1*x1^1*x4^1) + ((t^52+t^51+t^47+t^46+t^45+t^42+t^40+t^38+t^37+t^33+t^32+t^29+t^27+t^25+t^20+t^19+t^17+t^16+t^11+t^10+t^7+t^5+t^3+1)*x0^1*x2^1*x4^1) + ((t^52+t^49+t^48+t^47+t^46+t^43+t^42+t^41+t^40+t^38+t^36+t^35+t^34+t^31+t^30+t^27+t^26+t^22+t^19+t^17+t^14+t^12+t^11+t^10+t^7+t^4+t^3+t^2+t+1)*x1^1*x3^1*x4^1) + ((t^52+t^50+t^49+t^47+t^45+t^44+t^42+t^36+t^32+t^31+t^29+t^28+t^25+t^24+t^23+t^22+t^17+t^16+t^15+t^13+t^10+t^9+t^8+t^6+1)*x0^1*x1^1) + ((t^51+t^48+t^42+t^40+t^37+t^33+t^31+t^30+t^26+t^25+t^24+t^14+t^12+t^9+t^8+t^2)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x3^3*x4^2)");
    groebnerBasis.push_back("((+1)*x0^2*x4^3)");
    groebnerBasis.push_back("((+1)*x0^1*x1^1*x4^3) + ((t^50+t^49+t^48+t^45+t^44+t^41+t^40+t^39+t^38+t^36+t^35+t^32+t^31+t^30+t^29+t^27+t^26+t^25+t^24+t^22+t^21+t^20+t^19+t^18+t^17+t^16+t^13+t^12+t^10+t^9+t^8+t^7+t^5+t^4+t^3+t^2+t)*x2^3*x4^1) + ((t^51+t^50+t^47+t^44+t^41+t^40+t^39+t^38+t^37+t^36+t^30+t^29+t^28+t^27+t^25+t^24+t^21+t^20+t^19+t^15+t^14+t^10+t^9+t^8+t^6+t^5+t^4+t^3)*x1^1*x3^2*x4^1) + ((t^51+t^50+t^49+t^46+t^44+t^43+t^42+t^41+t^40+t^25+t^24+t^23+t^15+t^12+t^8+t^6+t^5+t^4+t)*x3^3*x4^1) + ((t^51+t^47+t^46+t^44+t^43+t^40+t^39+t^33+t^32+t^31+t^30+t^29+t^28+t^25+t^23+t^22+t^21+t^20+t^17+t^16+t^15+t^12+t^8+t^6+t^4+t^2+t+1)*x0^1*x2^1*x4^2) + ((t^52+t^51+t^50+t^46+t^44+t^42+t^41+t^38+t^35+t^34+t^33+t^32+t^30+t^29+t^28+t^26+t^24+t^22+t^20+t^19+t^18+t^10+t^9+t^8+t^7+t^4+t+1)*x2^2*x4^2) + ((t^52+t^51+t^48+t^45+t^44+t^43+t^41+t^40+t^39+t^38+t^32+t^29+t^28+t^26+t^24+t^21+t^18+t^16+t^15+t^14+t^11+t^9+t^7+t^6+t^4+t^3+t)*x0^1*x4^3) + ((t^48+t^45+t^44+t^42+t^41+t^40+t^37+t^35+t^33+t^29+t^27+t^24+t^22+t^21+t^20+t^19+t^17+t^16+t^15+t^14+t^13+t^11+t^8+t^7+t^6+t^4)*x0^1*x1^1*x2^1) + ((t^52+t^50+t^47+t^44+t^43+t^42+t^41+t^38+t^34+t^33+t^32+t^29+t^28+t^26+t^24+t^23+t^22+t^20+t^18+t^17+t^15+t^14+t^13+t^11+t^9+t^7+t^6+t^5+t^4+t^2+t+1)*x1^2*x3^1) + ((t^51+t^50+t^47+t^44+t^36+t^34+t^33+t^31+t^30+t^29+t^27+t^25+t^24+t^21+t^19+t^16+t^15+t^12+t^11+t^10+t^8+t^6+t^4+t^3)*x1^1*x3^2) + ((t^50+t^48+t^47+t^46+t^45+t^44+t^41+t^40+t^39+t^36+t^30+t^27+t^26+t^25+t^22+t^21+t^19+t^16+t^13+t^12+t^6+t^5+t^4+t)*x0^1*x2^1*x4^1) + ((t^52+t^51+t^48+t^47+t^44+t^43+t^41+t^40+t^39+t^38+t^37+t^35+t^34+t^33+t^29+t^24+t^23+t^22+t^20+t^15+t^14+t^12+t^9+t^6+t^4+t+1)*x0^1*x1^1) + ((t^52+t^51+t^48+t^47+t^46+t^45+t^44+t^42+t^41+t^39+t^38+t^37+t^36+t^31+t^30+t^29+t^27+t^23+t^20+t^14+t^13+t^12+t^11+t^9+t^8+t^7+t^2+t+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x0^1*x2^1*x4^3) + ((t^52+t^51+t^47+t^46+t^45+t^43+t^42+t^41+t^29+t^27+t^26+t^22+t^21+t^16+t^13+t^12+t^10+t^8+t^7+t^4+t^3+1)*x2^3*x4^1) + ((t^52+t^51+t^50+t^48+t^46+t^45+t^44+t^41+t^40+t^38+t^37+t^36+t^34+t^33+t^32+t^30+t^29+t^23+t^18+t^17+t^15+t^14+t^13+t^12+t^8+t^7+t+1)*x0^1*x2^1*x4^2)");
    groebnerBasis.push_back("((+1)*x2^2*x4^3) + ((t^51+t^50+t^48+t^45+t^44+t^43+t^39+t^32+t^28+t^25+t^23+t^20+t^19+t^12+t^5+t^4+t^3)*x2^3*x4^1) + ((t^52+t^48+t^43+t^41+t^38+t^35+t^33+t^32+t^31+t^30+t^28+t^25+t^24+t^20+t^17+t^14+t^12+t^11+t^10+t^9+t^8+t^6+t^3+t+1)*x1^1*x3^2*x4^1) + ((t^51+t^49+t^46+t^45+t^44+t^43+t^42+t^41+t^39+t^37+t^36+t^35+t^33+t^32+t^26+t^20+t^19+t^18+t^17+t^15+t^7+t^5+t^4+t^2+t)*x3^3*x4^1) + ((t^52+t^51+t^49+t^47+t^46+t^45+t^44+t^43+t^42+t^41+t^40+t^38+t^37+t^34+t^32+t^28+t^27+t^25+t^21+t^18+t^17+t^16+t^15+t^9+t^7+t^6+t^3+1)*x0^1*x2^1*x4^2) + ((t^51+t^49+t^47+t^45+t^44+t^41+t^39+t^34+t^33+t^32+t^31+t^30+t^27+t^21+t^20+t^19+t^17+t^13+t^9+t^7+t^5+t^4+t+1)*x2^2*x4^2) + ((t^51+t^50+t^48+t^46+t^43+t^40+t^36+t^35+t^34+t^32+t^27+t^24+t^23+t^22+t^17+t^14+t^12+t^11+t^10+t^9+t^7+t^6+t^5+t+1)*x0^1*x4^3) + ((t^50+t^49+t^47+t^46+t^44+t^43+t^42+t^40+t^38+t^35+t^34+t^33+t^32+t^31+t^29+t^27+t^25+t^24+t^23+t^20+t^17+t^16+t^10+t^9+t^5+t^4+t)*x0^1*x1^1*x2^1) + ((t^50+t^49+t^47+t^43+t^40+t^39+t^35+t^28+t^26+t^24+t^22+t^21+t^19+t^16+t^15+t^14+t^12+t^11+t^10+t^7+t^6+t^4+t^3+t)*x1^2*x3^1) + ((t^51+t^50+t^48+t^42+t^40+t^39+t^35+t^34+t^33+t^31+t^30+t^26+t^25+t^24+t^22+t^21+t^20+t^19+t^16+t^15+t^10+t^9+t^8+t^5+t^4+t^3+t^2)*x1^1*x3^2) + ((t^52+t^49+t^47+t^44+t^38+t^37+t^34+t^29+t^28+t^25+t^23+t^22+t^21+t^19+t^18+t^17+t^16+t^14+t^10+t^9+t^3+1)*x0^1*x2^1*x4^1) + ((t^50+t^48+t^44+t^41+t^39+t^36+t^35+t^34+t^30+t^28+t^23+t^22+t^21+t^20+t^18+t^17+t^16+t^13+t^12+t^10+t^9+t^8+t^7+t^6+t^5+t^3+1)*x0^1*x1^1) + ((t^52+t^50+t^49+t^48+t^47+t^45+t^43+t^38+t^36+t^35+t^31+t^30+t^27+t^26+t^25+t^23+t^22+t^21+t^20+t^18+t^17+t^16+t^14+t^11+t^8+t^6+t^5+t^4+t^3+t^2+t+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x1^1*x3^1*x4^3) + ((t^51+t^46+t^45+t^44+t^42+t^41+t^38+t^36+t^33+t^30+t^29+t^28+t^27+t^24+t^22+t^20+t^19+t^17+t^16+t^13+t^9+t^4+t^3+t^2)*x2^3*x4^1) + ((t^52+t^51+t^50+t^47+t^46+t^45+t^43+t^41+t^40+t^39+t^38+t^37+t^35+t^33+t^32+t^30+t^29+t^26+t^23+t^22+t^20+t^18+t^17+t^16+t^13+t^11+t^10+t^9+t^8+t^7+t^6+1)*x1^1*x3^2*x4^1) + ((t^46+t^43+t^40+t^39+t^38+t^37+t^36+t^35+t^33+t^32+t^31+t^29+t^28+t^24+t^23+t^22+t^18+t^17+t^16+t^15+t^14+t^13+t^10+t^9+t^7+t^5+t^4+t+1)*x3^3*x4^1) + ((t^52+t^51+t^50+t^48+t^47+t^46+t^44+t^43+t^41+t^40+t^39+t^38+t^36+t^35+t^34+t^31+t^28+t^26+t^22+t^21+t^19+t^18+t^17+t^13+t^12+t^10+t^9+t^8+t^7+t^6+t^4+t^3+t)*x0^1*x2^1*x4^2) + ((t^52+t^48+t^47+t^43+t^42+t^36+t^35+t^34+t^33+t^31+t^30+t^29+t^28+t^27+t^26+t^21+t^17+t^14+t^12+t^10+t^7+t^6+t^5+t^4+t^3+t^2+t+1)*x2^2*x4^2) + ((t^50+t^49+t^48+t^47+t^45+t^40+t^39+t^38+t^37+t^36+t^34+t^33+t^32+t^31+t^30+t^27+t^26+t^25+t^23+t^22+t^19+t^18+t^17+t^16+t^15+t^14+t^12+t^9+t^7+t^6+t^5+t^4+t^2+1)*x0^1*x4^3) + ((t^52+t^49+t^48+t^46+t^45+t^44+t^43+t^42+t^41+t^38+t^36+t^33+t^32+t^31+t^30+t^29+t^28+t^26+t^25+t^23+t^22+t^21+t^19+t^17+t^16+t^15+t^12+t^5+t^4+t^3+t^2+t)*x0^1*x1^1*x2^1) + ((t^51+t^48+t^46+t^45+t^44+t^40+t^37+t^35+t^31+t^29+t^27+t^26+t^22+t^21+t^17+t^15+t^13+t^12+t^11+t^8+t^6+t^2+t)*x1^2*x3^1) + ((t^51+t^46+t^45+t^44+t^40+t^38+t^36+t^31+t^25+t^23+t^21+t^15+t^11+t^10+t^8+t^2+t+1)*x1^1*x3^2) + ((t^50+t^48+t^47+t^46+t^45+t^41+t^40+t^39+t^38+t^36+t^34+t^31+t^28+t^27+t^23+t^21+t^18+t^16+t^11+t^10+t^9+t^6+t^4+t^2)*x0^1*x2^1*x4^1) + ((t^50+t^47+t^45+t^44+t^41+t^40+t^34+t^32+t^30+t^29+t^28+t^26+t^25+t^24+t^21+t^18+t^16+t^15+t^12+t^8+t^6+t^5+t^4+t^3+t^2+1)*x0^1*x1^1) + ((t^52+t^51+t^48+t^47+t^44+t^43+t^41+t^40+t^39+t^38+t^37+t^35+t^34+t^33+t^29+t^24+t^23+t^22+t^20+t^15+t^14+t^12+t^9+t^6+t^4+t+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x2^1*x3^1*x4^3) + ((t^51+t^49+t^48+t^47+t^46+t^45+t^44+t^41+t^37+t^36+t^25+t^24+t^22+t^21+t^20+t^17+t^14+t^12+t^11+t^7+t^5+t^2+1)*x0^1*x2^2*x4^1) + ((t^51+t^47+t^46+t^44+t^40+t^39+t^38+t^37+t^33+t^30+t^27+t^26+t^24+t^21+t^20+t^19+t^16+t^13+t^12+t^10+t^9+t^8+t^7+t^3+t^2)*x2^3*x4^1) + ((t^51+t^50+t^49+t^44+t^43+t^41+t^40+t^38+t^36+t^35+t^32+t^31+t^30+t^29+t^28+t^26+t^25+t^24+t^21+t^20+t^19+t^13+t^12+t^11+t^10+t^7+t^6+t^2)*x1^1*x3^2*x4^1) + ((t^49+t^48+t^46+t^44+t^43+t^42+t^38+t^35+t^33+t^32+t^30+t^28+t^26+t^25+t^24+t^23+t^22+t^21+t^20+t^19+t^11+t^8+t^7+t^5+t^3+t^2+t)*x3^3*x4^1) + ((t^52+t^51+t^50+t^49+t^48+t^46+t^44+t^43+t^42+t^41+t^40+t^38+t^34+t^33+t^32+t^31+t^29+t^28+t^25+t^24+t^23+t^16+t^14+t^12+t^11+t^10+t^8+t^7+t^4+t^2+t)*x0^1*x2^1*x4^2) + ((t^52+t^50+t^48+t^44+t^43+t^42+t^40+t^37+t^36+t^32+t^29+t^28+t^27+t^23+t^21+t^18+t^17+t^15+t^14+t^13+t^11+t^9+t^8+t^7+t^5+t^3+1)*x2^2*x4^2) + ((t^51+t^49+t^47+t^45+t^44+t^42+t^40+t^37+t^35+t^34+t^33+t^32+t^31+t^30+t^29+t^28+t^27+t^25+t^24+t^19+t^18+t^14+t^13+t^12+t^10+t^8+t^6+t^4+t^3+t+1)*x0^1*x4^3) + ((t^51+t^50+t^49+t^47+t^44+t^42+t^41+t^39+t^37+t^35+t^33+t^32+t^31+t^29+t^26+t^24+t^23+t^22+t^21+t^20+t^19+t^18+t^13+t^12+t^11+t^10+t^7+t^6+t^2+t)*x0^1*x1^1*x2^1) + ((t^52+t^51+t^49+t^47+t^46+t^44+t^42+t^41+t^37+t^32+t^29+t^28+t^27+t^26+t^25+t^22+t^21+t^19+t^10+t^9+t^8+t^7+t^4+1)*x1^2*x3^1) + ((t^52+t^46+t^45+t^44+t^40+t^39+t^38+t^34+t^32+t^31+t^30+t^29+t^27+t^24+t^23+t^22+t^20+t^16+t^15+t^12+t^11+t^9+t^7+t^4+t^3+1)*x1^1*x3^2) + ((t^49+t^45+t^44+t^43+t^37+t^33+t^31+t^30+t^29+t^28+t^26+t^22+t^21+t^20+t^16+t^15+t^14+t^13+t^12+t^11+t^9+t^6+t^2+t+1)*x0^1*x2^1*x4^1) + ((t^51+t^49+t^48+t^45+t^44+t^43+t^42+t^41+t^40+t^39+t^38+t^36+t^35+t^34+t^32+t^31+t^30+t^27+t^25+t^24+t^23+t^21+t^16+t^12+t^10+t^7+t^2+1)*x0^1*x1^1) + ((t^51+t^50+t^46+t^44+t^43+t^42+t^39+t^35+t^26+t^24+t^23+t^22+t^20+t^19+t^18+t^17+t^15+t^14+t^13+t^12+t^11+t^10+t^8+t^4+t^2+t+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x3^2*x4^3)");
    groebnerBasis.push_back("((+1)*x0^1*x4^4) + ((t^50+t^49+t^48+t^44+t^43+t^41+t^40+t^38+t^35+t^33+t^32+t^31+t^24+t^23+t^20+t^19+t^11+t^8+t^7+t^6+t^5+t^4+t^3+t)*x2^3*x4^1) + ((t^52+t^50+t^48+t^47+t^45+t^44+t^42+t^38+t^35+t^33+t^32+t^24+t^23+t^22+t^20+t^19+t^16+t^15+t^13+t^12+t^10+t^6+t^5+t^3+t^2)*x1^1*x3^2*x4^1) + ((t^52+t^48+t^47+t^45+t^42+t^40+t^36+t^33+t^31+t^30+t^29+t^27+t^26+t^23+t^17+t^16+t^15+t^14+t^13+t^11+t^10+1)*x3^3*x4^1) + ((t^51+t^49+t^44+t^37+t^35+t^34+t^33+t^32+t^31+t^30+t^26+t^25+t^22+t^20+t^17+t^15+t^12+t^10+t^9+t^8+t^7+t^4+t^3+t)*x0^1*x2^1*x4^2) + ((t^50+t^47+t^46+t^44+t^41+t^40+t^38+t^34+t^30+t^29+t^28+t^26+t^25+t^22+t^19+t^18+t^17+t^13+t^12+t^11+t^7+t^6+t^5+t^4+t^2)*x2^2*x4^2) + ((t^51+t^49+t^47+t^45+t^44+t^41+t^39+t^34+t^33+t^32+t^31+t^30+t^27+t^21+t^20+t^19+t^17+t^13+t^9+t^7+t^5+t^4+t+1)*x0^1*x4^3) + ((t^52+t^50+t^49+t^48+t^45+t^41+t^39+t^38+t^36+t^29+t^24+t^23+t^21+t^20+t^19+t^17+t^16+t^15+t^13+t^12+t^10+t^8+t^6+t^4+t^3+t+1)*x0^1*x1^1*x2^1) + ((t^50+t^49+t^48+t^47+t^46+t^44+t^43+t^40+t^39+t^38+t^36+t^35+t^34+t^29+t^26+t^24+t^21+t^20+t^19+t^16+t^12+t^7+t^4+t^3+t^2)*x1^2*x3^1) + ((t^51+t^48+t^44+t^42+t^41+t^40+t^37+t^35+t^34+t^31+t^29+t^28+t^24+t^21+t^20+t^15+t^12+t^10+t^6+t^5+t^3+t^2+t+1)*x1^1*x3^2) + ((t^48+t^45+t^44+t^42+t^41+t^40+t^37+t^35+t^33+t^29+t^27+t^24+t^22+t^21+t^20+t^19+t^17+t^16+t^15+t^14+t^13+t^11+t^8+t^7+t^6+t^4)*x0^1*x2^1*x4^1) + ((t^52+t^48+t^46+t^44+t^41+t^36+t^33+t^29+t^24+t^23+t^21+t^18+t^16+t^15+t^14+t^13+t^9+t^7+t^6+t^4+1)*x0^1*x1^1) + ((t^49+t^48+t^47+t^45+t^43+t^42+t^41+t^39+t^38+t^37+t^36+t^34+t^33+t^32+t^30+t^27+t^25+t^24+t^23+t^22+t^21+t^20+t^19+t^18+t^14+t^11+t^9+t^7+t^6+t^3+t^2)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x1^1*x4^4) + ((t^51+t^48+t^47+t^46+t^44+t^42+t^41+t^40+t^39+t^38+t^36+t^35+t^33+t^32+t^30+t^29+t^28+t^25+t^24+t^22+t^21+t^19+t^18+t^16+t^15+t^13+t^12+t^11+t^5+1)*x2^3*x4^1) + ((t^52+t^51+t^50+t^47+t^46+t^44+t^43+t^41+t^40+t^38+t^37+t^36+t^35+t^34+t^33+t^32+t^31+t^27+t^26+t^25+t^24+t^20+t^16+t^15+t^14+t^11+t^10+t^9+t^8+t^5+t+1)*x1^1*x3^2*x4^1) + ((t^51+t^44+t^43+t^42+t^37+t^34+t^33+t^31+t^30+t^29+t^28+t^25+t^24+t^18+t^14+t^8+t^5+t^4+t^3+t^2+t+1)*x3^3*x4^1) + ((t^52+t^50+t^48+t^47+t^44+t^42+t^39+t^38+t^37+t^35+t^34+t^24+t^23+t^20+t^19+t^12+t^8+t^4+t^3+t^2+t+1)*x0^1*x2^1*x4^2) + ((t^52+t^51+t^48+t^47+t^46+t^45+t^41+t^38+t^36+t^35+t^34+t^33+t^32+t^30+t^29+t^24+t^22+t^17+t^14+t^13+t^12+t^8+t^7+t^4+t^3)*x2^2*x4^2) + ((t^51+t^50+t^49+t^48+t^46+t^45+t^44+t^40+t^33+t^32+t^30+t^29+t^28+t^27+t^24+t^21+t^20+t^19+t^18+t^16+t^15+t^11+t^10+t^9+t^8+t^7+t^6+t^3+t^2)*x0^1*x4^3) + ((t^48+t^46+t^43+t^41+t^40+t^38+t^29+t^25+t^24+t^22+t^19+t^17+t^16+t^10+t^9+t^7+t^6+t^5+t^4)*x0^1*x1^1*x2^1) + ((t^51+t^50+t^49+t^48+t^47+t^46+t^44+t^42+t^39+t^38+t^37+t^36+t^34+t^33+t^31+t^30+t^27+t^24+t^23+t^17+t^16+t^13+t^11+t^10+t^9+t^8+t^3+t^2+1)*x1^2*x3^1) + ((t^51+t^49+t^47+t^44+t^42+t^41+t^39+t^34+t^33+t^31+t^30+t^23+t^19+t^16+t^14+t^12+t^11+t^9+t^6+t^4+t^3+t^2+1)*x1^1*x3^2) + ((t^50+t^49+t^48+t^47+t^46+t^45+t^43+t^40+t^39+t^38+t^35+t^33+t^32+t^31+t^30+t^29+t^26+t^25+t^24+t^23+t^19+t^17+t^15+t^14+t^11+t^9+t^7+t^6+t^5+t^3+t^2+t+1)*x0^1*x2^1*x4^1) + ((t^52+t^50+t^43+t^40+t^37+t^36+t^35+t^33+t^30+t^28+t^21+t^19+t^18+t^16+t^14+t^13+t^10+t^8+t^3+t)*x0^1*x1^1) + ((t^52+t^51+t^49+t^48+t^46+t^40+t^37+t^35+t^34+t^31+t^28+t^25+t^23+t^22+t^21+t^19+t^18+t^17+t^14+t^13+t^12+t^10+t^8+t^5+t^3+t^2)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x2^1*x4^4) + ((t^50+t^43+t^41+t^40+t^39+t^37+t^33+t^31+t^29+t^27+t^22+t^20+t^19+t^18+t^16+t^15+t^11+t^10+t^8+t^6+t^2+t+1)*x2^3*x4^1) + ((t^51+t^50+t^48+t^39+t^37+t^34+t^33+t^32+t^31+t^30+t^29+t^28+t^24+t^23+t^22+t^19+t^14+t^13+t^11+t^10+t^9+t^8+t^5+t^3+t^2+1)*x0^1*x2^1*x4^2)");
    groebnerBasis.push_back("((+1)*x3^1*x4^4) + ((t^50+t^48+t^46+t^43+t^42+t^40+t^38+t^37+t^35+t^33+t^31+t^30+t^28+t^27+t^23+t^22+t^20+t^19+t^18+t^14+t^13+t^12+t^11+t^9+t^6+t^5+t)*x2^3*x4^1) + ((t^52+t^51+t^48+t^47+t^45+t^44+t^43+t^42+t^41+t^39+t^38+t^37+t^35+t^33+t^31+t^30+t^28+t^23+t^21+t^20+t^19+t^17+t^15+t^13+t^12+t^10+t^8+t^5+t^4+t^3)*x1^1*x3^2*x4^1) + ((t^50+t^49+t^44+t^37+t^36+t^34+t^32+t^31+t^30+t^27+t^26+t^22+t^21+t^19+t^17+t^16+t^15+t^12+t^11+t^8+t^7+t^6+t^4+t^3+t^2)*x3^3*x4^1) + ((t^51+t^50+t^48+t^45+t^41+t^38+t^37+t^35+t^33+t^32+t^28+t^27+t^25+t^22+t^21+t^19+t^18+t^17+t^16+t^15+t^14+t^12+t^10+t^3+t^2+t)*x0^1*x2^1*x4^2) + ((t^52+t^50+t^49+t^48+t^46+t^45+t^44+t^43+t^42+t^41+t^40+t^39+t^38+t^37+t^36+t^34+t^33+t^32+t^31+t^29+t^27+t^26+t^23+t^21+t^20+t^19+t^18+t^16+t^13+t^12+t^11+t^10+t^9+t^8+t^7+t^4)*x2^2*x4^2) + ((t^52+t^51+t^50+t^47+t^45+t^44+t^41+t^38+t^37+t^35+t^33+t^32+t^31+t^30+t^24+t^22+t^21+t^20+t^19+t^18+t^17+t^15+t^14+t^8+t^6+t^5+t+1)*x0^1*x4^3) + ((t^51+t^49+t^45+t^44+t^43+t^42+t^41+t^40+t^38+t^34+t^30+t^29+t^28+t^27+t^25+t^22+t^21+t^20+t^18+t^17+t^16+t^13+t^12+t^11+t^10+t^9+t^7+t^6+t^3+1)*x0^1*x1^1*x2^1) + ((t^51+t^47+t^45+t^44+t^40+t^39+t^38+t^37+t^36+t^35+t^34+t^30+t^23+t^21+t^18+t^17+t^16+t^15+t^13+t^12+t^10+t^7+t^3)*x1^2*x3^1) + ((t^52+t^51+t^50+t^48+t^46+t^43+t^42+t^39+t^36+t^35+t^34+t^33+t^31+t^30+t^28+t^27+t^26+t^23+t^22+t^21+t^19+t^17+t^13+t^10+t^9+t^6+t^4+1)*x1^1*x3^2) + ((t^52+t^49+t^48+t^46+t^45+t^44+t^43+t^42+t^41+t^38+t^36+t^33+t^32+t^31+t^30+t^29+t^28+t^26+t^25+t^23+t^22+t^21+t^19+t^17+t^16+t^15+t^12+t^5+t^4+t^3+t^2+t)*x0^1*x2^1*x4^1) + ((t^51+t^40+t^38+t^37+t^36+t^35+t^34+t^33+t^32+t^31+t^30+t^29+t^28+t^26+t^25+t^24+t^23+t^21+t^20+t^19+t^17+t^15+t^13+t^12+t^3+t^2+t+1)*x0^1*x1^1) + ((t^52+t^48+t^46+t^44+t^41+t^36+t^33+t^29+t^24+t^23+t^21+t^18+t^16+t^15+t^14+t^13+t^9+t^7+t^6+t^4+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x4^5) + ((t^52+t^50+t^49+t^48+t^47+t^44+t^41+t^40+t^39+t^38+t^37+t^33+t^31+t^28+t^27+t^26+t^25+t^24+t^23+t^21+t^20+t^17+t^15+t^10+t^8+t^7+t^4+t^3+t)*x2^3*x4^1) + ((t^52+t^51+t^50+t^49+t^48+t^47+t^42+t^41+t^37+t^36+t^35+t^33+t^30+t^28+t^26+t^25+t^22+t^19+t^16+t^15+t^14+t^12+t^10+t^9+t^8+t^7+t^5+t^4+t^3+1)*x1^1*x3^2*x4^1) + ((t^52+t^50+t^49+t^47+t^41+t^39+t^38+t^36+t^35+t^33+t^32+t^31+t^29+t^28+t^26+t^25+t^21+t^20+t^19+t^15+t^9+t^2+t)*x3^3*x4^1) + ((t^52+t^51+t^50+t^47+t^46+t^44+t^42+t^41+t^38+t^36+t^35+t^33+t^30+t^29+t^27+t^26+t^24+t^21+t^20+t^17+t^15+t^13+t^12+t^9+t^7+t^5+t)*x0^1*x2^1*x4^2) + ((t^52+t^49+t^47+t^41+t^40+t^38+t^37+t^36+t^35+t^30+t^29+t^27+t^26+t^25+t^22+t^21+t^15+t^13+t^10+t^6+1)*x2^2*x4^2) + ((t^52+t^51+t^50+t^49+t^46+t^45+t^44+t^43+t^41+t^39+t^37+t^26+t^23+t^22+t^19+t^18+t^17+t^15+t^11+t^9+t^8+t^7+t^5+t^4+t^2+t+1)*x0^1*x4^3) + ((t^51+t^47+t^42+t^41+t^38+t^37+t^31+t^30+t^29+t^28+t^26+t^25+t^23+t^21+t^20+t^17+t^16+t^14+t^11+t^10+t^7+t^5+t^4+t^3+1)*x0^1*x1^1*x2^1) + ((t^52+t^49+t^47+t^45+t^44+t^41+t^38+t^37+t^36+t^29+t^27+t^26+t^25+t^22+t^19+t^18+t^17+t^12+t^10+t^9+t^7+t^4+t^3+t)*x1^2*x3^1) + ((t^52+t^47+t^44+t^43+t^41+t^38+t^37+t^36+t^33+t^32+t^30+t^26+t^25+t^24+t^23+t^22+t^21+t^19+t^18+t^17+t^15+t^11+t^10+t^9+t^6+t^4+t^3+t+1)*x1^1*x3^2) + ((t^48+t^46+t^43+t^41+t^40+t^38+t^29+t^25+t^24+t^22+t^19+t^17+t^16+t^10+t^9+t^7+t^6+t^5+t^4)*x0^1*x2^1*x4^1) + ((t^52+t^50+t^49+t^47+t^39+t^34+t^33+t^26+t^25+t^24+t^22+t^17+t^14+t^13+t^9+t^8+t^6+t^2+t)*x0^1*x1^1) + ((t^52+t^49+t^46+t^45+t^44+t^43+t^37+t^34+t^33+t^32+t^29+t^25+t^23+t^22+t^21+t^20+t^16+t^12+t^11+t^10+t^5+t^4+t^3+t^2+t)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x0^3*x1^1) + ((t^51+t^50+t^48+t^46+t^43+t^41+t^40+t^38+t^36+t^30+t^27+t^26+t^24+t^22+t^21+t^16+t^15+t^14+t^13+t^11+t^10+t^7+t^6+t^5+t^3+t)*x0^1*x2^2*x4^1) + ((t^52+t^51+t^48+t^44+t^42+t^39+t^37+t^36+t^35+t^33+t^31+t^30+t^29+t^28+t^26+t^23+t^22+t^19+t^17+t^16+t^15+t^14+t^12+t^9+t^5+t^3+t^2+t)*x2^3*x4^1) + ((t^51+t^50+t^48+t^47+t^46+t^45+t^44+t^41+t^38+t^35+t^33+t^32+t^31+t^28+t^26+t^23+t^22+t^21+t^14+t^12+t^8+t^4+t^2+t)*x1^2*x3^1*x4^1) + ((t^47+t^44+t^43+t^37+t^36+t^35+t^34+t^31+t^29+t^24+t^22+t^21+t^17+t^16+t^14+t^12+t^11+t^9+t^8+t^7+t^5+t^4+t^3)*x2^2*x3^1*x4^1) + ((t^52+t^51+t^50+t^49+t^41+t^39+t^36+t^34+t^31+t^28+t^19+t^18+t^16+t^9+t^7+t^4+t^3+1)*x1^1*x3^2*x4^1) + ((t^52+t^48+t^47+t^43+t^42+t^41+t^35+t^34+t^32+t^29+t^28+t^25+t^23+t^22+t^20+t^18+t^17+t^13+t^11+t^8+t^5+t^3+t^2)*x3^3*x4^1) + ((t^49+t^48+t^47+t^46+t^42+t^41+t^39+t^37+t^34+t^32+t^27+t^24+t^21+t^19+t^18+t^17+t^16+t^12+t^10+t^9+t^7+t^5+t^3)*x0^2*x4^2) + ((t^52+t^51+t^50+t^49+t^48+t^47+t^46+t^43+t^42+t^41+t^38+t^37+t^36+t^35+t^33+t^30+t^28+t^26+t^25+t^24+t^21+t^15+t^12+t^11+t^10+t^8+t^6+t^3+1)*x0^1*x1^1*x4^2) + ((t^51+t^50+t^49+t^44+t^42+t^41+t^40+t^39+t^38+t^35+t^32+t^30+t^29+t^28+t^22+t^21+t^17+t^16+t^15+t^14+t^13+t^12+t^11+t^10+t^2)*x0^1*x2^1*x4^2) + ((t^51+t^50+t^49+t^48+t^45+t^42+t^41+t^39+t^38+t^36+t^35+t^33+t^32+t^31+t^30+t^29+t^28+t^27+t^25+t^22+t^19+t^17+t^16+t^15+t^11+t^10+t^9+t^7+t^5+t^4+t^2+t+1)*x2^2*x4^2) + ((t^52+t^51+t^49+t^48+t^47+t^46+t^43+t^42+t^41+t^37+t^36+t^35+t^34+t^29+t^26+t^25+t^22+t^21+t^20+t^19+t^18+t^17+t^11+t^9+t^7+t^4+t^2+t+1)*x0^1*x4^3) + ((t^52+t^50+t^46+t^44+t^35+t^33+t^32+t^30+t^29+t^24+t^22+t^21+t^20+t^16+t^14+t^13+t^10+t^8+t^7+t^5+t^3+t+1)*x4^4) + ((t^52+t^51+t^42+t^41+t^38+t^34+t^33+t^32+t^30+t^29+t^28+t^24+t^22+t^21+t^17+t^11+t^10+t^9+t^8+t^6+t^5)*x0^1*x1^1*x2^1) + ((t^52+t^50+t^48+t^47+t^46+t^44+t^43+t^38+t^36+t^28+t^27+t^23+t^22+t^19+t^16+t^14+t^12+t^9+t^8+t^7+t^6+t^5+t^2+t)*x1^2*x3^1) + ((t^52+t^47+t^46+t^45+t^44+t^43+t^40+t^39+t^35+t^33+t^27+t^24+t^22+t^20+t^18+t^14+t^13+t^10+t^9+t^6+t^2)*x0^1*x3^2) + ((t^51+t^49+t^46+t^42+t^39+t^37+t^35+t^30+t^29+t^28+t^25+t^22+t^21+t^20+t^19+t^18+t^14+t^13+t^12+t^8+t^6+t^5+t^2+t+1)*x1^1*x3^2) + ((t^52+t^50+t^47+t^46+t^44+t^41+t^40+t^39+t^36+t^34+t^33+t^31+t^29+t^28+t^23+t^22+t^17+t^16+t^10+t^9+t^8+t^6+t^4+t^3+t^2+t)*x0^1*x1^1*x4^1) + ((t^50+t^49+t^47+t^45+t^43+t^41+t^37+t^34+t^32+t^30+t^28+t^27+t^26+t^22+t^21+t^19+t^17+t^16+t^15+t^13+t^12+t^10+t^9+t^8+t^7+t^5+t^3+t)*x0^1*x2^1*x4^1) + ((t^52+t^51+t^49+t^48+t^46+t^41+t^40+t^39+t^37+t^35+t^34+t^33+t^31+t^30+t^29+t^27+t^25+t^22+t^21+t^19+t^18+t^13+t^12+t^11+t^10+t^9+t^7+t^6+t^2)*x2^2*x4^1) + ((t^49+t^48+t^44+t^42+t^40+t^35+t^34+t^33+t^32+t^31+t^29+t^25+t^23+t^21+t^17+t^16+t^12+t^8+t^2+1)*x1^1*x3^1*x4^1) + ((t^49+t^45+t^44+t^43+t^41+t^40+t^34+t^33+t^29+t^28+t^27+t^26+t^25+t^23+t^21+t^19+t^18+t^12+t^9+t^8+t^7+t^6+t^4+t^3+t^2)*x0^1*x4^2) + ((t^52+t^50+t^49+t^46+t^45+t^43+t^38+t^31+t^28+t^23+t^21+t^19+t^18+t^17+t^16+t^14+t^13+t^12+t^8+t^2+t+1)*x0^1*x1^1) + ((t^52+t^49+t^46+t^43+t^38+t^37+t^31+t^30+t^28+t^27+t^26+t^25+t^23+t^20+t^18+t^17+t^16+t^15+t^14+t^12+t^11+t^9+t^6+t^4+t^3+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x0^1*x1^2*x2^1) + ((t^52+t^51+t^46+t^45+t^42+t^35+t^33+t^32+t^31+t^30+t^28+t^27+t^24+t^18+t^13+t^12+t^9+t^8+t^7+t^6+t^4+1)*x2^3*x4^1) + ((t^52+t^51+t^50+t^48+t^47+t^42+t^40+t^38+t^35+t^34+t^33+t^29+t^27+t^25+t^22+t^20+t^18+t^17+t^13+t^11+t^10+t^9+t^7+t^3+t)*x2^2*x3^1*x4^1) + ((t^49+t^48+t^45+t^44+t^43+t^41+t^40+t^37+t^36+t^35+t^33+t^32+t^30+t^22+t^21+t^20+t^19+t^17+t^16+t^15+t^13+t^12+t^11+t^10+t^9+t^7+t^6+t^5+t+1)*x1^1*x3^2*x4^1) + ((t^51+t^50+t^49+t^47+t^46+t^45+t^43+t^41+t^39+t^38+t^29+t^28+t^22+t^21+t^20+t^16+t^7+t^2+1)*x3^3*x4^1) + ((t^51+t^50+t^45+t^42+t^40+t^38+t^35+t^34+t^30+t^27+t^25+t^24+t^23+t^22+t^21+t^20+t^19+t^18+t^17+t^16+t^13+t^12+t^10+t^8+t^7+t^6+t^4+t^3+t+1)*x0^1*x2^1*x4^2) + ((t^51+t^49+t^48+t^46+t^45+t^43+t^41+t^37+t^35+t^30+t^29+t^27+t^26+t^21+t^19+t^18+t^17+t^16+t^15+t^14+t^13+t^9+t^5+t^4+t+1)*x2^2*x4^2) + ((t^51+t^50+t^49+t^48+t^45+t^42+t^41+t^40+t^39+t^38+t^34+t^33+t^30+t^28+t^26+t^24+t^22+t^20+t^18+t^17+t^16+t^12+t^10+t^7+t^5)*x0^1*x4^3) + ((t^52+t^51+t^50+t^41+t^40+t^39+t^38+t^37+t^35+t^33+t^31+t^30+t^28+t^27+t^26+t^25+t^24+t^21+t^20+t^19+t^14+t^13+t^10+t^9+t^7+t^5+t^4+t^3)*x0^1*x1^1*x2^1) + ((t^52+t^51+t^50+t^49+t^48+t^47+t^42+t^41+t^40+t^39+t^37+t^36+t^31+t^30+t^28+t^26+t^24+t^21+t^18+t^17+t^16+t^14+t^13+t^10+t^7+t^5+t^4)*x1^2*x3^1) + ((t^51+t^48+t^45+t^41+t^40+t^39+t^38+t^37+t^35+t^33+t^30+t^27+t^24+t^23+t^22+t^21+t^20+t^19+t^17+t^16+t^15+t^13+t^11+t^10+t^7+t^5+t^3+t^2+t)*x0^1*x3^2) + ((t^51+t^49+t^48+t^46+t^43+t^41+t^39+t^36+t^34+t^30+t^29+t^26+t^25+t^21+t^19+t^17+t^16+t^15+t^13+t^12+t^8+t^7+t^3+t^2)*x1^1*x3^2) + ((t^52+t^49+t^46+t^45+t^44+t^42+t^40+t^39+t^38+t^37+t^34+t^32+t^29+t^27+t^26+t^25+t^24+t^22+t^18+t^17+t^14+t^13+t^6+t^2+t)*x0^1*x2^1*x4^1) + ((t^51+t^47+t^44+t^43+t^39+t^38+t^36+t^35+t^32+t^31+t^30+t^29+t^28+t^27+t^26+t^24+t^20+t^17+t^15+t^14+t^13+t^11+t^8+t^6+t)*x2^2*x4^1) + ((t^52+t^51+t^49+t^48+t^43+t^41+t^40+t^39+t^38+t^37+t^34+t^33+t^32+t^31+t^30+t^29+t^28+t^26+t^23+t^22+t^21+t^20+t^18+t^17+t^16+t^15+t^10+t^7+t^5+t^2+t+1)*x0^1*x4^2) + ((t^52+t^50+t^48+t^47+t^45+t^44+t^43+t^38+t^37+t^34+t^28+t^24+t^21+t^17+t^16+t^15+t^13+t^12+t^9+t^8+t^2+t)*x0^1*x1^1) + ((t^50+t^49+t^46+t^43+t^40+t^38+t^37+t^36+t^32+t^31+t^29+t^22+t^20+t^17+t^15+t^13+t^11+t^10+t^5+t^4+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x0^1*x1^1*x2^2) + ((t^52+t^47+t^46+t^45+t^44+t^40+t^37+t^36+t^35+t^32+t^31+t^25+t^24+t^23+t^22+t^21+t^20+t^19+t^15+t^14+t^13+t^9+t^8+t^7+t^6+t^2)*x0^1*x2^2*x4^1) + ((t^52+t^51+t^50+t^46+t^44+t^42+t^35+t^31+t^29+t^28+t^27+t^24+t^22+t^20+t^19+t^16+t^14+t^11+t^9+t^8+t^7+t^6+t^3)*x2^3*x4^1) + ((t^48+t^46+t^45+t^43+t^42+t^35+t^34+t^32+t^31+t^30+t^29+t^27+t^24+t^23+t^21+t^20+t^18+t^16+t^14+t^13+t^12+t^7+t^6+t^2+t)*x1^1*x3^2*x4^1) + ((t^48+t^47+t^43+t^42+t^40+t^38+t^36+t^35+t^33+t^32+t^31+t^29+t^27+t^23+t^22+t^21+t^20+t^19+t^17+t^14+t^11+t^10+t^8+t)*x3^3*x4^1) + ((t^52+t^51+t^49+t^48+t^45+t^42+t^41+t^35+t^34+t^30+t^28+t^27+t^26+t^23+t^22+t^21+t^19+t^18+t^17+t^15+t^9+t^8+t^7+t^6+t^4+t)*x0^1*x2^1*x4^2) + ((t^48+t^47+t^45+t^44+t^43+t^42+t^39+t^38+t^37+t^36+t^35+t^34+t^31+t^26+t^25+t^24+t^23+t^22+t^20+t^18+t^17+t^12+t^11+t^6+t^5+t^3+t+1)*x2^2*x4^2) + ((t^50+t^47+t^46+t^45+t^44+t^40+t^39+t^38+t^37+t^34+t^32+t^31+t^30+t^28+t^27+t^26+t^24+t^23+t^22+t^21+t^17+t^12+t^8+t^6+t^4+t^2+t)*x0^1*x4^3) + ((t^52+t^50+t^49+t^46+t^45+t^44+t^41+t^39+t^38+t^37+t^35+t^34+t^30+t^28+t^25+t^23+t^18+t^17+t^15+t^13+t^12+t^8+t^7+t^6+t^5+t^4+t^2)*x0^1*x1^1*x2^1) + ((t^52+t^51+t^50+t^49+t^48+t^47+t^45+t^43+t^41+t^39+t^35+t^34+t^32+t^29+t^27+t^26+t^25+t^22+t^21+t^20+t^17+t^13+t^12+t^11+t^6+t^4+t)*x1^2*x3^1) + ((t^52+t^47+t^46+t^44+t^36+t^34+t^32+t^27+t^25+t^24+t^23+t^19+t^15+t^11+t^9+t^7+t^2+t+1)*x1^1*x3^2) + ((t^51+t^50+t^49+t^47+t^46+t^45+t^41+t^40+t^38+t^37+t^36+t^35+t^34+t^33+t^31+t^29+t^28+t^25+t^23+t^18+t^17+t^16+t^15+t^14+t^12+t^11+t^9+t^8+t^6+t^5+t^4)*x0^1*x2^1*x4^1) + ((t^51+t^47+t^45+t^43+t^42+t^39+t^37+t^36+t^34+t^32+t^30+t^27+t^25+t^24+t^23+t^22+t^21+t^20+t^18+t^12+t^11+t^10+t^9+t^8+t^7+t^6+t^5+t^4+t^3+t^2+1)*x0^1*x1^1) + ((t^51+t^47+t^46+t^44+t^43+t^42+t^41+t^39+t^38+t^37+t^32+t^31+t^23+t^21+t^18+t^16+t^15+t^12+t^10+t^9+t^7+t^5+t^2+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x2^4) + ((t^51+t^48+t^43+t^42+t^40+t^39+t^38+t^37+t^35+t^34+t^30+t^29+t^28+t^25+t^24+t^22+t^19+t^18+t^17+t^15+t^11+t^6+t^2+1)*x2^3*x4^1) + ((t^51+t^49+t^45+t^44+t^42+t^41+t^40+t^37+t^36+t^35+t^34+t^33+t^31+t^28+t^27+t^26+t^25+t^23+t^22+t^21+t^20+t^19+t^18+t^14+t^13+t^12+t^10+t^9+t^5+t^3+t)*x0^1*x2^1*x3^1*x4^1) + ((t^51+t^50+t^48+t^47+t^46+t^42+t^40+t^38+t^36+t^35+t^34+t^33+t^29+t^28+t^25+t^22+t^17+t^14+t^13+t^9+t^8+t^3+t^2+1)*x2^2*x3^1*x4^1) + ((t^52+t^51+t^50+t^47+t^45+t^42+t^41+t^40+t^38+t^37+t^36+t^34+t^33+t^29+t^27+t^22+t^21+t^19+t^18+t^17+t^14+t^12+t^11+t^10+t^9+t^8+t^5+t^2+t+1)*x1^1*x3^2*x4^1) + ((t^52+t^50+t^49+t^47+t^46+t^44+t^40+t^37+t^33+t^30+t^29+t^27+t^22+t^21+t^19+t^18+t^15+t^14+t^13+t^11+t^7+t^6+t^4+t^3+t+1)*x3^3*x4^1) + ((t^47+t^44+t^43+t^42+t^41+t^40+t^39+t^38+t^37+t^36+t^33+t^31+t^30+t^28+t^25+t^23+t^22+t^21+t^20+t^17+t^11+t^10+t^9+t^8+t^4+t^3+1)*x0^1*x2^1*x4^2) + ((t^50+t^49+t^48+t^47+t^46+t^45+t^43+t^41+t^39+t^38+t^36+t^35+t^34+t^30+t^29+t^26+t^25+t^24+t^22+t^18+t^14+t^9+t^6)*x2^2*x4^2) + ((t^51+t^50+t^47+t^46+t^44+t^43+t^39+t^37+t^34+t^32+t^31+t^26+t^23+t^21+t^17+t^16+t^13+t^11+t^9+t^8+t^7+t^6+t^4+t^3+t^2+t+1)*x0^1*x4^3) + ((t^50+t^48+t^46+t^42+t^41+t^39+t^38+t^37+t^36+t^33+t^32+t^30+t^28+t^27+t^24+t^23+t^21+t^20+t^18+t^15+t^13+t^8+t^4+t^3+t^2+1)*x0^1*x1^1*x2^1) + ((t^52+t^51+t^42+t^40+t^37+t^36+t^35+t^31+t^30+t^28+t^17+t^16+t^15+t^14+t^13+t^12+t^10+t^9+t^8+t^4+t^3+t+1)*x1^1*x2^2) + ((t^49+t^47+t^44+t^41+t^40+t^39+t^37+t^36+t^35+t^33+t^28+t^25+t^24+t^23+t^21+t^19+t^16+t^15+t^14+t^11+t^10+t^4+t^2+1)*x1^2*x3^1) + ((t^52+t^46+t^45+t^44+t^40+t^38+t^37+t^36+t^35+t^34+t^33+t^31+t^30+t^28+t^27+t^25+t^22+t^20+t^16+t^12+t^11+t^9+t^6)*x0^1*x3^2) + ((t^52+t^51+t^48+t^47+t^46+t^44+t^42+t^41+t^40+t^37+t^36+t^35+t^33+t^32+t^30+t^27+t^18+t^14+t^10+t^8+t^6+t^5+t^4+t^2+1)*x1^1*x3^2) + ((t^52+t^51+t^47+t^41+t^39+t^36+t^35+t^34+t^33+t^31+t^28+t^26+t^25+t^24+t^23+t^22+t^20+t^18+t^16+t^14+t^12+t^10+t^9+t^7+t^2+1)*x0^1*x2^1*x4^1) + ((t^51+t^50+t^49+t^46+t^45+t^41+t^40+t^39+t^38+t^34+t^30+t^29+t^26+t^22+t^21+t^16+t^14+t^13+t^12+t^4+t^3+t^2)*x2^2*x4^1) + ((t^52+t^48+t^47+t^45+t^44+t^43+t^41+t^39+t^38+t^37+t^36+t^35+t^33+t^31+t^30+t^29+t^26+t^24+t^23+t^20+t^19+t^16+t^14+t^6+t^5+t^4+t^2+1)*x0^1*x4^2) + ((t^52+t^51+t^50+t^48+t^47+t^46+t^45+t^43+t^39+t^36+t^33+t^32+t^31+t^29+t^25+t^24+t^23+t^21+t^19+t^17+t^14+t^13+t^7+t^6+t^5+t^4+t)*x4^3) + ((t^52+t^49+t^47+t^45+t^43+t^40+t^39+t^37+t^36+t^35+t^34+t^32+t^29+t^26+t^25+t^24+t^19+t^18+t^17+t^14+t^13+t^11+t^9+t^5+t)*x0^1*x1^1) + ((t^51+t^50+t^49+t^48+t^47+t^45+t^44+t^42+t^40+t^39+t^38+t^33+t^32+t^30+t^27+t^23+t^21+t^18+t^12+t^7+t^5+t^4+t^2+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x0^2*x1^1*x3^1) + ((t^51+t^49+t^48+t^43+t^41+t^40+t^37+t^35+t^34+t^33+t^32+t^28+t^26+t^25+t^24+t^22+t^21+t^18+t^17+t^15+t^13+t^10+t^5+t^4+t^3)*x0^1*x2^2*x4^1) + ((t^52+t^51+t^46+t^43+t^42+t^41+t^38+t^34+t^32+t^31+t^29+t^20+t^17+t^13+t^12+t^10+t^9+t^7+t^6+t^4+t^3+t^2+t)*x2^3*x4^1) + ((t^51+t^48+t^46+t^45+t^44+t^41+t^40+t^37+t^36+t^32+t^30+t^29+t^24+t^23+t^22+t^21+t^20+t^18+t^15+t^14+t^12+t^11+t^7+t^6+t^5+t)*x1^2*x3^1*x4^1) + ((t^52+t^45+t^44+t^42+t^41+t^39+t^36+t^35+t^34+t^32+t^29+t^28+t^26+t^25+t^21+t^20+t^19+t^16+t^14+t^13+t^12+t^8+t^6+t^5+t^4+t^3+t)*x2^2*x3^1*x4^1) + ((t^51+t^48+t^47+t^46+t^45+t^44+t^43+t^42+t^40+t^39+t^38+t^36+t^35+t^34+t^31+t^30+t^28+t^24+t^23+t^22+t^21+t^20+t^18+t^17+t^15+t^11+t^9+t^8+t^7+t^6+t^5)*x1^1*x3^2*x4^1) + ((t^51+t^50+t^49+t^47+t^42+t^40+t^38+t^37+t^36+t^35+t^34+t^31+t^28+t^27+t^26+t^25+t^23+t^20+t^17+t^16+t^14+t^12+t^9+t^4+t^3+1)*x3^3*x4^1) + ((t^52+t^49+t^48+t^47+t^46+t^45+t^43+t^42+t^40+t^36+t^34+t^32+t^30+t^29+t^28+t^27+t^26+t^25+t^24+t^23+t^21+t^17+t^10+t^7+t^3+1)*x0^2*x4^2) + ((t^50+t^49+t^45+t^44+t^42+t^40+t^37+t^35+t^34+t^33+t^31+t^29+t^28+t^27+t^26+t^25+t^24+t^23+t^22+t^21+t^19+t^18+t^16+t^14+t^13+t^10+t^7+t^6+t^5+t^2+1)*x0^1*x2^1*x4^2) + ((t^48+t^47+t^44+t^43+t^42+t^41+t^40+t^39+t^38+t^35+t^34+t^30+t^28+t^27+t^26+t^20+t^19+t^18+t^17+t^14+t^12+t^11+t^9+t^8+t^5+t^4+t^3)*x2^2*x4^2) + ((t^52+t^51+t^50+t^49+t^48+t^47+t^46+t^43+t^42+t^41+t^38+t^37+t^36+t^35+t^33+t^30+t^28+t^26+t^25+t^24+t^21+t^15+t^12+t^11+t^10+t^8+t^6+t^3+1)*x1^1*x3^1*x4^2) + ((t^51+t^49+t^47+t^46+t^44+t^42+t^41+t^36+t^35+t^30+t^29+t^27+t^26+t^23+t^20+t^18+t^17+t^16+t^12+t^10+t^8+t^5+t^3+t^2+t+1)*x0^1*x4^3) + ((t^51+t^50+t^48+t^47+t^46+t^43+t^42+t^37+t^35+t^32+t^28+t^18+t^17+t^9+t^8+t^5+t^3+t^2+1)*x4^4) + ((t^52+t^49+t^46+t^43+t^42+t^39+t^37+t^32+t^31+t^27+t^25+t^24+t^23+t^21+t^20+t^18+t^16+t^15+t^13+t^11+t^10+t^8+t^2+t+1)*x0^1*x1^1*x2^1) + ((t^52+t^50+t^45+t^42+t^41+t^37+t^35+t^34+t^33+t^32+t^30+t^29+t^28+t^26+t^25+t^23+t^22+t^21+t^19+t^16+t^14+t^13+t^12+t^10+t^9+t^8+t^3+t)*x1^2*x3^1) + ((t^52+t^46+t^43+t^37+t^35+t^32+t^28+t^26+t^24+t^23+t^17+t^13+t^12+t^9+t^5+t^4+t^2)*x0^1*x3^2) + ((t^50+t^48+t^47+t^45+t^42+t^37+t^36+t^35+t^33+t^32+t^30+t^29+t^26+t^25+t^24+t^21+t^20+t^19+t^18+t^17+t^16+t^13+t^11+t^10+t^9+t^8+t^7+t^6+t^4+t)*x1^1*x3^2) + ((t^49+t^46+t^43+t^42+t^41+t^36+t^35+t^34+t^30+t^27+t^26+t^25+t^24+t^23+t^21+t^20+t^19+t^15+t^14+t^13+t^11+t^7+t)*x0^1*x1^1*x4^1) + ((t^52+t^51+t^49+t^48+t^47+t^46+t^45+t^43+t^41+t^39+t^37+t^36+t^35+t^34+t^33+t^31+t^30+t^28+t^23+t^19+t^18+t^16+t^15+t^10+t^3+t)*x0^1*x2^1*x4^1) + ((t^51+t^50+t^49+t^48+t^47+t^44+t^43+t^42+t^41+t^36+t^35+t^33+t^32+t^31+t^29+t^27+t^23+t^22+t^21+t^20+t^18+t^17+t^16+t^15+t^14+t^13+t^12+t^11+t^9+t^8+t^7+t^6+t^4)*x2^2*x4^1) + ((t^51+t^50+t^49+t^48+t^44+t^43+t^41+t^39+t^37+t^36+t^35+t^33+t^30+t^24+t^23+t^22+t^17+t^16+t^15+t^14+t^13+t^11+t^7+t^5+t^4+t^2+t+1)*x1^1*x3^1*x4^1) + ((t^51+t^50+t^46+t^45+t^43+t^38+t^37+t^34+t^30+t^26+t^25+t^20+t^18+t^16+t^5+t^4+t^3+t^2+t)*x0^1*x4^2) + ((t^51+t^50+t^49+t^45+t^44+t^43+t^42+t^40+t^38+t^37+t^32+t^30+t^27+t^26+t^25+t^23+t^21+t^20+t^18+t^14+t^11+t^9+t^8+t^7+t^6+t^4+t^3+1)*x0^1*x1^1) + ((t^51+t^50+t^49+t^47+t^44+t^43+t^42+t^41+t^40+t^37+t^36+t^33+t^30+t^29+t^27+t^24+t^21+t^16+t^14+t^13+t^9+t^5+t^3+t^2+t+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x0^1*x1^2*x3^1) + ((t^52+t^51+t^47+t^46+t^40+t^39+t^36+t^35+t^34+t^33+t^31+t^27+t^26+t^25+t^24+t^19+t^17+t^13+t^12+t^10+t^9+t^8+t^6+t^4+t+1)*x0^1*x2^2*x4^1) + ((t^50+t^49+t^46+t^44+t^42+t^41+t^40+t^37+t^36+t^35+t^34+t^33+t^30+t^29+t^26+t^25+t^23+t^22+t^18+t^16+t^14+t^13+t^11+t^10+t^8+t^7+t)*x2^3*x4^1) + ((t^52+t^50+t^49+t^46+t^44+t^43+t^42+t^41+t^34+t^33+t^30+t^29+t^28+t^26+t^25+t^24+t^22+t^19+t^18+t^17+t^16+t^13+t^9+t^6+t^5+t^4+t^3+t^2+t+1)*x2^2*x3^1*x4^1) + ((t^51+t^50+t^48+t^46+t^45+t^41+t^40+t^37+t^36+t^35+t^34+t^33+t^30+t^29+t^28+t^27+t^24+t^22+t^21+t^19+t^14+t^11+t^7+t^6+t^4+t^3+t+1)*x1^1*x3^2*x4^1) + ((t^52+t^50+t^49+t^46+t^45+t^41+t^38+t^37+t^36+t^32+t^31+t^28+t^22+t^21+t^16+t^14+t^13+t^12+t^11+t^10+t^9+t^7+t^6+t^3+t^2+t+1)*x3^3*x4^1) + ((t^49+t^45+t^43+t^42+t^41+t^39+t^37+t^36+t^35+t^34+t^33+t^31+t^27+t^26+t^25+t^24+t^23+t^21+t^19+t^17+t^10+t^6+t^5+t^4+t+1)*x0^2*x4^2) + ((t^52+t^51+t^50+t^44+t^43+t^42+t^41+t^40+t^39+t^36+t^35+t^33+t^32+t^24+t^23+t^21+t^18+t^16+t^15+t^13+t^6+t^5+t^3+1)*x0^1*x1^1*x4^2) + ((t^52+t^50+t^49+t^47+t^46+t^45+t^42+t^40+t^39+t^38+t^35+t^31+t^29+t^27+t^26+t^18+t^17+t^15+t^13+t^9+t^8+t^5+t^3+t^2+t)*x0^1*x2^1*x4^2) + ((t^51+t^49+t^47+t^45+t^42+t^41+t^40+t^39+t^38+t^36+t^33+t^32+t^31+t^28+t^26+t^23+t^22+t^20+t^19+t^18+t^15+t^14+t^13+t^12+t^10+t^7+1)*x2^2*x4^2) + ((t^50+t^48+t^47+t^46+t^45+t^44+t^42+t^41+t^36+t^33+t^32+t^28+t^26+t^25+t^24+t^23+t^21+t^19+t^15+t^14+t^13+t^12+t^10+t^5+t^4+t^2+t+1)*x0^1*x4^3) + ((t^52+t^50+t^49+t^47+t^44+t^39+t^38+t^35+t^33+t^30+t^29+t^28+t^27+t^25+t^24+t^23+t^20+t^16+t^11+t^8+t^7+t^6+t^5+t^2+1)*x4^4) + ((t^52+t^50+t^47+t^45+t^43+t^42+t^41+t^40+t^38+t^36+t^35+t^32+t^31+t^30+t^28+t^27+t^25+t^24+t^23+t^22+t^21+t^20+t^17+t^14+t^13+t^11+t^7+t^6+t^4+t^3+t^2+1)*x0^2*x1^1) + ((t^52+t^51+t^50+t^49+t^43+t^41+t^39+t^38+t^37+t^35+t^34+t^32+t^30+t^26+t^22+t^21+t^19+t^12+t^7+t^4+t^3+t^2+1)*x0^1*x1^1*x2^1) + ((t^52+t^50+t^49+t^47+t^46+t^40+t^39+t^38+t^33+t^32+t^27+t^25+t^20+t^18+t^12+t^11+t^10+t^9+t^8+t^7+t^5+t^4+t^3+t^2+t)*x0^1*x1^1*x3^1) + ((t^52+t^51+t^49+t^48+t^44+t^43+t^42+t^39+t^37+t^35+t^34+t^33+t^31+t^29+t^25+t^24+t^23+t^22+t^21+t^20+t^18+t^16+t^15+t^14+t^13+t^11+t^10+t^9+t^8+t^4+t^3+t)*x1^2*x3^1) + ((t^50+t^47+t^42+t^38+t^37+t^35+t^34+t^32+t^31+t^28+t^27+t^26+t^25+t^22+t^21+t^20+t^19+t^15+t^13+t^12+t^9+t^8+t^7+t^6+t^4+t^2+t+1)*x0^1*x3^2) + ((t^48+t^46+t^44+t^42+t^40+t^38+t^37+t^35+t^34+t^33+t^32+t^30+t^28+t^27+t^25+t^23+t^21+t^20+t^19+t^17+t^13+t^12+t^11+t^10+t^8+t^7)*x1^1*x3^2) + ((t^51+t^50+t^48+t^47+t^45+t^39+t^35+t^34+t^31+t^27+t^24+t^23+t^22+t^21+t^19+t^15+t^13+t^11+t^9+t^6+t^4+1)*x0^1*x2^1*x4^1) + ((t^52+t^51+t^50+t^49+t^47+t^46+t^45+t^41+t^40+t^39+t^37+t^36+t^35+t^34+t^31+t^30+t^29+t^26+t^25+t^24+t^16+t^15+t^13+t^12+t^10+t^6+t^5+t^4+t+1)*x2^2*x4^1) + ((t^51+t^50+t^48+t^42+t^40+t^39+t^38+t^36+t^34+t^33+t^32+t^31+t^29+t^24+t^23+t^22+t^21+t^20+t^19+t^18+t^17+t^12+t^8+t^5+t^3+t^2+t)*x0^1*x4^2) + ((t^49+t^44+t^41+t^38+t^36+t^35+t^33+t^31+t^29+t^27+t^25+t^21+t^20+t^18+t^16+t^10+t^7+t^6+t^4+t^2+t+1)*x0^1*x1^1) + ((t^52+t^51+t^48+t^47+t^46+t^45+t^44+t^43+t^42+t^37+t^33+t^32+t^30+t^27+t^26+t^23+t^22+t^18+t^17+t^15+t^13+t^11+t^10+t^6+t^3+t^2)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x1^3*x3^1) + ((t^51+t^48+t^46+t^44+t^43+t^42+t^41+t^40+t^38+t^36+t^33+t^32+t^31+t^30+t^29+t^28+t^27+t^26+t^22+t^20+t^19+t^18+t^17+t^13+t^12+t^8+t^4+t^3)*x2^3*x4^1) + ((t^51+t^50+t^45+t^42+t^39+t^37+t^34+t^31+t^30+t^29+t^28+t^27+t^26+t^25+t^24+t^23+t^22+t^19+t^12+t^10+t^7+t^6+t^4+t^3+t^2+t+1)*x0^1*x1^1*x3^1*x4^1) + ((t^52+t^51+t^50+t^49+t^47+t^46+t^45+t^44+t^43+t^41+t^39+t^38+t^37+t^34+t^31+t^28+t^26+t^23+t^20+t^19+t^18+t^14+t^13+t^12+t^10+t^9+t^8+t^7+t^6+t^4+t^3)*x2^2*x3^1*x4^1) + ((t^52+t^51+t^49+t^48+t^46+t^45+t^44+t^41+t^40+t^39+t^38+t^37+t^35+t^34+t^31+t^30+t^28+t^27+t^25+t^24+t^22+t^21+t^19+t^18+t^17+t^16+t^15+t^14+t^13+t^12+t^11+t^10+t^9+t^6+t^5+t^3+t^2+t)*x1^1*x3^2*x4^1) + ((t^52+t^51+t^50+t^47+t^44+t^43+t^41+t^40+t^39+t^38+t^35+t^29+t^25+t^24+t^22+t^21+t^13+t^12+t^10+t^9+t^8+t^7+t^5+t^4+t^3+t)*x3^3*x4^1) + ((t^52+t^49+t^48+t^47+t^46+t^45+t^43+t^42+t^41+t^40+t^38+t^37+t^32+t^31+t^30+t^28+t^25+t^24+t^23+t^21+t^20+t^19+t^18+t^17+t^15+t^14+t^13+t^12+t^10+t^7+t^6+t^5+t^4+t^2+1)*x0^1*x2^1*x4^2) + ((t^49+t^48+t^47+t^44+t^42+t^40+t^39+t^36+t^33+t^32+t^31+t^28+t^27+t^26+t^24+t^22+t^20+t^19+t^12+t^6+t^3+t^2+t+1)*x2^2*x4^2) + ((t^51+t^48+t^47+t^44+t^41+t^40+t^38+t^36+t^31+t^30+t^29+t^28+t^27+t^25+t^23+t^22+t^21+t^18+t^13+t^8+t^6+t^3+t)*x0^1*x4^3) + ((t^52+t^50+t^47+t^45+t^43+t^42+t^41+t^40+t^38+t^36+t^35+t^32+t^31+t^30+t^28+t^27+t^25+t^24+t^23+t^22+t^21+t^20+t^17+t^14+t^13+t^11+t^7+t^6+t^4+t^3+t^2+1)*x0^1*x1^2) + ((t^51+t^50+t^49+t^46+t^45+t^44+t^43+t^41+t^40+t^37+t^34+t^33+t^31+t^30+t^28+t^26+t^25+t^21+t^20+t^17+t^15+t^14+t^12+t^11+t^9+t^8+t^4+1)*x0^1*x1^1*x2^1) + ((t^52+t^51+t^46+t^43+t^40+t^39+t^38+t^37+t^33+t^32+t^31+t^30+t^28+t^26+t^24+t^23+t^19+t^16+t^15+t^13+t^10+t^7+t^5+t^2+1)*x0^1*x1^1*x3^1) + ((t^50+t^46+t^45+t^43+t^42+t^41+t^40+t^39+t^38+t^37+t^34+t^33+t^32+t^31+t^28+t^26+t^23+t^21+t^20+t^18+t^16+t^10+t^9+t^7+t^6+t^5+t^4)*x1^2*x3^1) + ((t^51+t^50+t^48+t^45+t^44+t^43+t^41+t^40+t^35+t^29+t^28+t^25+t^23+t^20+t^19+t^16+t^15+t^11+t^8+t^7+t^6+t^3+t+1)*x0^1*x3^2) + ((t^52+t^51+t^47+t^45+t^44+t^40+t^37+t^35+t^33+t^31+t^30+t^28+t^25+t^20+t^19+t^18+t^16+t^15+t^14+t^13+t^12+t^8+t^7+t^6+t^5+t^2+1)*x1^1*x3^2) + ((t^48+t^46+t^44+t^43+t^42+t^41+t^40+t^38+t^35+t^30+t^28+t^24+t^23+t^22+t^21+t^19+t^18+t^17+t^16+t^8+t^7+t^6+t^5+t^4+t^3+t^2+1)*x3^3) + ((t^50+t^49+t^47+t^43+t^41+t^40+t^39+t^36+t^34+t^33+t^32+t^29+t^27+t^24+t^23+t^22+t^20+t^15+t^14+t^13+t^10+t^9+t^6+t^5+t^4+t^3+t^2+t+1)*x0^1*x2^1*x4^1) + ((t^52+t^51+t^49+t^46+t^45+t^41+t^39+t^38+t^37+t^36+t^32+t^31+t^30+t^29+t^28+t^24+t^21+t^18+t^17+t^16+t^11+t^9+t^8+t^7+t^6+t^5+t^2+t)*x2^2*x4^1) + ((t^52+t^50+t^49+t^48+t^46+t^44+t^40+t^39+t^38+t^34+t^31+t^27+t^25+t^24+t^22+t^21+t^18+t^17+t^16+t^13+t^11+t^7+t^6+t^5+t^4+t^2+t)*x0^1*x4^2) + ((t^48+t^47+t^45+t^44+t^43+t^42+t^41+t^38+t^37+t^34+t^33+t^31+t^30+t^26+t^25+t^24+t^22+t^18+t^17+t^15+t^12+t^11+t^4+t^3+t)*x0^1*x1^1) + ((t^52+t^49+t^47+t^46+t^44+t^41+t^40+t^38+t^36+t^32+t^29+t^27+t^26+t^22+t^21+t^19+t^18+t^17+t^16+t^14+t^13+t^11+t^10+t^6+t^5+t^3+t+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x0^1*x1^1*x2^1*x3^1) + ((t^52+t^51+t^50+t^48+t^44+t^43+t^42+t^41+t^39+t^38+t^35+t^33+t^28+t^27+t^25+t^24+t^23+t^19+t^18+t^16+t^15+t^10+t^8+t^6+t^2)*x2^3*x4^1) + ((t^52+t^47+t^46+t^45+t^44+t^40+t^37+t^36+t^35+t^32+t^31+t^25+t^24+t^23+t^22+t^21+t^20+t^19+t^15+t^14+t^13+t^9+t^8+t^7+t^6+t^2)*x0^1*x2^1*x3^1*x4^1) + ((t^52+t^50+t^49+t^48+t^47+t^45+t^42+t^39+t^37+t^33+t^31+t^29+t^26+t^25+t^24+t^23+t^22+t^21+t^20+t^19+t^17+t^16+t^15+t^14+t^12+t^11+t^10+t^8+t^7+t^6+t^5+t^2)*x1^1*x3^2*x4^1) + ((t^52+t^51+t^50+t^49+t^47+t^46+t^44+t^43+t^42+t^41+t^40+t^39+t^38+t^37+t^36+t^34+t^29+t^27+t^26+t^25+t^19+t^15+t^12+t^10+t^8+t+1)*x3^3*x4^1) + ((t^52+t^51+t^50+t^47+t^41+t^40+t^38+t^33+t^30+t^28+t^26+t^25+t^24+t^20+t^18+t^17+t^16+t^15+t^14+t^13+t^11+t^10+t^6+t^5+t^4+t^2+1)*x0^1*x2^1*x4^2) + ((t^52+t^47+t^46+t^44+t^43+t^42+t^36+t^35+t^31+t^27+t^24+t^23+t^19+t^18+t^17+t^14+t^13+t^10+t^7+t^6+t^5+t^4+t^2+1)*x2^2*x4^2) + ((t^52+t^49+t^48+t^46+t^44+t^42+t^41+t^40+t^39+t^37+t^35+t^34+t^33+t^31+t^29+t^28+t^24+t^21+t^20+t^19+t^17+t^16+t^14+t^13+t^9+t^8+t^4+t^2)*x0^1*x4^3) + ((t^52+t^51+t^48+t^47+t^45+t^44+t^42+t^41+t^40+t^39+t^38+t^36+t^35+t^33+t^32+t^29+t^24+t^23+t^17+t^12+t^10+t^9+t^8+t^6+t^5+t^4+t^2+t)*x0^1*x1^1*x2^1) + ((t^50+t^49+t^47+t^46+t^45+t^43+t^42+t^38+t^37+t^36+t^33+t^32+t^30+t^27+t^26+t^24+t^23+t^20+t^15+t^12+t^9+t^8+t^5+t^4+t^3+t)*x1^2*x3^1) + ((t^52+t^49+t^45+t^44+t^43+t^40+t^36+t^35+t^34+t^33+t^31+t^29+t^28+t^26+t^25+t^23+t^17+t^16+t^15+t^14+t^11+t^7+t^6+t^5+t^4+t^2)*x1^1*x3^2) + ((t^51+t^49+t^47+t^46+t^44+t^42+t^38+t^35+t^27+t^23+t^22+t^17+t^15+t^14+t^13+t^9+t^8+t^7+t^3+t^2+t+1)*x0^1*x2^1*x4^1) + ((t^51+t^50+t^49+t^46+t^44+t^43+t^42+t^38+t^35+t^34+t^30+t^29+t^28+t^27+t^26+t^25+t^24+t^23+t^21+t^20+t^19+t^16+t^14+t^11+t^7+t^4+t^2+t+1)*x0^1*x1^1) + ((t^49+t^47+t^46+t^45+t^43+t^42+t^40+t^39+t^38+t^37+t^36+t^35+t^33+t^30+t^28+t^27+t^26+t^23+t^21+t^20+t^19+t^18+t^17+t^15+t^13+t^10+t^5+t^4+t^3+t^2+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x1^2*x2^1*x3^1) + ((t^50+t^49+t^48+t^44+t^41+t^40+t^38+t^37+t^32+t^30+t^27+t^26+t^24+t^23+t^22+t^21+t^20+t^17+t^16+t^14+t^13+t^10+t^8)*x1^1*x2^1*x3^2) + ((t^52+t^50+t^49+t^48+t^47+t^42+t^41+t^40+t^38+t^37+t^36+t^34+t^33+t^31+t^29+t^27+t^22+t^20+t^18+t^16+t^15+t^12+t^10+t^9+t^3+1)*x2^3*x4^1) + ((t^47+t^45+t^43+t^42+t^40+t^37+t^32+t^30+t^29+t^28+t^26+t^25+t^24+t^23+t^21+t^20+t^17+t^15+t^14+t^13+t^12+t^11+t^9+t^8+t^5+t^4+t^3)*x0^1*x2^1*x4^2) + ((t^52+t^50+t^47+t^45+t^43+t^42+t^41+t^40+t^38+t^36+t^35+t^32+t^31+t^30+t^28+t^27+t^25+t^24+t^23+t^22+t^21+t^20+t^17+t^14+t^13+t^11+t^7+t^6+t^4+t^3+t^2+1)*x0^1*x1^1*x2^1) + ((t^52+t^50+t^49+t^47+t^46+t^40+t^39+t^38+t^33+t^32+t^27+t^25+t^20+t^18+t^12+t^11+t^10+t^9+t^8+t^7+t^5+t^4+t^3+t^2+t)*x1^1*x2^1*x3^1)");
    groebnerBasis.push_back("((+1)*x0^2*x3^2) + ((t^50+t^48+t^47+t^46+t^44+t^40+t^39+t^37+t^34+t^33+t^30+t^27+t^26+t^25+t^21+t^20+t^18+t^16+t^14+t^11+t^9+t^7+t^5+t^3)*x0^1*x2^2*x4^1) + ((t^52+t^46+t^44+t^43+t^41+t^38+t^37+t^33+t^31+t^28+t^27+t^25+t^23+t^21+t^20+t^18+t^17+t^14+t^13+t^12+t^9+t^6+t^2+t+1)*x0^2*x4^2)");
    groebnerBasis.push_back("((+1)*x0^1*x1^1*x3^2) + ((t^51+t^49+t^48+t^45+t^43+t^41+t^40+t^38+t^37+t^35+t^34+t^33+t^28+t^27+t^26+t^25+t^23+t^20+t^18+t^17+t^16+t^9+t^8+t^6+t^5+t^2+1)*x0^1*x2^2*x4^1) + ((t^50+t^49+t^46+t^37+t^36+t^30+t^28+t^25+t^23+t^17+t^16+t^13+t^11+t^10+t^9+t^6+t^5+t^4+t)*x2^3*x4^1) + ((t^50+t^49+t^47+t^46+t^45+t^42+t^40+t^39+t^37+t^36+t^35+t^34+t^33+t^32+t^29+t^28+t^24+t^22+t^20+t^17+t^14+t^13+t^12+t^10+t^9+t^8+t^7+t^5+t^2+t+1)*x2^2*x3^1*x4^1) + ((t^51+t^49+t^47+t^46+t^43+t^40+t^39+t^38+t^37+t^36+t^35+t^32+t^30+t^29+t^27+t^26+t^25+t^24+t^23+t^20+t^18+t^15+t^14+t^7+t^5+t^3+1)*x1^1*x3^2*x4^1) + ((t^51+t^50+t^47+t^46+t^41+t^37+t^36+t^35+t^34+t^33+t^32+t^31+t^30+t^29+t^28+t^25+t^22+t^21+t^19+t^18+t^13+t^11+t^10+t^8+t^6+t^5+t^3+t^2)*x3^3*x4^1) + ((t^52+t^51+t^50+t^49+t^48+t^46+t^44+t^42+t^40+t^39+t^36+t^35+t^32+t^30+t^28+t^27+t^25+t^24+t^21+t^19+t^17+t^16+t^15+t^14+t^12+t^4+t^2+t)*x0^2*x4^2) + ((t^52+t^46+t^44+t^43+t^41+t^38+t^37+t^33+t^31+t^28+t^27+t^25+t^23+t^21+t^20+t^18+t^17+t^14+t^13+t^12+t^9+t^6+t^2+t+1)*x0^1*x1^1*x4^2) + ((t^52+t^51+t^50+t^48+t^46+t^44+t^43+t^37+t^32+t^31+t^26+t^24+t^17+t^14+t^12+t^10+t^9+t^8+t^7+t^6+t^5+1)*x0^1*x2^1*x4^2) + ((t^52+t^51+t^50+t^49+t^47+t^44+t^42+t^40+t^37+t^36+t^35+t^34+t^32+t^31+t^28+t^27+t^26+t^25+t^24+t^23+t^21+t^20+t^18+t^17+t^16+t^14+t^13+t^10+t^9+t^8+t^5+t^4+t)*x2^2*x4^2) + ((t^52+t^50+t^47+t^44+t^43+t^42+t^41+t^37+t^33+t^30+t^29+t^27+t^25+t^24+t^22+t^19+t^17+t^16+t^14+t^12+t^11+t^10+t^6+1)*x0^1*x4^3) + ((t^52+t^50+t^48+t^42+t^40+t^39+t^38+t^36+t^34+t^32+t^31+t^29+t^27+t^26+t^25+t^22+t^19+t^18+t^16+t^15+t^14+t^13+t^12+t^8+t^6+t^5+t^4+t^3)*x4^4) + ((t^52+t^50+t^48+t^47+t^46+t^45+t^44+t^43+t^42+t^41+t^39+t^37+t^36+t^35+t^33+t^31+t^28+t^24+t^22+t^20+t^19+t^15+t^12+t^11+t^10+t^6+t^4+1)*x0^1*x1^1*x2^1) + ((t^49+t^47+t^44+t^43+t^42+t^41+t^40+t^37+t^36+t^35+t^32+t^31+t^29+t^24+t^19+t^18+t^17+t^16+t^14+t^13+t^12+t^11+t^9+t^7+t^6+t^5+t^4+t^3+t^2)*x1^2*x3^1) + ((t^52+t^50+t^49+t^47+t^46+t^40+t^39+t^38+t^33+t^32+t^27+t^25+t^20+t^18+t^12+t^11+t^10+t^9+t^8+t^7+t^5+t^4+t^3+t^2+t)*x0^1*x3^2) + ((t^52+t^51+t^48+t^46+t^40+t^37+t^36+t^35+t^33+t^32+t^29+t^28+t^27+t^26+t^25+t^23+t^20+t^19+t^18+t^16+t^15+t^14+t^13+t^12+t^11+t^9+t^8+t^6+t^5+t^3+1)*x1^1*x3^2) + ((t^51+t^50+t^48+t^47+t^46+t^43+t^42+t^41+t^40+t^39+t^35+t^29+t^26+t^22+t^20+t^17+t^16+t^15+t^14+t^13+t^10+t^5+t^3+t^2+t)*x0^1*x2^1*x4^1) + ((t^51+t^50+t^48+t^46+t^44+t^43+t^42+t^41+t^38+t^36+t^34+t^32+t^30+t^26+t^25+t^24+t^20+t^17+t^14+t^13+t^12+t^8+t^6+t^5+t^4+t+1)*x2^2*x4^1) + ((t^52+t^51+t^50+t^49+t^48+t^43+t^41+t^39+t^38+t^33+t^32+t^29+t^28+t^27+t^26+t^22+t^20+t^17+t^16+t^15+t^14+t^13+t^10+t^9+t^6+t^4+t^3+1)*x0^1*x4^2) + ((t^52+t^51+t^50+t^49+t^46+t^44+t^43+t^42+t^41+t^40+t^39+t^37+t^36+t^35+t^32+t^31+t^29+t^28+t^27+t^25+t^24+t^20+t^19+t^16+t^12+t^11+t^10+t^9+t^7+t^6+t^4)*x0^1*x1^1) + ((t^51+t^48+t^46+t^45+t^42+t^41+t^40+t^38+t^36+t^34+t^31+t^29+t^28+t^27+t^25+t^21+t^20+t^18+t^13+t^11+t^10+t^9+t^4+t^2)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x1^2*x3^2) + ((t^52+t^49+t^48+t^47+t^46+t^45+t^40+t^36+t^35+t^31+t^29+t^28+t^27+t^26+t^25+t^24+t^23+t^22+t^21+t^20+t^18+t^17+t^15+t^13+t^12+t^11+t^7+t^6+t)*x2^3*x4^1) + ((t^52+t^51+t^49+t^45+t^44+t^43+t^42+t^40+t^39+t^35+t^33+t^29+t^28+t^27+t^22+t^21+t^20+t^17+t^12+t^11+t^10+t^9+t^8+t^7+t^5+t^4+t^3+t^2+t+1)*x1^1*x3^2*x4^1) + ((t^51+t^50+t^47+t^46+t^44+t^42+t^41+t^39+t^38+t^33+t^31+t^30+t^25+t^24+t^23+t^22+t^20+t^19+t^15+t^14+t^12+t^11+t^8+t^7+t^6+t^5+t^4+t^2+t)*x3^3*x4^1) + ((t^51+t^50+t^49+t^45+t^44+t^41+t^39+t^38+t^35+t^33+t^31+t^29+t^28+t^23+t^22+t^21+t^20+t^18+t^17+t^12+t^11+t^10+t^6+t^4+t^3+t)*x0^1*x2^1*x4^2) + ((t^52+t^51+t^50+t^49+t^48+t^47+t^46+t^45+t^44+t^42+t^41+t^40+t^39+t^37+t^36+t^29+t^28+t^26+t^24+t^23+t^22+t^21+t^20+t^17+t^16+t^11+t^9+t^8+t^7+t^6+t^2)*x2^2*x4^2) + ((t^51+t^47+t^45+t^44+t^43+t^40+t^38+t^37+t^35+t^33+t^28+t^27+t^26+t^19+t^18+t^17+t^16+t^15+t^14+t^12+t^11+t^9+t^7+t^6+t^5+t^3+t^2+1)*x0^1*x4^3) + ((t^52+t^49+t^48+t^47+t^44+t^43+t^40+t^38+t^35+t^33+t^30+t^29+t^28+t^26+t^25+t^23+t^22+t^21+t^19+t^18+t^17+t^16+t^12+t^8+t^7+t^6+t^3+t^2+t+1)*x0^1*x1^1*x2^1) + ((t^52+t^50+t^47+t^45+t^43+t^42+t^41+t^40+t^38+t^36+t^35+t^32+t^31+t^30+t^28+t^27+t^25+t^24+t^23+t^22+t^21+t^20+t^17+t^14+t^13+t^11+t^7+t^6+t^4+t^3+t^2+1)*x0^1*x1^1*x3^1) + ((t^50+t^49+t^47+t^45+t^44+t^42+t^41+t^40+t^37+t^36+t^35+t^31+t^28+t^27+t^26+t^25+t^22+t^19+t^18+t^16+t^15+t^14+t^13+t^9+t^8+t^7+t^6+t^5+t^2)*x1^2*x3^1) + ((t^52+t^51+t^46+t^43+t^40+t^39+t^38+t^37+t^33+t^32+t^31+t^30+t^28+t^26+t^24+t^23+t^19+t^16+t^15+t^13+t^10+t^7+t^5+t^2+1)*x0^1*x3^2) + ((t^52+t^51+t^49+t^48+t^47+t^44+t^43+t^41+t^40+t^39+t^38+t^37+t^36+t^34+t^33+t^31+t^30+t^29+t^28+t^27+t^24+t^22+t^20+t^19+t^18+t^17+t^16+t^15+t^14+t^11+t^10+t^8+t^5+t^3)*x1^1*x3^2) + ((t^50+t^47+t^42+t^38+t^37+t^35+t^34+t^32+t^31+t^28+t^27+t^26+t^25+t^22+t^21+t^20+t^19+t^15+t^13+t^12+t^9+t^8+t^7+t^6+t^4+t^2+t+1)*x3^3) + ((t^49+t^47+t^45+t^44+t^41+t^40+t^39+t^33+t^30+t^27+t^26+t^23+t^22+t^19+t^18+t^17+t^16+t^15+t^14+t^13+t^10+t^9+t^7+t^6+t^5+t^2)*x0^1*x2^1*x4^1) + ((t^49+t^47+t^46+t^42+t^40+t^39+t^37+t^36+t^34+t^32+t^31+t^27+t^25+t^24+t^22+t^20+t^19+t^17+t^15+t^14+t^13+t^11+t^9+t^8+t^6+t^5+t+1)*x0^1*x1^1) + ((t^52+t^42+t^40+t^39+t^38+t^34+t^33+t^31+t^30+t^26+t^25+t^22+t^20+t^17+t^15+t^14+t^13+t^10+t^9+t^5+t^4+t^3+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x0^1*x2^1*x3^2)");
    groebnerBasis.push_back("((+1)*x0^1*x3^3) + ((t^50+t^48+t^47+t^46+t^44+t^40+t^39+t^37+t^34+t^33+t^30+t^27+t^26+t^25+t^21+t^20+t^18+t^16+t^14+t^11+t^9+t^7+t^5+t^3)*x2^2*x3^1*x4^1) + ((t^52+t^46+t^45+t^44+t^41+t^38+t^37+t^36+t^33+t^29+t^28+t^25+t^23+t^20+t^19+t^17+t^16+t^13+t^10+t^9+t^3+t+1)*x0^1*x3^2) + ((t^50+t^48+t^46+t^44+t^43+t^42+t^41+t^39+t^38+t^37+t^33+t^32+t^31+t^28+t^26+t^24+t^21+t^20+t^18+t^16+t^13+t^12+t^9+t^8+t^5+t^4+t^2+t)*x2^2*x4^1) + ((t^51+t^45+t^44+t^43+t^40+t^37+t^36+t^27+t^25+t^23+t^20+t^18+t^17+t^16+t^12+t^11+t^9+t^8+t^5+t^2+t+1)*x0^1*x4^2)");
    groebnerBasis.push_back("((+1)*x1^1*x3^3) + ((t^51+t^50+t^48+t^45+t^41+t^39+t^38+t^36+t^34+t^32+t^31+t^30+t^28+t^26+t^25+t^24+t^22+t^21+t^20+t^19+t^16+t^15+t^11+t^10+t^9+t^8+t^4+t^3+t^2)*x2^3*x4^1) + ((t^50+t^46+t^45+t^42+t^41+t^40+t^39+t^37+t^36+t^35+t^34+t^31+t^30+t^29+t^28+t^27+t^26+t^24+t^22+t^18+t^15+t^14+t^13+t^12+t^10+t^6+t^5+t^3+t^2+1)*x1^1*x3^2*x4^1) + ((t^52+t^51+t^47+t^42+t^39+t^38+t^37+t^34+t^29+t^27+t^24+t^22+t^21+t^20+t^19+t^18+t^17+t^15+t^11+t^9+t^6+t^3+t^2)*x3^3*x4^1) + ((t^52+t^51+t^50+t^48+t^46+t^45+t^42+t^41+t^40+t^37+t^35+t^34+t^32+t^31+t^30+t^29+t^28+t^26+t^20+t^19+t^17+t^16+t^15+t^14+t^13+t^12+t^11+t^10+t^5+t^4+t^3+t)*x0^1*x2^1*x4^2) + ((t^52+t^48+t^46+t^43+t^41+t^40+t^39+t^38+t^36+t^33+t^32+t^27+t^26+t^25+t^24+t^22+t^21+t^19+t^18+t^17+t^16+t^15+t^14+t^13+t^11+t^5+t^4+t^3+t)*x2^2*x4^2) + ((t^43+t^42+t^41+t^40+t^38+t^37+t^35+t^33+t^30+t^29+t^27+t^26+t^25+t^23+t^21+t^17+t^16+t^15+t^14+t^13+t^12+t^10+t^9+t^8+t^6+t^5+t^4+t^3+t^2+t)*x0^1*x4^3) + ((t^52+t^51+t^50+t^49+t^46+t^44+t^42+t^40+t^36+t^35+t^33+t^29+t^27+t^26+t^25+t^23+t^21+t^20+t^19+t^16+t^12+t^8+t^6+t^4+t^2+1)*x0^1*x1^1*x2^1) + ((t^52+t^50+t^46+t^45+t^44+t^43+t^42+t^41+t^39+t^38+t^36+t^35+t^31+t^28+t^27+t^26+t^25+t^24+t^22+t^21+t^18+t^17+t^12+t^11+t^9+t^8+t^7+t^6+t^5+t)*x1^2*x3^1) + ((t^52+t^50+t^47+t^45+t^43+t^42+t^41+t^40+t^38+t^36+t^35+t^32+t^31+t^30+t^28+t^27+t^25+t^24+t^23+t^22+t^21+t^20+t^17+t^14+t^13+t^11+t^7+t^6+t^4+t^3+t^2+1)*x0^1*x3^2) + ((t^51+t^50+t^48+t^47+t^45+t^43+t^42+t^41+t^40+t^38+t^37+t^36+t^35+t^33+t^31+t^30+t^27+t^26+t^24+t^23+t^22+t^21+t^20+t^19+t^18+t^16+t^15+t^11+t^10+t^9+t^8+t^6+t^4+1)*x1^1*x3^2) + ((t^52+t^50+t^49+t^47+t^46+t^40+t^39+t^38+t^33+t^32+t^27+t^25+t^20+t^18+t^12+t^11+t^10+t^9+t^8+t^7+t^5+t^4+t^3+t^2+t)*x3^3) + ((t^47+t^45+t^43+t^42+t^41+t^39+t^38+t^37+t^35+t^33+t^32+t^31+t^29+t^28+t^27+t^25+t^22+t^20+t^19+t^17+t^16+t^15+t^13+t^12+t^11+t^8+t^6+t^4+t^3)*x0^1*x2^1*x4^1) + ((t^51+t^49+t^46+t^44+t^42+t^37+t^36+t^33+t^30+t^25+t^22+t^21+t^17+t^16+t^15+t^13+t^12+t^10+t^9+t^6+t^5+t^4+t^2+t+1)*x0^1*x1^1) + ((t^50+t^48+t^47+t^44+t^43+t^41+t^39+t^34+t^33+t^32+t^31+t^27+t^26+t^25+t^23+t^22+t^19+t^18+t^16+t^15+t^12+t^10+t^9+t^8+t^6+t^4+t^2+t)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x2^1*x3^3) + ((t^52+t^46+t^45+t^44+t^43+t^42+t^41+t^39+t^36+t^33+t^32+t^31+t^28+t^27+t^25+t^22+t^20+t^18+t^17+t^16+t^14+t^11+t^8+t^5+1)*x2^3*x4^1) + ((t^51+t^45+t^44+t^43+t^42+t^39+t^37+t^34+t^33+t^32+t^31+t^29+t^26+t^24+t^23+t^22+t^19+t^18+t^17+t^14+t^12+t^11+t^8+t^7+t^5+t^3)*x1^1*x2^1*x3^1*x4^1) + ((t^51+t^50+t^43+t^42+t^39+t^35+t^31+t^24+t^23+t^21+t^19+t^15+t^12+t^11+t^3+t^2+t)*x1^1*x3^2*x4^1) + ((t^52+t^51+t^50+t^47+t^46+t^42+t^40+t^39+t^36+t^33+t^32+t^31+t^30+t^27+t^25+t^23+t^22+t^16+t^15+t^14+t^13+t^12+t^11+t^8+t^7+t^5+t^4+t^3+t^2+t)*x3^3*x4^1) + ((t^52+t^50+t^49+t^47+t^40+t^39+t^34+t^32+t^31+t^29+t^27+t^25+t^23+t^20+t^19+t^17+t^16+t^14+t^13+t^11+t^9+t^8+t^6+t^4+t^3+t^2)*x0^1*x2^1*x4^2) + ((t^52+t^51+t^50+t^49+t^48+t^41+t^38+t^36+t^33+t^30+t^29+t^28+t^27+t^23+t^18+t^16+t^15+t^14+t^12+t^10+t^8+t^7+t^6+t^3)*x2^2*x4^2) + ((t^51+t^50+t^45+t^43+t^42+t^41+t^40+t^37+t^36+t^33+t^29+t^28+t^25+t^24+t^23+t^22+t^19+t^17+t^14+t^12+t^5)*x0^1*x4^3) + ((t^48+t^46+t^45+t^42+t^39+t^38+t^35+t^34+t^33+t^30+t^29+t^28+t^25+t^24+t^19+t^18+t^15+t^14+t^12+t^11+t^9+t^8+t^6+t^4+t^2+t)*x0^1*x1^1*x2^1) + ((t^51+t^49+t^48+t^46+t^45+t^42+t^41+t^31+t^29+t^28+t^26+t^25+t^24+t^23+t^22+t^21+t^18+t^16+t^15+t^14+t^13+t^11+t^8+t^7+t^6+t^4+t)*x1^2*x3^1) + ((t^52+t^46+t^45+t^39+t^38+t^29+t^27+t^26+t^25+t^22+t^21+t^19+t^18+t^17+t^16+t^15+t^14+t^11+t^10+t^9+t^8+t^6+t^4+t^3+t^2+t+1)*x1^1*x3^2) + ((t^48+t^47+t^46+t^45+t^44+t^43+t^42+t^41+t^40+t^39+t^36+t^35+t^33+t^32+t^30+t^29+t^26+t^23+t^22+t^21+t^19+t^17+t^13+t^12+t^7+t^5+t^4+t^3+t^2)*x0^1*x2^1*x4^1) + ((t^52+t^51+t^50+t^48+t^47+t^45+t^41+t^40+t^39+t^37+t^36+t^35+t^34+t^32+t^30+t^29+t^28+t^25+t^24+t^23+t^22+t^18+t^17+t^16+t^15+t^12+t^9+t^7+t^5+t^4+t^3+t^2+t)*x0^1*x1^1) + ((t^52+t^51+t^50+t^49+t^48+t^47+t^42+t^41+t^36+t^31+t^30+t^25+t^24+t^23+t^21+t^20+t^19+t^17+t^16+t^13+t^12+t^10+t^8+t^6+t^4+t^3+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x0^1*x1^2*x4^1) + ((t^52+t^49+t^48+t^47+t^43+t^42+t^39+t^38+t^36+t^33+t^32+t^31+t^30+t^28+t^27+t^24+t^23+t^21+t^20+t^18+t^17+t^15+t^13+t^12+t^10+t^7+t^4+t^2+t+1)*x0^1*x2^2*x4^1) + ((t^50+t^49+t^46+t^45+t^43+t^42+t^41+t^39+t^38+t^37+t^36+t^35+t^34+t^33+t^32+t^31+t^28+t^26+t^25+t^24+t^23+t^22+t^18+t^16+t^14+t^12+t^11+t^10+t^9+t^8+t^7+t^3+t^2+t)*x2^3*x4^1) + ((t^50+t^49+t^48+t^46+t^43+t^39+t^38+t^37+t^36+t^33+t^32+t^30+t^29+t^28+t^26+t^23+t^18+t^17+t^15+t^13+t^12+t^10+t^9+t^8+t^3+t^2+1)*x0^1*x1^1*x3^1*x4^1) + ((t^52+t^51+t^50+t^48+t^41+t^40+t^39+t^34+t^33+t^32+t^31+t^30+t^29+t^28+t^27+t^26+t^25+t^19+t^15+t^14+t^13+t^10+t^6+t^4+t^3+t^2)*x2^2*x3^1*x4^1) + ((t^51+t^50+t^48+t^47+t^43+t^42+t^41+t^37+t^36+t^32+t^30+t^28+t^24+t^23+t^22+t^21+t^18+t^15+t^14+t^13+t^11+t^6+t^5+1)*x1^1*x3^2*x4^1) + ((t^52+t^50+t^48+t^47+t^46+t^44+t^43+t^41+t^40+t^39+t^38+t^37+t^36+t^35+t^33+t^31+t^26+t^25+t^23+t^22+t^20+t^17+t^15+t^14+t^12+t^10+t^9+t^5+t^4+t^3+t^2+t+1)*x3^3*x4^1) + ((t^51+t^50+t^45+t^42+t^39+t^37+t^34+t^31+t^30+t^29+t^28+t^27+t^26+t^25+t^24+t^23+t^22+t^19+t^12+t^10+t^7+t^6+t^4+t^3+t^2+t+1)*x0^2*x4^2) + ((t^51+t^50+t^48+t^44+t^41+t^39+t^34+t^33+t^32+t^30+t^29+t^25+t^24+t^23+t^22+t^21+t^20+t^15+t^13+t^11+t^10+t^8+t^7+t^4+t+1)*x0^1*x2^1*x4^2) + ((t^46+t^44+t^42+t^41+t^38+t^37+t^36+t^34+t^29+t^28+t^24+t^23+t^19+t^17+t^16+t^15+t^13+t^12+t^10+t^8+t^7+t^5+t^4+t)*x2^2*x4^2) + ((t^52+t^50+t^48+t^47+t^46+t^38+t^34+t^24+t^22+t^21+t^19+t^17+t^15+t^14+t^13+t^12+t^10+t^7+t^6+t^3+t^2+t+1)*x0^1*x4^3) + ((t^52+t^51+t^50+t^49+t^48+t^45+t^44+t^42+t^41+t^40+t^38+t^37+t^34+t^33+t^32+t^31+t^30+t^27+t^23+t^18+t^17+t^16+t^14+t^13+t^12+t^11+t^10+t^9+t^8+t^5+t^4+t^2+t+1)*x0^1*x1^1*x2^1) + ((t^52+t^50+t^48+t^47+t^46+t^45+t^44+t^40+t^39+t^38+t^36+t^35+t^34+t^33+t^32+t^31+t^30+t^25+t^24+t^23+t^19+t^17+t^16+t^15+t^13+t^11+t^10+t^8+t^6+t^5+t^4+t^3+t^2+t)*x1^2*x3^1) + ((t^52+t^49+t^47+t^46+t^45+t^43+t^40+t^37+t^36+t^32+t^27+t^26+t^24+t^20+t^18+t^14+t^13+t^12+t^11+t^10+t^7+t^6+t^4+t^2+t)*x0^1*x3^2) + ((t^50+t^48+t^38+t^35+t^31+t^27+t^25+t^23+t^19+t^14+t^13+t^12+t^9+t^8+t^7+t^5+t^4+t^2+1)*x1^1*x3^2) + ((t^52+t^50+t^48+t^47+t^46+t^42+t^39+t^37+t^34+t^30+t^29+t^27+t^25+t^23+t^21+t^20+t^19+t^18+t^17+t^15+t^12+t^11+t^10+t^9+t^6+t^2+t+1)*x3^3) + ((t^52+t^50+t^49+t^47+t^46+t^40+t^39+t^38+t^33+t^32+t^27+t^25+t^20+t^18+t^12+t^11+t^10+t^9+t^8+t^7+t^5+t^4+t^3+t^2+t)*x0^1*x1^1*x4^1) + ((t^50+t^49+t^47+t^44+t^43+t^41+t^38+t^36+t^33+t^32+t^31+t^28+t^27+t^26+t^24+t^23+t^20+t^17+t^15+t^14+t^12+t^10+t^9+t^4+t^2)*x0^1*x2^1*x4^1) + ((t^51+t^47+t^46+t^45+t^44+t^42+t^40+t^39+t^38+t^37+t^36+t^35+t^33+t^30+t^29+t^26+t^25+t^23+t^22+t^21+t^20+t^19+t^18+t^13+t^10+t^9+t^8+t^5+t^2+t+1)*x2^2*x4^1) + ((t^51+t^47+t^46+t^44+t^41+t^37+t^32+t^31+t^30+t^28+t^27+t^25+t^22+t^21+t^20+t^18+t^14+t^13+t^6+t^5+t^4+t^3+t^2+t+1)*x1^1*x3^1*x4^1) + ((t^52+t^49+t^48+t^47+t^45+t^43+t^42+t^39+t^38+t^32+t^31+t^28+t^27+t^22+t^21+t^20+t^18+t^17+t^16+t^15+t^9+t^7+t^6+t^5+t^2+t)*x0^1*x4^2) + ((t^52+t^46+t^45+t^44+t^42+t^40+t^36+t^33+t^31+t^24+t^22+t^17+t^14+t^12+t^9+t^6)*x0^1*x1^1) + ((t^52+t^49+t^45+t^38+t^37+t^33+t^31+t^30+t^22+t^21+t^20+t^19+t^16+t^14+t^13+t^12+t^11+t^10+t^4+t^3+t+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x1^3*x4^1) + ((t^52+t^51+t^50+t^47+t^45+t^42+t^41+t^40+t^39+t^38+t^37+t^36+t^35+t^34+t^29+t^28+t^27+t^26+t^25+t^24+t^22+t^18+t^17+t^15+t^13+t^12+t^11+t^9+t^7+t^4+t^3+t)*x0^1*x2^2*x4^1) + ((t^52+t^51+t^50+t^46+t^44+t^43+t^42+t^38+t^35+t^34+t^33+t^32+t^31+t^30+t^29+t^28+t^24+t^23+t^20+t^19+t^17+t^16+t^12+t^5+t^4+t^3+t)*x2^3*x4^1) + ((t^46+t^44+t^43+t^41+t^40+t^39+t^36+t^33+t^29+t^28+t^27+t^24+t^22+t^21+t^20+t^18+t^16+t^15+t^14+t^12+t^9+t^3+t^2+1)*x1^2*x3^1*x4^1) + ((t^50+t^49+t^48+t^46+t^45+t^44+t^42+t^39+t^38+t^36+t^35+t^32+t^27+t^26+t^25+t^23+t^22+t^20+t^19+t^18+t^17+t^15+t^14+t^11+t^10+t^6+t^5+t^4+t^3)*x1^1*x3^2*x4^1) + ((t^50+t^49+t^43+t^42+t^40+t^39+t^36+t^35+t^33+t^32+t^27+t^26+t^24+t^23+t^22+t^20+t^17+t^15+t^12+t^9+t^7+1)*x3^3*x4^1) + ((t^50+t^49+t^48+t^47+t^45+t^43+t^42+t^36+t^35+t^27+t^26+t^24+t^23+t^21+t^17+t^14+t^13+t^12+t^9+t^8+t^6+t^5+t^4+t^2+t)*x0^1*x2^1*x4^2) + ((t^47+t^46+t^42+t^41+t^39+t^38+t^37+t^35+t^34+t^32+t^31+t^30+t^24+t^23+t^22+t^21+t^20+t^19+t^18+t^16+t^15+t^14+t^13+t^10+t^5+t^3+t+1)*x2^2*x4^2) + ((t^52+t^51+t^49+t^48+t^46+t^42+t^40+t^39+t^38+t^36+t^34+t^33+t^32+t^28+t^26+t^22+t^19+t^17+t^16+t^14+t^13+t^12+t^5+t^3+t^2)*x0^1*x4^3) + ((t^51+t^49+t^48+t^47+t^46+t^45+t^44+t^43+t^42+t^40+t^39+t^35+t^34+t^31+t^28+t^25+t^24+t^22+t^19+t^18+t^17+t^16+t^12+t^11+t^10+t^9+t^7+t^5+t^4+t^3+t^2+t+1)*x4^4) + ((t^47+t^46+t^43+t^42+t^40+t^39+t^37+t^36+t^32+t^31+t^30+t^29+t^23+t^20+t^18+t^17+t^16+t^14+t^11+t^7+t^5+t^3+t^2)*x0^1*x1^1*x2^1) + ((t^51+t^49+t^48+t^46+t^45+t^42+t^41+t^39+t^38+t^37+t^32+t^31+t^30+t^29+t^23+t^20+t^19+t^14+t^13+t^12+t^10+t^9+t^8+t^7+t^6+1)*x1^2*x3^1) + ((t^51+t^49+t^48+t^46+t^43+t^42+t^36+t^34+t^33+t^29+t^28+t^26+t^25+t^22+t^21+t^20+t^19+t^15+t^12+t^11+t^10+t^9+t^8+t^7+t^6+t^5+t^3+t^2+t)*x1^1*x3^2) + ((t^52+t^51+t^50+t^48+t^47+t^46+t^44+t^40+t^39+t^38+t^37+t^36+t^35+t^34+t^33+t^32+t^31+t^30+t^26+t^25+t^23+t^21+t^18+t^15+t^14+t^13+t^12+t^11+t^10+t^8+t^7+t^6+t^3+t)*x0^1*x2^1*x4^1) + ((t^51+t^48+t^47+t^44+t^41+t^38+t^36+t^35+t^34+t^32+t^30+t^28+t^26+t^25+t^22+t^21+t^20+t^19+t^14+t^12+t^11+t^9+t^8+t^7+t^6+t^5+t^4+t+1)*x0^1*x1^1) + ((t^50+t^48+t^46+t^45+t^44+t^41+t^40+t^35+t^34+t^32+t^31+t^29+t^27+t^24+t^22+t^15+t^14+t^13+t^11+t^9+t^8+t^7+1)*x1^1*x3^1) + ((t^49+t^47+t^44+t^43+t^42+t^38+t^35+t^32+t^29+t^27+t^21+t^19+t^17+t^16+t^14+t^11+t^9+t^6+t^4+t^3+1)*x3^2)");
    groebnerBasis.push_back("((+1)*x0^2*x2^1*x4^1) + ((t^51+t^45+t^44+t^41+t^38+t^37+t^36+t^33+t^32+t^30+t^29+t^28+t^27+t^26+t^25+t^21+t^20+t^19+t^18+t^16+t^12+t^11+t^8+t^6+t^5+t^4+t^3)*x2^3*x4^1) + ((t^51+t^48+t^46+t^44+t^37+t^36+t^35+t^28+t^27+t^26+t^25+t^22+t^20+t^18+t^16+t^12+t^11+t^10+t^7+t^6+t^2+t+1)*x0^1*x2^1*x3^1*x4^1) + ((t^52+t^51+t^49+t^48+t^47+t^46+t^44+t^40+t^30+t^22+t^21+t^19+t^18+t^17+t^14+t^13+t^12+t^11+t^10+t^6+t^5+t^3+t^2)*x0^1*x2^1*x4^2)");
    groebnerBasis.push_back("((+1)*x0^1*x1^1*x2^1*x4^1) + ((t^50+t^49+t^48+t^46+t^41+t^40+t^39+t^38+t^37+t^36+t^34+t^33+t^32+t^31+t^29+t^28+t^27+t^26+t^22+t^21+t^19+t^18+t^16+t^15+t^13+t^11+t^8+t^7+t^4+t^3+t^2+1)*x2^3*x4^1) + ((t^52+t^51+t^50+t^49+t^44+t^42+t^41+t^40+t^38+t^35+t^34+t^32+t^31+t^28+t^27+t^23+t^22+t^21+t^20+t^19+t^17+t^16+t^15+t^12+t^11+t^10+t^9+t^7+t^6+t^5+t^4+t)*x1^1*x3^2*x4^1) + ((t^52+t^51+t^50+t^49+t^46+t^45+t^40+t^38+t^37+t^32+t^31+t^28+t^27+t^25+t^22+t^21+t^18+t^16+t^15+t^11+t^9+t^7+t^5+t^4+t^3+t^2+1)*x3^3*x4^1) + ((t^52+t^46+t^45+t^43+t^42+t^41+t^40+t^37+t^36+t^34+t^30+t^28+t^26+t^25+t^22+t^19+t^17+t^16+t^15+t^14+t^12+t^11+t^10+t^9+t^6+t^5+t+1)*x0^1*x2^1*x4^2) + ((t^52+t^50+t^49+t^45+t^40+t^37+t^36+t^35+t^33+t^32+t^30+t^29+t^27+t^26+t^25+t^24+t^20+t^18+t^17+t^16+t^13+t^12+t^10+t^9+t^6+t^5+t^4+t^3+t^2+t)*x2^2*x4^2) + ((t^52+t^51+t^50+t^49+t^47+t^43+t^40+t^32+t^31+t^29+t^27+t^22+t^17+t^16+t^14+t^12+t^11+t^10+t^8+t^4+1)*x0^1*x4^3) + ((t^51+t^50+t^49+t^46+t^45+t^42+t^41+t^40+t^38+t^36+t^34+t^29+t^27+t^24+t^23+t^19+t^15+t^14+t^12+t^10+t^9+t^8+t^7+t^5+t^3+t+1)*x0^1*x1^1*x2^1) + ((t^52+t^51+t^48+t^47+t^45+t^44+t^43+t^38+t^34+t^33+t^28+t^27+t^26+t^25+t^24+t^22+t^19+t^18+t^17+t^15+t^12+t^10+t^8+t^7+t^5+t^4+t^2+t)*x1^2*x3^1) + ((t^49+t^47+t^45+t^42+t^40+t^38+t^34+t^32+t^30+t^29+t^28+t^27+t^26+t^25+t^24+t^23+t^22+t^18+t^16+t^12+t^11+t^9+t^6+t^4+t^2+t)*x1^1*x3^2) + ((t^51+t^50+t^49+t^47+t^45+t^44+t^43+t^42+t^41+t^40+t^38+t^35+t^33+t^32+t^31+t^29+t^28+t^23+t^22+t^21+t^20+t^19+t^16+t^14+t^12+t^11+t^6+t^4+t^3+t)*x0^1*x2^1*x4^1) + ((t^52+t^50+t^49+t^46+t^45+t^43+t^40+t^38+t^37+t^36+t^31+t^29+t^28+t^27+t^26+t^25+t^24+t^22+t^21+t^20+t^18+t^17+t^15+t^14+t^11+t^8)*x0^1*x1^1) + ((t^52+t^51+t^50+t^48+t^47+t^45+t^44+t^42+t^40+t^38+t^37+t^36+t^34+t^32+t^29+t^27+t^24+t^23+t^22+t^20+t^18+t^14+t^13+t^11+t^9+t^7+t^5+t^3+t^2+t+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x1^1*x2^2*x4^1) + ((t^48+t^47+t^45+t^42+t^41+t^39+t^38+t^37+t^34+t^33+t^32+t^30+t^28+t^27+t^24+t^23+t^22+t^21+t^18+t^16+t^15+t^13+t^12+t^11+t^8+t^6+t^5+t)*x2^3*x4^1) + ((t^52+t^51+t^50+t^49+t^48+t^47+t^44+t^39+t^37+t^36+t^35+t^34+t^33+t^32+t^27+t^25+t^24+t^22+t^18+t^17+t^16+t^15+t^14+t^13+t^12+t^11+t^10+t^9+t^8+t^7+t^6+t^4+t^3+1)*x1^1*x3^2*x4^1) + ((t^52+t^49+t^48+t^47+t^44+t^42+t^41+t^35+t^34+t^33+t^26+t^22+t^19+t^16+t^14+t^13+t^10+t^7+t^6+t^4+t^3+t^2+t+1)*x3^3*x4^1) + ((t^52+t^51+t^50+t^49+t^48+t^47+t^46+t^44+t^39+t^38+t^37+t^35+t^32+t^31+t^29+t^28+t^27+t^24+t^21+t^19+t^18+t^17+t^16+t^13+t^12+t^11+t^10+t^5+t^3+t^2+t)*x0^1*x2^1*x4^2) + ((t^52+t^50+t^48+t^41+t^40+t^38+t^36+t^33+t^31+t^27+t^26+t^25+t^21+t^20+t^19+t^18+t^16+t^14+t^13+t^12+t^11+t^10+t^9+t^7+t^6+t^5+t^4+t^3+t^2+1)*x2^2*x4^2) + ((t^52+t^50+t^48+t^44+t^42+t^41+t^39+t^36+t^34+t^32+t^31+t^30+t^28+t^27+t^26+t^25+t^19+t^18+t^17+t^13+t^12+t^11+t^8+t^7+t^6+t^5+t^4+t^3+t^2)*x0^1*x4^3) + ((t^50+t^48+t^46+t^42+t^41+t^40+t^38+t^33+t^31+t^30+t^29+t^28+t^26+t^25+t^21+t^20+t^19+t^17+t^16+t^14+t^13+t^5+t^2)*x4^4) + ((t^52+t^47+t^46+t^45+t^44+t^40+t^39+t^36+t^35+t^33+t^32+t^31+t^30+t^28+t^27+t^25+t^23+t^18+t^15+t^14+t^11+t^10+t^9+t^7+t^6+t^3+t+1)*x0^1*x1^1*x2^1) + ((t^51+t^48+t^47+t^45+t^42+t^41+t^40+t^33+t^32+t^31+t^27+t^23+t^21+t^20+t^18+t^13+t^9+t^8+t^7+t^5+t+1)*x1^2*x3^1) + ((t^51+t^49+t^48+t^46+t^45+t^43+t^41+t^37+t^35+t^33+t^32+t^29+t^27+t^24+t^23+t^22+t^21+t^20+t^18+t^16+t^15+t^13+t^12+t^10+t^7+t^2+t)*x1^1*x3^2) + ((t^50+t^47+t^46+t^44+t^42+t^41+t^40+t^39+t^38+t^37+t^33+t^27+t^26+t^23+t^21+t^19+t^18+t^17+t^16+t^15+t^12+t^8+t^6+t^4+t^3+t^2+t)*x0^1*x2^1*x4^1) + ((t^50+t^49+t^46+t^41+t^38+t^36+t^33+t^30+t^29+t^28+t^26+t^23+t^21+t^18+t^17+t^15+t^14+t^12+t^11+t^10+t^9+t^8+t^7+t^4+t^3+t^2+t+1)*x0^1*x1^1) + ((t^52+t^46+t^45+t^43+t^42+t^37+t^35+t^34+t^33+t^32+t^30+t^29+t^28+t^27+t^23+t^22+t^21+t^20+t^19+t^18+t^17+t^16+t^12+t^11+t^10+t^8+t^4+1)*x1^1*x3^1)");
    groebnerBasis.push_back("((+1)*x0^1*x3^2*x4^1) + ((t^50+t^48+t^47+t^46+t^44+t^40+t^39+t^37+t^34+t^33+t^30+t^27+t^26+t^25+t^21+t^20+t^18+t^16+t^14+t^11+t^9+t^7+t^5+t^3)*x2^2*x4^2) + ((t^52+t^46+t^44+t^43+t^41+t^38+t^37+t^33+t^31+t^28+t^27+t^25+t^23+t^21+t^20+t^18+t^17+t^14+t^13+t^12+t^9+t^6+t^2+t+1)*x0^1*x4^3)");
    groebnerBasis.push_back("((+1)*x0^1*x3^1*x4^2) + ((t^49+t^47+t^46+t^45+t^41+t^37+t^36+t^32+t^28+t^27+t^25+t^24+t^23+t^22+t^19+t^16+t^15+t^14+t^11+t^9+t^8+t^7+t^5+t^4)*x0^1*x3^2) + ((t^52+t^51+t^50+t^49+t^47+t^45+t^40+t^37+t^34+t^33+t^30+t^29+t^27+t^26+t^24+t^23+t^21+t^20+t^18+t^16+t^13+t^9+t^5+t^2+t+1)*x2^2*x4^1) + ((t^52+t^46+t^45+t^44+t^41+t^38+t^37+t^36+t^33+t^29+t^28+t^25+t^23+t^20+t^19+t^17+t^16+t^13+t^10+t^9+t^3+t+1)*x0^1*x4^2)");

    /* 64 bits */
    bool test = true;
    size_t i = 0;
    while(i < basis.size() && test == true )
    {
        test = test && (groebnerBasis[i].compare(basis[i])==0);
        i++;
    }
    if(test==true)
    {
        cout << "Test random gf2 extension 64 bits pass" << endl;
        return 0;
    }
    else
    {
        cout << "Test random gf2 extension 64 bits failed" << endl;
        return -1;
    }
}


